module.exports = {
  env: {
    browser: true,
    es6: true,
    node: true,
  },
  ignorePatterns: ['node_modules/**/*'],
  parser: '@typescript-eslint/parser',
  plugins: [
    'eslint-plugin-import',
    'eslint-plugin-jsdoc',
    '@angular-eslint/eslint-plugin',
    '@typescript-eslint',
    '@typescript-eslint/eslint-plugin-tslint',
  ],
  root: true,
  parserOptions: {
    ecmaVersion: 2020,
  },
  rules: {
    '@angular-eslint/component-class-suffix': 'error',
    '@angular-eslint/directive-class-suffix': 'error',
    '@angular-eslint/no-input-rename': 'error',
    '@angular-eslint/no-output-on-prefix': 'off',
    '@angular-eslint/no-output-rename': 'error',
    '@angular-eslint/use-pipe-transform-interface': 'error',
    '@typescript-eslint/consistent-type-definitions': 'error',
    '@typescript-eslint/dot-notation': 'off',
    '@typescript-eslint/explicit-member-accessibility': [
      'error', {
        'accessibility': 'no-public',
      }],
    '@typescript-eslint/member-delimiter-style': [
      'error',
      {
        multiline: {
          delimiter: 'semi',
          requireLast: true,
        },
        singleline: {
          delimiter: 'semi',
          requireLast: false,
        },
      },
    ],
    '@typescript-eslint/member-ordering': [
      'error',
      {
        default: [
          'signature',
          /**
           * static fields
           * [sort: public -> protected -> private]
           **/
          'public-static-field',
          'protected-static-field',
          'private-static-field',

          /**
           * abstract fields
           * [sort: public -> protected -> private]
           **/
          'public-abstract-field',
          'protected-abstract-field',

          /**
           * instance fields
           * [sort: private -> protected -> public]
           * [sort: decorated -> non-decorated]
           **/
          'private-decorated-field',
          'private-instance-field',
          'protected-decorated-field',
          'protected-instance-field',
          'public-decorated-field',
          'public-instance-field',

          /**
           * constructors
           * [sort: public -> protected -> private]
           **/
          'public-constructor',
          'protected-constructor',
          'private-constructor',

          /**
           * @note(splincode):
           * We have deliberately disabled this rule
           * because it can break a lot of the codebase.
           *
           * static accessors
           * [sort: public -> protected -> private]
           **/
          /*
           * 'public-static-get',
           * 'public-static-set',
           * 'protected-static-get',
           * 'protected-static-set',
           * 'private-static-get',
           * 'private-static-set',
           */

          /**
           * @note(splincode):
           * We have deliberately disabled this rule
           * because it can break a lot of the codebase.
           *
           * instance accessors
           * [sort: public -> protected -> private]
           * [sort: decorated -> non-decorated]
           **/

          'public-decorated-get',
          'public-instance-get',
          'public-decorated-set',
          'public-instance-set',
          'protected-decorated-get',
          'protected-instance-get',
          'protected-decorated-set',
          'protected-instance-set',
          'private-decorated-get',
          'private-instance-get',
          'private-decorated-set',
          'private-instance-set',


          /**
           * static methods
           * [sort: public -> protected -> private]
           **/
          'public-static-method',
          'protected-static-method',
          'private-static-method',

          /**
           * abstract
           * [sort: public -> private -> protected]
           **/
          'public-abstract-get',
          'public-abstract-set',
          'protected-abstract-get',
          'protected-abstract-set',
          'public-abstract-method',
          'protected-abstract-method',

          /**
           * methods
           * [sort: public -> protected -> private]
           * [sort: decorated -> non-decorated]
           **/
          'public-decorated-method',
          'public-instance-method',
          'protected-decorated-method',
          'protected-instance-method',
          'private-decorated-method',
          'private-instance-method',
        ],
      }],
    '@typescript-eslint/naming-convention': [
      'error',
      {
        selector: 'variable',
        format: ['camelCase', 'UPPER_CASE', 'PascalCase'],
        leadingUnderscore: 'forbid',
        trailingUnderscore: 'forbid',
      },
    ],
    '@typescript-eslint/no-empty-function': 'off',
    '@typescript-eslint/no-empty-interface': 'error',
    '@typescript-eslint/no-inferrable-types': [
      'error',
      {
        ignoreParameters: true,
      },
    ],
    '@typescript-eslint/no-misused-new': 'error',
    '@typescript-eslint/no-non-null-assertion': 'error',
    '@typescript-eslint/no-shadow': [
      'error',
      {
        hoist: 'all',
      },
    ],
    '@typescript-eslint/no-unused-expressions': 'error',
    '@typescript-eslint/prefer-function-type': 'error',
    '@typescript-eslint/quotes': ['error', 'single'],
    '@typescript-eslint/semi': ['error', 'always'],
    '@typescript-eslint/type-annotation-spacing': 'error',
    '@typescript-eslint/unified-signatures': 'error',
    'arrow-body-style': 'error',
    'brace-style': ['error', '1tbs'],
    'constructor-super': 'error',
    curly: 'error',
    'dot-notation': 'off',
    'eol-last': 'error',
    eqeqeq: ['error', 'smart'],
    'guard-for-in': 'error',
    'id-denylist': 'off',
    'id-match': 'off',
    'import/no-deprecated': 'warn',
    indent: 'off',
    'jsdoc/no-types': 'error',
    'max-len': [
      'error',
      {
        code: 140,
      },
    ],
    'no-bitwise': 'error',
    'no-caller': 'error',
    'no-console': [
      'error',
      {
        allow: [
          'warn',
          'dir',
          'timeLog',
          'assert',
          'clear',
          'count',
          'countReset',
          'group',
          'groupEnd',
          'table',
          'dirxml',
          'error',
          'groupCollapsed',
          'Console',
          'profile',
          'profileEnd',
          'timeStamp',
          'context',
        ],
      },
    ],
    'no-debugger': 'error',
    'no-empty': 'off',
    'no-empty-function': 'off',
    'no-eval': 'error',
    'no-fallthrough': 'error',
    'no-new-wrappers': 'error',
    'no-restricted-imports': ['error', 'rxjs/Rx'],
    'no-shadow': 'off',
    'no-throw-literal': 'error',
    'no-trailing-spaces': 'off',
    'no-undef-init': 'error',
    'no-underscore-dangle': 'off',
    'no-unused-expressions': 'off',
    'no-unused-labels': 'error',
    'no-var': 'error',
    'prefer-const': 'error',
    quotes: 'off',
    radix: 'error',
    semi: 'off',
    'spaced-comment': [
      'error',
      'always',
      {
        markers: ['/'],
      },
    ],
  },
};
