import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from './auth/guards';
import { ProfileComponent } from './components/profile/profile.component';
import { LoginComponent } from './components/login/login.component';
import { LogoutComponent } from './components/logout/logout.component';
import { RegistrationComponent } from './components/registration/registration.component';
import { DashboardComponent } from './containers/dashboard/dashboard.component';
import { ResetPasswordComponent } from '@core-project/components/reset-password/reset-password.component';
import { LandingComponent } from '@core-project/components/landing/landing.component';
import { DATA_APP_TITLE_KEY } from 'data-access/store';
import { FEATURE_ISSUE_ROUTE, FEATURE_WORK_ROUTE, OutletsEnum, RIGHT_SIDEBAR_ROUTE } from 'utils/core-project-routing-utils';
import { registrationGuard } from './components/registration/registration.guard';

const routes: Routes = [
  { path: '', redirectTo: 'profile', pathMatch: 'full' },
  { path: 'dashboard', component: DashboardComponent, canActivate: [AuthGuard], data: { [DATA_APP_TITLE_KEY]: 'DASHBOARD.HEADER' } },
  { path: 'landing', component: LandingComponent },
  { path: 'profile', component: ProfileComponent, canActivate: [AuthGuard], data: { [DATA_APP_TITLE_KEY]: 'PROFILE.TITLE' } },
  { path: 'login', component: LoginComponent },
  { path: 'logout', component: LogoutComponent },
  { path: 'registration', component: RegistrationComponent, canActivate: [registrationGuard] },
  { path: 'password-reset', component: ResetPasswordComponent },
  { path: 'devtools', canActivate: [AuthGuard], loadChildren: () => import('features/devtools').then((m) => m.DevtoolsModule) },
  {
    path: RIGHT_SIDEBAR_ROUTE,
    outlet: OutletsEnum.RIGHT_SIDEBAR,
    children: [
      // feature modules
      { path: FEATURE_ISSUE_ROUTE, loadChildren: () => import('features/issues').then((m) => m.IssuesModule) },
      { path: FEATURE_WORK_ROUTE, loadChildren: () => import('features/works').then((m) => m.WorksModule) },
    ],
  },
];

@NgModule({
  exports: [RouterModule],
  imports: [RouterModule.forRoot(routes)],
})
export class AppRoutingModule {}
