// core
import {HTTP_INTERCEPTORS, HttpClient, HttpClientModule} from '@angular/common/http';
import {APP_INITIALIZER, LOCALE_ID, NgModule} from '@angular/core';
import {BrowserModule} from '@angular/platform-browser';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import localeRu from '@angular/common/locales/ru';
import {TranslateHttpLoader} from '@ngx-translate/http-loader';
import {TranslateLoader, TranslateModule, TranslateService} from '@ngx-translate/core';
import {forkJoin, of} from 'rxjs';
import {filter, first} from 'rxjs/operators';
import {registerLocaleData} from '@angular/common';
import {NgxsModule, Store} from '@ngxs/store';
import {NgxsRouterPluginModule} from '@ngxs/router-plugin';
import {NgxsReduxDevtoolsPluginModule} from '@ngxs/devtools-plugin';

// services
import {AuthenticationService} from './services/authentication.service';
import {SidemenuService} from './services/sidemenu.service';
import {LoadingService} from './services/loading.service';
import {UserSettingsService} from '@core-project/services/user-settings.service';
import {BreadcrumbService} from '@core-project/services/breadcrumb.service';

// components
import {AppComponent} from './app.component';
import {LoginComponent} from './components/login/login.component';
import {ProfileComponent} from './components/profile/profile.component';
import {SidemenuComponent} from './components/sidemenu/sidemenu.component';
import {SubmenuComponent} from './components/submenu/submenu.component';
import {LogoutComponent} from './components/logout/logout.component';
import {RegistrationComponent} from './components/registration/registration.component';
import {AppTopBarComponent} from '@core-project/components/topbar/app.topbar.component';
import {AppBreadcrumbComponent} from '@core-project/components/breadcrumb/app.breadcrumb.component';
import {ResetPasswordComponent} from './components/reset-password/reset-password.component';
import {LandingComponent} from './components/landing/landing.component';
import {SidemenuItemComponent} from './components/sidemenu-item/sidemenu-item.component';

// modules
import {AppRoutingModule} from './app-routing.module';
import {ProfileModule} from './components/profile/profile.module';
import {DashboardModule} from './containers/dashboard/dashboard.module';
import {ProjectsModule} from './containers/projects/projects.module';
import {DepartmentModule} from './containers/department/department.module';
import {ProgramsModule} from './containers/programs/programs.module';
import {OrganizationsModule} from './containers/organizations/organizations.module';
import {ProjectDocumentsModule} from './containers/project/project-documents/project-documents.module';
import {NewsFormModule} from './modules/forms/news-form/news-form.module';
import {KnowledgebaseModule} from './containers/knowledgebase/knowledgebase.module';
import {ProcurementsModule} from './containers/procurements/procurements.module';
import {RisksModule} from './containers/risks/risks.module';
import {ProjectModule} from './containers/project/project.module';
import {NewsfeedModule} from './containers/newsfeed/newsfeed.module';
import {DocumentsModule} from '@core-project/containers/documents/documents.module';
import {UserWidgetsModule} from '@core-project/modules/user-widgets/user-widgets.module';
import {DirectivesModule} from '@core-project/directives/directives.module';
import {SharedComponentsModule} from '@core-project/modules/shared-components/shared-components.module';

// primeng
import {ConfirmationService, MessageService} from 'primeng/api';
import {DialogService, DynamicDialogModule} from 'primeng/dynamicdialog';
import {SlideMenuModule} from 'primeng/slidemenu';
import {MenuModule} from 'primeng/menu';
import {ScrollPanelModule} from 'primeng/scrollpanel';
import {InputTextModule} from 'primeng/inputtext';
import {CalendarModule} from 'primeng/calendar';
import {MessagesModule} from 'primeng/messages';
import {SpinnerModule} from 'primeng/spinner';
import {CheckboxModule} from 'primeng/checkbox';
import {ProgressSpinnerModule} from 'primeng/progressspinner';
import {MessageModule} from 'primeng/message';
import {DialogModule} from 'primeng/dialog';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {StepsModule} from 'primeng/steps';
import {FieldsetModule} from 'primeng/fieldset';
import {ButtonModule} from 'primeng/button';
import {OverlayPanelModule} from 'primeng/overlaypanel';
import {TreeModule} from 'primeng/tree';
import {RippleModule} from 'primeng/ripple';
import {FileUploadModule} from 'primeng/fileupload';
import {ToastModule} from 'primeng/toast';
import {RatingModule} from 'primeng/rating';
import {CardModule} from 'primeng/card';
import {ImageModule} from 'primeng/image';
import {AvatarModule} from 'primeng/avatar';
import {AutoCompleteModule} from 'primeng/autocomplete';
import {MenubarModule} from 'primeng/menubar';
import {BreadcrumbModule} from 'primeng/breadcrumb';
import {SidebarModule} from 'primeng/sidebar';
import {DividerModule} from 'primeng/divider';
import {PasswordModule} from 'primeng/password';
import {TabMenuModule} from 'primeng/tabmenu';
import {BadgeModule} from 'primeng/badge';

// other
import {ErrorInterceptor} from './auth/helpers';
import {environment} from '../environments/environment';
import {
  GlobalRulesSelectors,
  GlobalRulesState,
  GlobalState,
  IssuesState,
  NotificationsState,
  ProjectRulesState,
  RightSidebarState,
  WebsocketsState,
  WorksState,
} from 'data-access/store';
import {ApiModule, UsersService} from 'data-access/sdk/api2';
import {WebsocketsApiService} from 'data-access/sdk/internal-api';
import {NotificationsModule} from 'features/notifications';
import {RightSidebarModule} from 'features/right-sidebar';
import {CustomIconsComponent} from 'ui/custom-icons';


registerLocaleData(localeRu);

registerLocaleData(localeRu);

const primengModules = [
  MenuModule,
  ScrollPanelModule,
  InputTextModule,
  CalendarModule,
  MessagesModule,
  SlideMenuModule,
  SpinnerModule,
  CheckboxModule,
  ProgressSpinnerModule,
  MessageModule,
  DialogModule,
  ConfirmDialogModule,
  StepsModule,
  FieldsetModule,
  ButtonModule,
  OverlayPanelModule,
  TreeModule,
  RippleModule,
  FileUploadModule,
  ToastModule,
  RatingModule,
  CardModule,
  ImageModule,
  AvatarModule,
  AutoCompleteModule,
  MenubarModule,
  BreadcrumbModule,
  SidebarModule,
  DividerModule,
  PasswordModule,
];

export function httpLoaderFactory(http: HttpClient) {
  return new TranslateHttpLoader(http, '/assets/i18n/', '.json');
}

export function appInitializerFactory(
  translate: TranslateService,
  auth: AuthenticationService,
  userSettings: UserSettingsService,
  store: Store,
) {
  return () => {
    translate.setDefaultLang('ru');
    translate.addLangs(['ru', 'en']);

    const lang = localStorage.getItem('lang') || translate.defaultLang;
    const token = localStorage.getItem('token');

    userSettings.loadFromLS();

    return forkJoin([
      translate.use(lang),
      token ? auth.loginByToken(token) : of({}),
      store.select(GlobalRulesSelectors.permissionsInited).pipe(
        filter((inited) => !!inited),
        first(),
      ),
    ]);
  };
}

const ngxsModules = [
  NgxsModule.forRoot([
    GlobalState,
    GlobalRulesState,
    ProjectRulesState,
    WebsocketsState,
    NotificationsState,
    RightSidebarState,
    IssuesState,
    WorksState,
  ], {
    developmentMode: !environment.production,
  }),
  NgxsReduxDevtoolsPluginModule.forRoot({
    disabled: environment.production,
  }),
  NgxsRouterPluginModule.forRoot(),
];

@NgModule({
  declarations: [
    AppComponent,
    SidemenuComponent,
    SubmenuComponent,
    ProfileComponent,
    LoginComponent,
    RegistrationComponent,
    LogoutComponent,
    AppTopBarComponent,
    AppBreadcrumbComponent,
    ResetPasswordComponent,
    LandingComponent,
    SidemenuItemComponent,
  ],
  imports: [
    ...primengModules,
    ...ngxsModules,
    BrowserModule,
    BrowserAnimationsModule,
    ReactiveFormsModule,
    HttpClientModule,
    ApiModule,
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: httpLoaderFactory,
        deps: [HttpClient],
      },
    }),
    DashboardModule,
    ProjectsModule,
    DepartmentModule,
    BrowserAnimationsModule,
    AppRoutingModule,
    ProgramsModule,
    KnowledgebaseModule,
    ProcurementsModule,
    RisksModule,
    ProjectModule,
    NewsFormModule,
    NewsfeedModule,
    FormsModule,
    OrganizationsModule,
    ProfileModule,
    DynamicDialogModule,
    ProjectDocumentsModule,
    DocumentsModule,
    UserWidgetsModule,
    DirectivesModule,
    TabMenuModule,
    SharedComponentsModule,
    NotificationsModule,
    BadgeModule,
    RightSidebarModule,
    CustomIconsComponent,
  ],
  providers: [
    {provide: HTTP_INTERCEPTORS, useClass: ErrorInterceptor, multi: true},
    {provide: LOCALE_ID, useValue: 'ru-RU'},
    {
      provide: APP_INITIALIZER,
      useFactory: appInitializerFactory,
      deps: [TranslateService, AuthenticationService, UserSettingsService, Store],
      multi: true,
    },
    WebsocketsApiService,
    UsersService,
    SidemenuService,
    MessageService,
    ConfirmationService,
    LoadingService,
    DialogService,
    BreadcrumbService,
    ConfirmationService,
    DialogService,
  ],
  exports: [],
  bootstrap: [AppComponent],
})
export class AppModule {
}
