import {Component, OnInit} from '@angular/core';
import {ProfileFormComponent} from '@core-project/modules/forms/profile-form/profile-form.component';
import {firstValueFrom, forkJoin} from 'rxjs';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {DialogService} from 'primeng/dynamicdialog';
import {TranslateService} from '@ngx-translate/core';
import {User, UsersService} from 'data-access/sdk/api2';

@Component({
  selector: 'app-profile',
  templateUrl: './profile.component.html',
  styleUrls: ['./profile.component.scss'],
})
export class ProfileComponent implements OnInit {
  user: User;
  perms: string[];

  defaultAvatar = 'assets/images/default_photo.png';

  constructor(
    private readonly authenticationService: AuthenticationService,
    private readonly usersService: UsersService,
    private readonly dialogService: DialogService,
    private readonly translateService: TranslateService,
  ) {
  }

  ngOnInit() {
    this.loadData();
  }

  userEdit() {
    const user = this.user;
    const telephones = this.user.rel_telephones;

    const dialogRef = this.dialogService.open(ProfileFormComponent, {
      data: {user, telephones},
      header: this.translateService.instant('PROFILE.EDIT_HEADER'),
      width: '40%',
    });

    firstValueFrom(dialogRef.onClose).then(() => {
      this.loadData();
      this.authenticationService.updateCurrentUser();
    });
  }

  private loadData() {
    const user$ = this.usersService.usersRead({
      id: this.authenticationService.user.id,
      include: 'telephones, department',
    });
    const perms$ = this.usersService.usersPermissions({id: this.authenticationService.user.id});

    forkJoin([user$, perms$]).subscribe((res) => {
      this.user = res[0];
      this.perms = res[1].permissions;
    });
  }
}
