import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { MessageService } from 'primeng/api';
import { firstValueFrom } from 'rxjs';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-registration',
  templateUrl: './registration.component.html',
  styleUrls: ['./registration.component.scss'],
})
export class RegistrationComponent implements OnInit {
  loading = false;
  form: FormGroup;

  constructor(
    private readonly formBuilder: FormBuilder,
    private readonly authService: AuthService,
    private readonly router: Router,
    private readonly messageService: MessageService,
    private readonly translateService: TranslateService
  ) {}

  ngOnInit() {
    this.form = this.formBuilder.group({
      email: ['', Validators.compose([Validators.required, Validators.email])],
      first_name: ['', Validators.required],
      last_name: ['', Validators.required],
    });
  }

  onSubmit() {
    this.loading = true;
    firstValueFrom(this.authService.authUsersCreate(this.form.value))
      .then((_u) => {
        this.messageService.add({
          severity: 'success',
          summary: this.translateService.instant('REGISTRATION.SUCCESS_HDR'),
          detail: this.translateService.instant('REGISTRATION.SUCCESS_MSG'),
          sticky: true,
        });
        this.router.navigateByUrl('/login');
      })
      .catch((_e) => (this.loading = false));
  }
}

