import { animate, state, style, transition, trigger } from '@angular/animations';
import { Location } from '@angular/common';
import { Component, Input, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { MenuItem } from 'primeng/api';
import { SidemenuComponent } from '../sidemenu/sidemenu.component';

@Component({
  /* tslint:disable:component-selector */
  selector: '[app-submenu]',
  /* tslint:enable:component-selector */
  templateUrl: './submenu.component.html',
  styleUrls: ['./submenu.component.scss'],
  animations: [
    trigger('children', [
      state(
        'void',
        style({
          height: '0px',
        })
      ),
      state(
        'hiddenAnimated',
        style({
          height: '0px',
        })
      ),
      state(
        'visibleAnimated',
        style({
          height: '*',
        })
      ),
      state(
        'visible',
        style({
          height: '*',
          'z-index': 100,
        })
      ),
      state(
        'hidden',
        style({
          height: '0px',
          'z-index': '*',
        })
      ),
      transition('visibleAnimated => hiddenAnimated', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
      transition('hiddenAnimated => visibleAnimated', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
      transition('void => visibleAnimated, visibleAnimated => void', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
    ]),
  ],
})
export class SubmenuComponent {
  @Input() item: MenuItem;
  @Input() root: boolean;
  @Input() visible: boolean;
  activeIndex: number;

  constructor(public router: Router, public location: Location, public appMenu: SidemenuComponent) {}

  _reset: boolean;

  get reset(): boolean {
    return this._reset;
  }

  @Input() set reset(val: boolean) {
    this._reset = val;

    if (this._reset && (this.appMenu.isHorizontal() || this.appMenu.isOverlay())) {
      this.activeIndex = null;
    }
  }

  _parentActive: boolean;

  get parentActive(): boolean {
    return this._parentActive;
  }

  @Input() set parentActive(val: boolean) {
    this._parentActive = val;

    if (!this._parentActive) {
      this.activeIndex = null;
    }
  }

  itemClick(event: Event, item: MenuItem, index: number) {
    if (this.root) {
      this.appMenu.menuHoverActive = !this.appMenu.menuHoverActive;
      event.preventDefault();
    }

    // avoid processing disabled items
    if (item.disabled) {
      event.preventDefault();
      return true;
    }

    // activate current item and deactivate active sibling if any
    if (item.routerLink || item.items || item.command || item.url) {
      this.activeIndex = (this.activeIndex as number) === index ? -1 : index;
    }

    // execute command
    if (item.command) {
      item.command({ originalEvent: event, item: item });
    }

    // prevent hash change
    if (item.items || (!item.url && !item.routerLink)) {
      setTimeout(() => {
        this.appMenu.layoutMenuScrollerViewChild.moveBar();
      }, 450);
      event.preventDefault();
    }

    // hide menu
    if (!item.items) {
      if (this.appMenu.isMobile()) {
        this.appMenu.sidebarActive = false;
        this.appMenu.mobileMenuActive = false;
      }

      if (this.appMenu.isHorizontal()) {
        this.appMenu.resetMenu = true;
      } else {
        this.appMenu.resetMenu = false;
      }

      this.appMenu.menuHoverActive = !this.appMenu.menuHoverActive;
    }
  }

  onMouseEnter(index: number) {
    if (this.root && this.appMenu.menuHoverActive && this.appMenu.isHorizontal() && !this.appMenu.isMobile() && !this.appMenu.isTablet()) {
      this.activeIndex = index;
    }
  }

  isActive(index: number): boolean {
    return this.activeIndex === index;
  }
}
