import { Component, EventEmitter, Input, OnDestroy, OnInit, Output, ViewChild } from '@angular/core';
import { UserWidget } from 'data-access/sdk/api2';
import { WidgetContainerDirective } from 'utils/shared-directives';
import { WidgetParams } from '@core-project/interfaces';
import { Subscription } from 'rxjs';
import { widgetsParams } from '@core-project/helpers/widget';
import * as _ from 'lodash-es';

@Component({
  selector: 'app-base-widget',
  templateUrl: './base-widget.component.html',
  styleUrls: ['./base-widget.component.scss'],
})
export class BaseWidgetComponent implements OnInit, OnDestroy {
  @Input() userWidget: UserWidget;

  @Output() onChange = new EventEmitter<UserWidget>();
  @Output() onClose = new EventEmitter();

  @ViewChild(WidgetContainerDirective, { static: true }) contentContainer: WidgetContainerDirective;

  public loading: boolean;
  public widgetParams: WidgetParams;

  private dataChSub: Subscription;

  ngOnInit() {
    this.widgetParams = _.find(widgetsParams, ['name', this.userWidget.widget]);
    console.log(this.contentContainer);
    this.renderComponent();
  }

  ngOnDestroy() {
    this.dataChSub?.unsubscribe();
  }

  private dataChange(newData: any) {
    // console.log('DATA CHANGE', newData);
    this.userWidget.properties = newData;
    this.onChange.emit(this.userWidget);
  }

  private renderComponent() {
    const componentRef = this.contentContainer.viewContainerRef.createComponent(this.widgetParams.component);
    if (this.userWidget.properties) {
      Object.keys(this.userWidget.properties).forEach((k) => {
        componentRef.setInput(k, this.userWidget.properties[k]);
      });
    }

    this.dataChSub = componentRef.instance.onChange.subscribe((ch) => this.dataChange(ch));
  }
}
