import { Component, Input, OnInit } from '@angular/core';
import { DepartmentTotalTasksData, DepartmentTotalTasksOptions } from '../../interfaces/department-total-tasks';

@Component({
  selector: 'app-department-total-tasks',
  templateUrl: './department-total-tasks.component.html',
  styleUrls: ['./department-total-tasks.component.scss'],
})
export class DepartmentTotalTasksComponent implements OnInit {
  @Input() data: DepartmentTotalTasksData;
  @Input() options: DepartmentTotalTasksOptions;

  barChartData: any;
  barChartOptions: any;

  constructor() {}

  ngOnInit() {
    const datasets = [];

    if (this.data.closed) {
      datasets.push({
        label: 'Завершенные ранее',
        backgroundColor: 'rgba(0, 0, 0, 0.2)',
        data: this.data.closed,
      });
    }
    if (this.data.closedRecently) {
      datasets.push({
        label: 'Завершенные на этой неделе',
        backgroundColor: 'rgba(0, 255, 0, 0.2)',
        data: this.data.closedRecently,
      });
    }
    if (this.data.onValidation) {
      datasets.push({
        label: 'На проверке',
        backgroundColor: 'rgba(255, 255, 0, 0.2)',
        data: this.data.onValidation,
      });
    }
    if (this.data.current) {
      datasets.push({
        label: 'Текущие',
        backgroundColor: 'rgba(0, 0, 255, 0.2)',
        data: this.data.current,
      });
    }
    if (this.data.delayed) {
      datasets.push({
        label: 'Задерживаются',
        backgroundColor: 'rgba(255,118,0,0.21)',
        data: this.data.delayed,
      });
    }
    if (this.data.failed) {
      datasets.push({
        label: 'Сорванные',
        backgroundColor: 'rgba(255, 0, 0, 0.2)',
        data: this.data.failed,
      });
    }

    this.barChartData = {
      labels: this.data.labels,
      datasets: datasets,
    };

    this.barChartOptions = {
      title: {
        display: this.options.showLabel,
        text: this.options.label ? this.options.label : 'Исполнение задач',
        fontSize: this.options.fontSize ? this.options.fontSize : 16,
      },
      legend: {
        position: this.options.legendPosition ? this.options.legendPosition : 'bottom',
      },
      scales: {
        xAxes: [
          {
            stacked: true,
          },
        ],
        yAxes: [
          {
            stacked: true,
            ticks: {
              min: this.getMinY(),
            },
          },
        ],
      },
      elements: {
        rectangle: {
          borderColor: 'gray',
          borderWidth: 2,
        },
      },
    };
  }

  getMinY() {
    if (this.data.closed) {
      const min = Math.min.apply(null, this.data.closed);
      return min > 10 ? min - 10 : 0;
    } else {
      return 0;
    }
  }
}
