import {Component, OnDestroy, OnInit} from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import {ActivatedRoute} from '@angular/router';
import {Department, DepartmentsService} from 'data-access/sdk/api2';
import {MenuItem} from 'primeng/api';
import {Store} from '@ngxs/store';
import {GlobalActions} from 'data-access/store';

@Component({
  selector: 'app-department',
  templateUrl: './department.component.html',
  styleUrls: ['./department.component.scss'],
})
export class DepartmentComponent implements OnInit, OnDestroy {
  departmentId: number;
  department: Department;

  constructor(
    private departmentService: DepartmentsService,
    private route: ActivatedRoute,
    private translateService: TranslateService,
    private store: Store,
  ) {
  }

  ngOnInit() {
    this.route.params.subscribe((params) => {
      this.departmentId = params.departmentId;
      this.departmentService.departmentsRead({id: this.departmentId}).subscribe((department) => {
        this.department = department;

        this.store.dispatch(new GlobalActions.SetAppTitle({key: this.department.name}));
        console.log('set name');

        const menuItems: MenuItem[] = [
          {label: this.translateService.instant('PROJECT.WORKS'), routerLink: `/department/${this.departmentId}/tasks`, icon: 'works'},
          {label: this.translateService.instant('PROJECT.PLAN'), routerLink: `/department/${this.departmentId}/plan`, icon: 'plans'},
        ];

        this.store.dispatch(new GlobalActions.SetTopMenuItems(menuItems));
      });
    });
  }

  ngOnDestroy() {
    this.store.dispatch(new GlobalActions.SetTopMenuItems([]));
  }
}
