import {NgModule} from '@angular/core';
import {RouterModule, Routes} from '@angular/router';
import {AuthGuard} from '@core-project/auth/guards';
import {CompanyStructureComponent} from './structure/company-structure.component';
import {MethodologyComponent} from '@core-project/containers/knowledgebase/methodology/methodology.component';
import {ProjectGroupsComponent} from '@core-project/containers/knowledgebase/methodology/project-groups/project-groups.component';
import {RolesComponent} from '@core-project/containers/knowledgebase/methodology/roles/roles.component';
import {FinanceComponent} from '@core-project/containers/knowledgebase/methodology/finance/finance.component';
import {DocumentsComponent} from '@core-project/containers/knowledgebase/methodology/documents/documents.component';
import {ApprovalsComponent} from '@core-project/containers/knowledgebase/methodology/approvals/approvals.component';
import {ProjectOfficeComponent} from '@core-project/containers/knowledgebase/methodology/project-office/project-office.component';
import {ProjectMetasComponent} from '@core-project/containers/knowledgebase/methodology/project-metas/project-metas.component';
import {RisksComponent} from '@core-project/containers/knowledgebase/methodology/risks/risks.component';
import {CompanyComponent} from '@core-project/containers/knowledgebase/company/company.component';
import {WorkTemplatesComponent} from '@core-project/containers/knowledgebase/methodology/work-templates/work-templates.component';
import {DATA_APP_TITLE_KEY} from 'data-access/store';

const knowledgebaseRoutes: Routes = [
  {
    path: 'company-structure',
    component: CompanyStructureComponent,
    canActivate: [AuthGuard],
    data: {[DATA_APP_TITLE_KEY]: 'KB.STRUCTURE.HEADER'},
  },
  {
    path: 'settings',
    component: MethodologyComponent,
    canActivate: [AuthGuard],
    data: {[DATA_APP_TITLE_KEY]: 'Настройки'},
    children: [
      {
        path: 'project-cats',
        loadChildren: () => import('./methodology/project-categories/project-categories.module').then((m) => m.ProjectCategoriesModule),
      },
      {path: 'project-metas', component: ProjectMetasComponent},
      {path: 'templates', component: WorkTemplatesComponent},
      {path: 'stages', loadChildren: () => import('./methodology/stages/stages.module').then((m) => m.StagesModule)},
      {path: 'plugins', loadChildren: () => import('features/plugins-loader').then((m) => m.PluginsLoaderModule)},
      {path: 'project-groups', component: ProjectGroupsComponent},
      {path: 'roles', component: RolesComponent},
      {path: 'documents', component: DocumentsComponent},
      {path: 'approvals', component: ApprovalsComponent},
      {path: 'project-office', component: ProjectOfficeComponent},
      {path: 'finance', component: FinanceComponent},
      {path: 'risks', component: RisksComponent},
      {path: 'company', component: CompanyComponent},
    ],
  },
];

@NgModule({
  declarations: [],
  imports: [RouterModule.forChild(knowledgebaseRoutes)],
  exports: [RouterModule],
})
export class KnowledgebaseRoutingModule {
}
