import { Component, OnDestroy, OnInit } from '@angular/core';
import { MegaMenuItem, MenuItem } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';
import { BreadcrumbService } from '@core-project/services/breadcrumb.service';

@Component({
  selector: 'app-knowledgebase',
  templateUrl: './knowledgebase.component.html',
  styleUrls: ['./knowledgebase.component.scss'],
})
export class KnowledgebaseComponent implements OnInit, OnDestroy {
  menuItems: MenuItem[];

  constructor(private translateService: TranslateService, private bcs: BreadcrumbService) {}

  ngOnInit() {
    this.menuItems = [
      {
        label: this.translateService.instant('KB.MENU.STRUCTURE'),
        routerLink: '/knowledgebase/company-structure',
      },
      {
        label: this.translateService.instant('KB.MENU.METHODOLOGY'),
        items: [
          { label: this.translateService.instant('METHODOLOGY.MENU.PR_CATS'), routerLink: '/knowledgebase/methodology/project-cats' },
          { label: this.translateService.instant('METHODOLOGY.MENU.PR_METAS'), routerLink: '/knowledgebase/methodology/project-metas' },
          { label: this.translateService.instant('METHODOLOGY.MENU.WORK_TEMPLATES'), routerLink: '/knowledgebase/methodology/templates' },
          { label: this.translateService.instant('METHODOLOGY.MENU.PR_GROUPS'), routerLink: '/knowledgebase/methodology/project-groups' },
          { label: this.translateService.instant('METHODOLOGY.MENU.ROLES'), routerLink: '/knowledgebase/methodology/roles' },
          { label: this.translateService.instant('METHODOLOGY.MENU.DOCUMENTS'), routerLink: '/knowledgebase/methodology/documents' },
          { label: this.translateService.instant('METHODOLOGY.MENU.APPROVALS'), routerLink: '/knowledgebase/methodology/approvals' },
          { label: this.translateService.instant('METHODOLOGY.MENU.PR_OFFICE'), routerLink: '/knowledgebase/methodology/project-office' },
          { label: this.translateService.instant('METHODOLOGY.MENU.FINANCE'), routerLink: '/knowledgebase/methodology/finance' },
          { label: this.translateService.instant('METHODOLOGY.MENU.RISKS'), routerLink: '/knowledgebase/methodology/risks' },
        ],
      },
      { label: this.translateService.instant('KB.MENU.STATS'), routerLink: '/knowledgebase/stats' },
      { label: this.translateService.instant('KB.MENU.METRICS'), routerLink: '/knowledgebase/metrics' },
      { label: this.translateService.instant('KB.MENU.KB'), routerLink: '/knowledgebase/kb' },

      // {label: 'Контакты', routerLink: '/knowledgebase/contacts', icon: 'fa-light fa-id-card'},
    ];
    this.bcs.setItems([{ id: 'KB', label: this.translateService.instant('SIDEMENU.KB') }]);
  }

  ngOnDestroy() {
    this.bcs.removeItemsUpTo('KB');
  }
}
