import {NgModule} from '@angular/core';
import {CommonModule} from '@angular/common';
import {FormsModule} from '@angular/forms';
import {TranslateModule} from '@ngx-translate/core';

// primeng
import {TabMenuModule} from 'primeng/tabmenu';
import {TableModule} from 'primeng/table';
import {RippleModule} from 'primeng/ripple';
import {ButtonModule} from 'primeng/button';
import {ChipModule} from 'primeng/chip';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {CardModule} from 'primeng/card';
import {TreeModule} from 'primeng/tree';
import {OrganizationChartModule} from 'primeng/organizationchart';
import {AvatarModule} from 'primeng/avatar';
import {MegaMenuModule} from 'primeng/megamenu';
import {MenubarModule} from 'primeng/menubar';
import {AccordionModule} from 'primeng/accordion';
import {ImageModule} from 'primeng/image';
import {ToggleButtonModule} from 'primeng/togglebutton';
import {DropdownModule} from 'primeng/dropdown';
import {FileUploadModule} from 'primeng/fileupload';
import {IndicatorsComponent} from './methodology/indicators/indicators.component';
import {OrderListModule} from 'primeng/orderlist';

import {KnowledgebaseComponent} from './knowledgebase.component';
import {KnowledgebaseRoutingModule} from './knowledgebase-routing.module';
import {KbComponent} from './kb/kb.component';
import {CompanyStructureComponent} from './structure/company-structure.component';
import {ContactsComponent} from './contacts/contacts.component';
import {SharedComponentsModule} from '@core-project/modules/shared-components/shared-components.module';
import {StatsComponent} from './stats/stats.component';
import {MetricsComponent} from './metrics/metrics.component';
import {MethodologyComponent} from '@core-project/containers/knowledgebase/methodology/methodology.component';
import {ProjectGroupsComponent} from './methodology/project-groups/project-groups.component';
import {RolesComponent} from './methodology/roles/roles.component';
import {DocumentsComponent} from './methodology/documents/documents.component';
import {ApprovalsComponent} from './methodology/approvals/approvals.component';
import {ProjectOfficeComponent} from './methodology/project-office/project-office.component';
import {FinanceComponent} from './methodology/finance/finance.component';
import {FinanceFormsModule} from '@core-project/modules/forms/finance-forms/finance-forms.module';
import {RoleFormsModule} from '@core-project/modules/forms/role-forms/role-forms.module';
import {ProjectMetasComponent} from './methodology/project-metas/project-metas.component';
import {ProjectMetaGroupFormModule} from '@core-project/modules/forms/project-meta-group-form/project-meta-group-form.module';
import {ProjectMetaFormModule} from '@core-project/modules/forms/project-meta-form/project-meta-form.module';
import {ProjectTypeFormModule} from '@core-project/modules/forms/project-type-form/project-type-form.module';
import {DepartmentFormModule} from '@core-project/modules/forms/department-form/department-form.module';
import {UserFormsModule} from '@core-project/modules/forms/user-forms/user-forms.module';
import {RisksComponent} from './methodology/risks/risks.component';
import {IssueFormsModule} from '@core-project/modules/forms/issue-forms/issue-forms.module';
import {CompanyComponent} from './company/company.component';
import {CompanyFormModule} from '@core-project/modules/forms/company-form/company-form.module';
import {WorkTemplatesComponent} from './methodology/work-templates/work-templates.component';
import {ProjectGroupFormModule} from '@core-project/modules/forms/project-group-form/project-group-form.module';
import {DocumentSubtypesTableComponent} from './methodology/documents/document-subtypes-table/document-subtypes-table.component';
import {DocumentMetaFieldsTableComponent} from './methodology/documents/document-meta-fields-table/document-meta-fields-table.component';
import {
  MetaFieldsTableComponent,
} from '@core-project/containers/knowledgebase/methodology/project-metas/meta-fields-table/meta-fields-table.component';

import {StringInfoOverlaysModule} from 'features/string-info-overlays';
import {UnfreezePipe} from 'utils/shared-pipes';
import {HasPermissionDirective} from 'utils/shared-directives';

// primeng
const primeNgModules = [
  TabMenuModule,
  TableModule,
  RippleModule,
  ButtonModule,
  ChipModule,
  ConfirmDialogModule,
  CardModule,
  TreeModule,
  OrganizationChartModule,
  AvatarModule,
  MegaMenuModule,
  MenubarModule,
  AccordionModule,
  ImageModule,
  ToggleButtonModule,
  DropdownModule,
  FileUploadModule,
  OrderListModule,
];

@NgModule({
  declarations: [
    KnowledgebaseComponent,
    KbComponent,
    CompanyStructureComponent,
    ContactsComponent,
    StatsComponent,
    MetricsComponent,
    MethodologyComponent,
    ProjectGroupsComponent,
    RolesComponent,
    DocumentsComponent,
    ApprovalsComponent,
    ProjectOfficeComponent,
    FinanceComponent,
    ProjectMetasComponent,
    RisksComponent,
    CompanyComponent,
    WorkTemplatesComponent,
    IndicatorsComponent,
    DocumentSubtypesTableComponent,
    DocumentMetaFieldsTableComponent,
    MetaFieldsTableComponent,
  ],
  imports: [
    ...primeNgModules,
    CommonModule,
    TranslateModule,
    FormsModule,
    KnowledgebaseRoutingModule,
    SharedComponentsModule,
    FinanceFormsModule,
    RoleFormsModule,
    ProjectGroupFormModule,
    ProjectMetaGroupFormModule,
    ProjectMetaFormModule,
    ProjectTypeFormModule,
    DepartmentFormModule,
    UserFormsModule,
    IssueFormsModule,
    CompanyFormModule,
    HasPermissionDirective,
    UnfreezePipe,
    StringInfoOverlaysModule,
  ],
})
export class KnowledgebaseModule {
}
