import { Component, OnInit } from '@angular/core';
import { FinanceService } from 'data-access/sdk/api2';
import { BudgetCategory } from 'data-access/sdk/api2';
import { BudgetType } from 'data-access/sdk/api2';
import { forkJoin, Observable } from 'rxjs';
import { filter, map } from 'rxjs/operators';
import { DialogService } from 'primeng/dynamicdialog';
import { BudgetCategoryFormComponent } from '@core-project/modules/forms/finance-forms/budget-category-form/budget-category-form.component';
import { TranslateService } from '@ngx-translate/core';
import { BudgetTypeFormComponent } from '@core-project/modules/forms/finance-forms/budget-type-form/budget-type-form.component';
import { MessageService } from 'primeng/api';

@Component({
  selector: 'app-finance',
  templateUrl: './finance.component.html',
  styleUrls: ['./finance.component.scss'],
})
export class FinanceComponent implements OnInit {
  public categories$: Observable<Array<BudgetCategory>>;
  public types$: Observable<Array<BudgetType>>;

  constructor(
    private financeService: FinanceService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private messageService: MessageService
  ) {}

  ngOnInit(): void {
    this.loadData();
  }

  loadData() {
    this.categories$ = this.financeService.financeBudgetCategoriesList();
    this.types$ = this.financeService.financeBudgetTypesList();
  }

  editCategory(category?: BudgetCategory) {
    const ref = this.dialogService.open(BudgetCategoryFormComponent, {
      header: category
        ? this.translateService.instant('KB.METHODOLOGY.FINANCE.CATEGORY_FORM.HEADER_EDIT')
        : this.translateService.instant('KB.METHODOLOGY.FINANCE.CATEGORY_FORM.HEADER_ADD'),
      width: '40%',
      data: {
        category: category,
      },
    });

    ref.onClose.subscribe((editedCat) => {
      if (editedCat) {
        let res: Observable<BudgetCategory>;
        if (category) {
          res = this.financeService.financeBudgetCategoriesPartialUpdate({ id: category.id, data: editedCat });
        } else {
          res = this.financeService.financeBudgetCategoriesCreate(editedCat);
        }
        res.subscribe((_r) => this.loadData());
      }
    });
  }

  deleteCategory(category: BudgetCategory) {
    if (category.budget_types.length > 0) {
      this.messageService.add({
        severity: 'error',
        detail: this.translateService.instant('KB.METHODOLOGY.FINANCE.DEL_CATEGORY_HAS_TYPES_ERROR'),
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
      });
    } else {
      this.financeService.financeBudgetCategoriesDelete(category.id).subscribe((_r) => this.loadData());
    }
  }

  editType(bType?: BudgetType) {
    const ref = this.dialogService.open(BudgetTypeFormComponent, {
      header: bType
        ? this.translateService.instant('KB.METHODOLOGY.FINANCE.TYPE_FORM.HEADER_EDIT')
        : this.translateService.instant('KB.METHODOLOGY.FINANCE.TYPE_FORM.HEADER_ADD'),
      width: '40%',
      data: {
        bType: bType,
      },
    });

    ref.onClose.subscribe((editedType) => {
      if (editedType) {
        let res: Observable<BudgetType>;
        if (bType) {
          res = this.financeService.financeBudgetTypesPartialUpdate({ id: bType.id, data: editedType });
        } else {
          res = this.financeService.financeBudgetTypesCreate(editedType);
        }
        res.subscribe((_r) => this.loadData());
      }
    });
  }

  deleteType(bType: BudgetType) {
    if (bType.project_budget_types.length > 0) {
      this.messageService.add({
        severity: 'error',
        detail: this.translateService.instant('KB.METHODOLOGY.FINANCE.DEL_TYPE_HAS_PROJECTS_ERROR'),
        summary: this.translateService.instant('ERRORS.DELETE_ERROR'),
      });
    } else {
      this.financeService.financeBudgetTypesDelete(bType.id).subscribe((_r) => this.loadData());
    }
  }
}
