import {Component, OnDestroy, OnInit} from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import {MenuItem} from 'primeng/api';
import {Store} from '@ngxs/store';
import {GlobalActions} from 'data-access/store';

@Component({
  selector: 'app-methodology',
  templateUrl: './methodology.component.html',
  styleUrls: ['./methodology.component.scss'],
})
export class MethodologyComponent implements OnInit, OnDestroy {

  constructor(
    private readonly translateService: TranslateService,
    private readonly store: Store,
  ) {
  }

  ngOnInit(): void {
    const menuItems: MenuItem[] = [
      {label: this.translateService.instant('METHODOLOGY.MENU.PR_CATS'), routerLink: '/settings/project-cats'},
      {label: this.translateService.instant('METHODOLOGY.MENU.PR_METAS'), routerLink: '/settings/project-metas'},
      {label: this.translateService.instant('METHODOLOGY.MENU.WORK_TEMPLATES'), routerLink: '/settings/templates'},
      {label: this.translateService.instant('METHODOLOGY.MENU.STAGES'), routerLink: '/settings/stages'},
      {label: this.translateService.instant('METHODOLOGY.MENU.PR_GROUPS'), routerLink: '/settings/project-groups'},
      {label: this.translateService.instant('METHODOLOGY.MENU.ROLES'), routerLink: '/settings/roles'},
      {label: this.translateService.instant('METHODOLOGY.MENU.DOCUMENTS'), routerLink: '/settings/documents'},
      {label: this.translateService.instant('METHODOLOGY.MENU.FINANCE'), routerLink: '/settings/finance'},
      {label: this.translateService.instant('METHODOLOGY.MENU.RISKS'), routerLink: '/settings/risks'},
      {label: this.translateService.instant('METHODOLOGY.MENU.COMPANY'), routerLink: '/settings/company'},
      {label: this.translateService.instant('METHODOLOGY.MENU.PLUGINS'), routerLink: '/settings/plugins'},
    ];
    this.store.dispatch(new GlobalActions.SetTopMenuItems(menuItems));
  }

  ngOnDestroy() {
    this.store.dispatch(new GlobalActions.SetTopMenuItems([]));
  }
}
