import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { TranslateModule } from '@ngx-translate/core';
import { NgxsModule } from '@ngxs/store';
import { ReactiveFormsModule } from '@angular/forms';

// primeng
import { AccordionModule } from 'primeng/accordion';
import { ChipModule } from 'primeng/chip';
import { ButtonModule } from 'primeng/button';
import { RippleModule } from 'primeng/ripple';
import { TreeModule } from 'primeng/tree';
import { ConfirmDialogModule } from 'primeng/confirmdialog';
import { TableModule } from 'primeng/table';
import { DropdownModule } from 'primeng/dropdown';
import { OverlayPanelModule } from 'primeng/overlaypanel';

import { StageOrderPipe } from '@core-project/containers/knowledgebase/methodology/stages/pipes/stage-order.pipe';
import { FilterStagesPipe } from '@core-project/containers/knowledgebase/methodology/stages/pipes/filter-stages.pipe';
import { SharedComponentsModule } from '@core-project/modules/shared-components/shared-components.module';
import { RouteTransformerDirective } from '@core-project/directives/route-transformer-directive.directive';
import { StagesSharedModule } from '@core-project/containers/knowledgebase/methodology/stages/stages-shared.module';

import { ProjectCategoriesRoutingModule } from './project-categories-routing.module';
import { ProjectCategoriesFacadeComponent } from './project-categories-facade.component';
import { ProjectStagesComponent } from './containers/project-stages/project-stages.component';


import { ProjectStagesState } from 'data-access/store';
import {HasPermissionDirective} from 'utils/shared-directives';
import {UnfreezePipe, UnsafePipe} from 'utils/shared-pipes';

const primeNgModules = [
  AccordionModule,
  ChipModule,
  ButtonModule,
  RippleModule,
  TreeModule,
  ConfirmDialogModule,
  TableModule,
  DropdownModule,
  OverlayPanelModule,
];

@NgModule({
  declarations: [ProjectCategoriesFacadeComponent, ProjectStagesComponent],
  imports: [
    ...primeNgModules,
    CommonModule,
    ReactiveFormsModule,
    ProjectCategoriesRoutingModule,
    TranslateModule,
    NgxsModule.forFeature([ProjectStagesState]),
    SharedComponentsModule,
    StagesSharedModule,
    StageOrderPipe,
    UnfreezePipe,
    FilterStagesPipe,
    UnsafePipe,
    RouteTransformerDirective,
    HasPermissionDirective,
  ],
})
export class ProjectCategoriesModule {}
