import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { Select, Store } from '@ngxs/store';

import { ProjectType } from 'data-access/sdk/api2';

import {GlobalRulesSelectors, ProjectStagesAction, ProjectStagesSelectors} from 'data-access/store';
import { StageModel } from 'data-access/sdk/internal-api';

@Injectable({
  providedIn: 'root',
})
export class ProjectStagesStoreService {
  @Select(GlobalRulesSelectors.canDelete('projectstage')) projectStagesCanDelete$: Observable<boolean>;
  @Select(GlobalRulesSelectors.canUpdate('projectstage')) projectStagesCanUpdate$: Observable<boolean>;

  @Select(ProjectStagesSelectors.stages) stages$: Observable<StageModel[]>;
  @Select(ProjectStagesSelectors.stagesForAdd) stagesForAdd$: Observable<StageModel[]>;
  @Select(ProjectStagesSelectors.loadingStages) stagesLoading$: Observable<boolean>;
  @Select(ProjectStagesSelectors.deleteProjectStageLoading) deleteProjectStageLoading$: Observable<boolean>;

  constructor(private readonly store: Store) {}

  createProjectStage(projectStageId: number, selectedTypeId: ProjectType['id']) {
    return this.store.dispatch(new ProjectStagesAction.CreateProjectStage(selectedTypeId, projectStageId));
  }

  deleteProjectStage(replaceWithStage: StageModel['id'], selectedStageForDeleteId: number) {
    return this.store.dispatch(new ProjectStagesAction.DeleteProjectStage(selectedStageForDeleteId, replaceWithStage));
  }

  updateProjectStages(stages: StageModel[]) {
    return this.store.dispatch(new ProjectStagesAction.UpdateProjectStages(stages));
  }

  readProjectStages(projectTypeId: ProjectType['id']) {
    return this.store.dispatch(new ProjectStagesAction.ReadProjectStages(projectTypeId));
  }
}
