import { Component, OnInit } from '@angular/core';
import { IssuesService } from 'data-access/sdk/api2';
import { firstValueFrom, Observable } from 'rxjs';
import { IssueDomain } from 'data-access/sdk/api2';
import { MessageService } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';
import { DialogService } from 'primeng/dynamicdialog';
import { IssueDomainFormComponent } from '@core-project/modules/forms/issue-forms/issue-domain-form/issue-domain-form.component';

@Component({
  selector: 'app-risks',
  templateUrl: './risks.component.html',
  styleUrls: ['./risks.component.scss'],
})
export class RisksComponent implements OnInit {
  domains$: Observable<IssueDomain[]>;

  constructor(
    private issuesService: IssuesService,
    private messageService: MessageService,
    private ts: TranslateService,
    private dialogService: DialogService
  ) {}

  ngOnInit() {
    this.loadDomains();
  }

  loadDomains() {
    this.domains$ = this.issuesService.issuesDomainsList();
  }

  editDomain(domain?: IssueDomain) {
    const ref = this.dialogService.open(IssueDomainFormComponent, {
      header: domain ? this.ts.instant('KB.METHODOLOGY.ISSUES.EDIT_DOMAIN') : this.ts.instant('KB.METHODOLOGY.ISSUES.ADD_DOMAIN'),
      width: '40%',
      data: { domain: domain },
    });

    firstValueFrom(ref.onClose).then((d) => {
      if (!d) {
        return;
      }

      if (domain) {
        Object.assign(domain, d);
      } else {
        this.loadDomains();
      }
    });
  }

  deleteDomain(domain: IssueDomain) {}
}
