import {Component, OnInit, ViewEncapsulation} from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import {Role, RolesService} from 'data-access/sdk/api2';
import {DialogService} from 'primeng/dynamicdialog';
import {BehaviorSubject, Observable} from 'rxjs';
import {RoleFormComponent} from '@core-project/modules/forms/role-forms/role-form/role-form.component';
import {ConfirmationService} from 'primeng/api';

@Component({
  selector: 'app-roles',
  templateUrl: './roles.component.html',
  styleUrls: ['./roles.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class RolesComponent implements OnInit {
  readonly rolesSubject: BehaviorSubject<Role[]> = new BehaviorSubject([]);
  readonly roles$: Observable<Role[]> = this.rolesSubject.asObservable();


  constructor(
    private translateService: TranslateService,
    private rolesService: RolesService,
    private dialogService: DialogService,
    private confirmationService: ConfirmationService,
  ) {
  }

  ngOnInit(): void {
    this.loadData();
  }

  loadData() {
    this.rolesService.rolesList({})
      .subscribe((roles) => {
        this.rolesSubject.next(roles);
      });
  }

  editRole(role?: Role) {
    const ref = this.dialogService.open(RoleFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.ROLES.ROLE_FORM.HEADER_EDIT'),
      width: '70%',
      data: {role},
    });

    ref.onClose.subscribe((editedRole) => {
      if (editedRole) {
        this.rolesService.rolesPartialUpdate({id: role.id, data: editedRole})
          .subscribe((_r) => this.loadData());
      }
    });
  }

  createRole() {
    const ref = this.dialogService.open(RoleFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.ROLES.ROLE_FORM.HEADER_ADD'),
      width: '70%',
    });

    ref.onClose
      .subscribe((editedRole) => {
        if (editedRole) {
          this.rolesService.rolesCreate(editedRole)
            .subscribe((_r) => this.loadData());
        }
      });
  }

  copyRole(role: Role) {
    const roleCopy = JSON.parse(JSON.stringify(role));
    roleCopy.id = null;
    roleCopy.name = '';

    const ref = this.dialogService.open(RoleFormComponent, {
      header: this.translateService.instant('KB.METHODOLOGY.ROLES.ROLE_FORM.HEADER_ADD'),
      width: '70%',
      data: {role: roleCopy},
    });

    ref.onClose.subscribe((editedRole) => {
      if (editedRole) {
        this.rolesService.rolesCreate(editedRole)
          .subscribe((_r) => this.loadData());
      }
    });
  }

  deleteRole(role: Role) {
    this.confirmationService.confirm({
      message: this.translateService.instant('KB.METHODOLOGY.ROLES.DELETE_MSG'),
      accept: () => {
        this.rolesService.rolesDelete(role.id).subscribe((_r) => this.loadData());
      },
    });
  }
}
