import { ChangeDetectionStrategy, Component, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { OverlayPanel } from 'primeng/overlaypanel';

import { ProjectEnums } from '@core-project/enums/project';

import { StageModel } from 'data-access/sdk/internal-api';

interface CreateFG {
  name: FormControl<string>;
  status: FormControl<ProjectEnums.StatusEnum>;
}

@Component({
  selector: 'app-create-stage',
  templateUrl: './create-stage.component.html',
  styleUrls: ['./create-stage.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CreateStageComponent {
  @ViewChild('panel') overlayPanel: OverlayPanel;

  @Input() loading: boolean;

  @Output() onCreate: EventEmitter<Pick<StageModel, 'name' | 'status'>> = new EventEmitter();

  formGroup: FormGroup<CreateFG> = this.createFG();

  get statusFC() {
    return this.formGroup.get('status') as FormControl<ProjectEnums.StatusEnum>;
  }

  @Input() toggle(event: Event) {
    this.overlayPanel.toggle(event);
  }

  @Input() hide() {
    this.overlayPanel.hide();
    this.formGroup.reset();
  }

  create() {
    if (this.formGroup.invalid) {
      console.error('Form is invalid!', JSON.stringify(this.formGroup.value));
      return;
    }

    this.onCreate.emit({
      name: this.formGroup.value.name,
      status: this.formGroup.value.status,
    });
  }

  private createFG(): FormGroup<CreateFG> {
    return new FormGroup({
      name: new FormControl<string>('', [Validators.required]),
      status: new FormControl<ProjectEnums.StatusEnum>(ProjectEnums.StatusEnum.FUT),
    });
  }
}
