import { ChangeDetectionStrategy, Component, EventEmitter, Input, OnChanges, Output, SimpleChanges, ViewChild } from '@angular/core';
import { FormControl, Validators } from '@angular/forms';
import { OverlayPanel } from 'primeng/overlaypanel';

import { StageModel } from 'data-access/sdk/internal-api';

@Component({
  selector: 'app-delete-stage-overlay',
  templateUrl: './delete-stage-overlay.component.html',
  styleUrls: ['./delete-stage-overlay.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DeleteStageOverlayComponent implements OnChanges {
  @ViewChild('panel') overlayPanel: OverlayPanel;

  @Input() stageForDeleteId: number | undefined;
  @Input() stagesForReplace: StageModel[] = [];
  @Input() loading = false;

  @Output() onDelete = new EventEmitter<StageModel['id']>(); // replace with id

  formControl = new FormControl<StageModel>(undefined, [Validators.required]);

  ngOnChanges(changes: SimpleChanges): void {
    if (changes['loading'] !== undefined && changes['loading'].currentValue !== this.formControl.disabled) {
      if (changes['loading'].currentValue) {
        this.formControl.disable();
      } else {
        this.formControl.enable();
      }
    }
  }

  show(event: MouseEvent) {
    this.overlayPanel.show(event);
  }

  hide() {
    this.overlayPanel.hide();
  }

  delete() {
    if (this.formControl.invalid) {
      console.error('Form is invalid!', JSON.stringify(this.formControl.value));
      return;
    }

    const replaceWithStage = this.formControl.value.id;
    this.onDelete.emit(replaceWithStage);
    this.formControl.reset();
  }
}
