import { ChangeDetectionStrategy, Component, EventEmitter, Input, OnInit, Output } from '@angular/core';

import { StageModel } from 'data-access/sdk/internal-api';

interface Column {
  field: string;
  header: string;
}

@Component({
  selector: 'app-stages-table',
  templateUrl: './stages-table.component.html',
  styleUrls: ['./stages-table.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class StagesTableComponent implements OnInit {
  @Input() stages: StageModel[] = [];
  @Input() loading = false;
  @Input() showDelete = false;
  @Input() reorderedCols = true;

  @Output() onRowReorder = new EventEmitter<StageModel[]>();
  @Output() onDeleteSelection = new EventEmitter<{ event: MouseEvent; stage: StageModel }>();
  @Output() onRowClick = new EventEmitter();

  cols: Column[] = [
    { field: 'name', header: 'PROJECT_STAGES.STAGE_NAME' },
    { field: 'status', header: 'PROJECT_STAGES.STAGE_STATUS' },
  ];

  ngOnInit() {
    if (this.showDelete) {
      this.cols.push({ field: 'delete', header: '' });
    }
  }

  rowReorder() {
    this.onRowReorder.emit(this.stages);
  }

  rowClick(stage: StageModel) {
    this.onRowClick.emit(stage);
  }

  delete(event: MouseEvent, stage: StageModel) {
    this.onDeleteSelection.emit({ event: event, stage });
  }
}
