import { ChangeDetectionStrategy, Component, OnInit, ViewChild } from '@angular/core';
import { Select, Store } from '@ngxs/store';
import { Observable } from 'rxjs';
import { Table } from 'primeng/table';

import {Unsubscriber} from 'utils/shared-services';
import {GlobalRulesSelectors, StagesAction, StagesSelectors} from 'data-access/store';
import {StageModel} from 'data-access/sdk/internal-api';

import { CreateStageComponent } from '../components/create-stage/create-stage.component';

@Component({
  selector: 'app-stages',
  templateUrl: './stages.component.html',
  styleUrls: ['./stages.component.scss'],
  providers: [Unsubscriber],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class StagesComponent implements OnInit {
  @ViewChild('table') table: Table;
  @ViewChild('createStageRef') createStageRef: CreateStageComponent;

  @Select(GlobalRulesSelectors.canDelete('stage')) canDelete$: Observable<boolean>;
  @Select(GlobalRulesSelectors.canUpdate('stage')) canUpdate$: Observable<boolean>;

  stages$ = this.store.select(StagesSelectors.stages);
  stagesLoading$ = this.store.select(StagesSelectors.stagesLoading);
  createStageLoading$ = this.store.select(StagesSelectors.createStageLoading);

  constructor(private readonly store: Store) {}

  ngOnInit() {
    this.store.dispatch(new StagesAction.ReadStagesList()).subscribe();
  }

  openEditModal(stage: StageModel) {
    this.store.dispatch(new StagesAction.OpenEditModal(stage.id)).subscribe();
  }

  createStage(stage: Pick<StageModel, 'name' | 'status'>) {
    this.store.dispatch(new StagesAction.CreateStage(stage)).subscribe(() => this.createStageRef.hide());
  }

  updateStages(stages: StageModel[]) {
    this.store.dispatch(new StagesAction.UpdateStagesList(stages)).subscribe();
  }
}
