import {Component, OnInit} from '@angular/core';
import {AuthenticationService} from '@core-project/services/authentication.service';
import {MessageService} from 'primeng/api';
import {OrganizationFormComponent} from '@core-project/modules/forms/organization-form/organization-form.component';
import {DynamicDialogService} from '@core-project/services/dynamic-dialog.service';
import {OrganizationService, Partner, User, UsersService} from 'data-access/sdk/api2';

@Component({
  selector: 'app-organizations',
  templateUrl: './organizations.component.html',
  styleUrls: ['./organizations.component.scss'],
})
export class OrganizationsComponent implements OnInit {
  perms: string[];
  user: User;
  columns: any[] = this.initColumns();
  allOrganizations: Partner[] = [];
  organizations: Partner[];

  constructor(
    private readonly authenticationService: AuthenticationService,
    private readonly usersService: UsersService,
    private readonly messageService: MessageService,
    private readonly organizationService: OrganizationService,
    private readonly dynamicDialogService: DynamicDialogService,
  ) {
  }

  ngOnInit(): void {
    this.authenticationService.user$.subscribe((user) => {
      this.user = user;
      if (user) {
        this.getOrganization().then(() => {
        });
        this.usersService.usersPermissions({id: this.user.id}).subscribe((userPermissions) => {
          this.perms = userPermissions.permissions;
        });
      }
    });
  }

  organizationCreateOrEdit(organization?: Partner) {
    const header = organization ? 'Просмотр данных об организации' : 'Добавление организации';

    this.dynamicDialogService
      .showDialog(OrganizationFormComponent, {organization, header: header, showHeader: true}, '80%')
      .then(() => this.getOrganization());
  }

  private getOrganization(): Promise<void | Partner> {
    return this.organizationService
      .organizationList({
        company: String(this.user.company),
        include: 'contacts,telephones,emails,contacts__telephones,contacts__emails',
      })
      .toPromise()
      .then((organizations) => {
        this.allOrganizations = organizations;
        this.organizations = organizations;
      })
      .catch((err) => {
        this.messageService.add({
          severity: 'error',
          summary: 'Во время загрузки компаний произошла ошибка:',
          detail: err,
        });
      });
  }

  private initColumns() {
    return [
      {
        field: 'name',
        header: 'Название',
        style: {width: '40%'},
      },
      {
        field: 'contacts',
        header: 'Контакты',
        style: {width: '60%'},
      },
    ];
  }
}
