import {ChangeDetectionStrategy, Component, EventEmitter, Input, Output} from '@angular/core';
import {ProjectEnums} from '@core-project/enums/project';
import StatusEnum = ProjectEnums.StatusEnum;

@Component({
  selector: 'app-project-manage-actions',
  templateUrl: './project-manage-actions.component.html',
  styleUrls: ['./project-manage-actions.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class ProjectManageActionsComponent {
  @Output()
  onPause = new EventEmitter<void>();
  @Output()
  onCancel = new EventEmitter<void>();
  @Output()
  onResume = new EventEmitter<void>();
  @Output()
  onDelete = new EventEmitter<void>();

  @Input()
  status: string;
  @Input()
  name: string;

  statuses = StatusEnum;

  get isPauseAvailable(): boolean {
    return this.status !== this.statuses.PSP && this.status !== this.statuses.CNC;
  }

  get isCancelAvailable(): boolean {
    return this.status !== this.statuses.PSP && this.status !== this.statuses.CNC;
  }

  get isResumeAvailable(): boolean {
    return this.status === this.statuses.PSP || this.status === this.statuses.CNC;
  }

  pauseProject(): void {
    this.onPause.emit();
  }

  cancelProject(): void {
    this.onCancel.emit();
  }

  resumeProject(): void {
    this.onResume.emit();
  }

  deleteProject(): void {
    this.onDelete.emit();
  }
}
