import { NgModule } from '@angular/core';
import { CommonModule, NgOptimizedImage } from '@angular/common';
import { ReactiveFormsModule } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';

import { InputTextareaModule } from 'primeng/inputtextarea';
import { MultiSelectModule } from 'primeng/multiselect';
import { OverlayPanelModule } from 'primeng/overlaypanel';
import { PaginatorModule } from 'primeng/paginator';
import { SkeletonModule } from 'primeng/skeleton';
import { TooltipModule } from 'primeng/tooltip';
import { InputTextModule } from 'primeng/inputtext';
import { RippleModule } from 'primeng/ripple';
import { TriStateCheckboxModule } from 'primeng/tristatecheckbox';
import { ButtonModule } from 'primeng/button';
import { ImageModule } from 'primeng/image';
import { CheckboxModule } from 'primeng/checkbox';

import { StringInfoOverlaysModule } from 'features/string-info-overlays';
import { UiCustomCalendarModule } from 'ui/custom-calendar';
import { UiMilestonesTimelineComponent } from 'ui/milestones-timeline';
import { InplaceCalendarInputComponent, InplaceDropdownInputComponent, InplaceStringInputComponent } from 'ui/forms-ui';
import { HasPermissionDirective } from 'utils/shared-directives';

import { DirectivesModule } from '@core-project/directives/directives.module';
import { SharedComponentsModule } from '@core-project/modules/shared-components/shared-components.module';
import { PipesModule } from '@core-project/pipes/pipes.module';
import { ConfirmOverlaySComponent } from '@core-project/modules/b-kit/confirm-delete/confirm-overlay-s.component';

import { ProjectDetailsContainerComponent } from './containers/project-details-container/project-details-container.component';
import { DetailsParticipantsComponent } from './components/details-participants/details-participants.component';
import { DetailsStatusComponent } from './components/details-status/details-status.component';
import { DetailsAttributesComponent } from './components/details-attributes/details-attributes.component';
import { DetailsBudgetComponent } from './components/details-budget/details-budget.component';
import { ProjectDetailsService } from './services/project-details.service';
import { InplaceEditorComponent } from './components/inplace-editor/inplace-editor.component';
import { ProjectManageActionsComponent } from './components/project-manage-actions/project-manage-actions.component';
import { ProjectInfoFormContainerComponent } from './containers/project-info-form-container/project-info-form-container.component';
import { UserInfoComponent } from './components/user-info/user-info.component';
import { ProjectDetailsRoutingModule } from './project-details-routing.module';
import { AttributesFormComponent } from './components/forms/attributes-form/attributes-form.component';

// primeNgModules
const primengModules = [
  MultiSelectModule,
  PaginatorModule,
  InputTextareaModule,
  ImageModule,
  InputTextModule,
  RippleModule,
  SkeletonModule,
  TooltipModule,
  OverlayPanelModule,
  TriStateCheckboxModule,
  ButtonModule,
  TriStateCheckboxModule,
  CheckboxModule,
];

@NgModule({
  declarations: [
    ProjectDetailsContainerComponent,
    DetailsParticipantsComponent,
    DetailsStatusComponent,
    DetailsAttributesComponent,
    DetailsBudgetComponent,
    InplaceEditorComponent,
    ProjectManageActionsComponent,
    ProjectInfoFormContainerComponent,
    UserInfoComponent,
    AttributesFormComponent,
  ],
  providers: [ProjectDetailsService],
  imports: [
    ...primengModules,
    ProjectDetailsRoutingModule,
    CommonModule,
    ReactiveFormsModule,
    TranslateModule,
    UiCustomCalendarModule,
    HasPermissionDirective,
    DirectivesModule,
    SharedComponentsModule,
    StringInfoOverlaysModule,
    ConfirmOverlaySComponent,
    NgOptimizedImage,
    PipesModule,
    UiMilestonesTimelineComponent,
    InplaceCalendarInputComponent,
    InplaceStringInputComponent,
    InplaceDropdownInputComponent,
  ],
})
export class ProjectDetailsModule {}
