import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { ProjectBudget, ProjectBudgetType } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-budget-approval',
  templateUrl: './project-budget-approval.component.html',
  styleUrls: ['./project-budget-approval.component.scss'],
})
export class ProjectBudgetApprovalComponent implements OnInit {
  projectBudget: ProjectBudget;
  projectBudgetTypes: Array<ProjectBudgetType>;

  pbForm: FormGroup;
  pbtForms: Array<FormGroup>;

  constructor(private ref: DynamicDialogRef, private config: DynamicDialogConfig, private formBuilder: FormBuilder) {}

  ngOnInit(): void {
    this.projectBudget = this.config.data.projectBudget;
    this.projectBudgetTypes = this.config.data.projectBudgetTypes;
    this.initForms();
  }

  initForms() {
    this.pbForm = this.formBuilder.group({
      id: [this.projectBudget.id],
      code: [this.projectBudget.code, Validators.required],
    });

    this.pbtForms = this.projectBudgetTypes.map((pbt) =>
      this.formBuilder.group({
        id: [pbt.id],
        code: [pbt.code, Validators.required],
        name: [pbt.name],
      })
    );
  }

  save() {
    this.ref.close({ pb: this.pbForm.value, pbts: this.pbtForms.map((pbtf) => pbtf.value) });
  }
}
