import { Component, OnInit } from '@angular/core';
import { FinanceService } from 'data-access/sdk/api2';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { ProjectBudget } from 'data-access/sdk/api2';
import { FormBuilder, FormGroup } from '@angular/forms';
import { firstValueFrom } from 'rxjs';

@Component({
  selector: 'app-project-budget-params',
  templateUrl: './project-budget-params.component.html',
})
export class ProjectBudgetParamsComponent implements OnInit {
  private projectBudget: ProjectBudget;
  form: FormGroup;

  constructor(
    private financeService: FinanceService,
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef,
    private fb: FormBuilder
  ) {}

  ngOnInit() {
    this.projectBudget = this.config.data.projectBudget;
    this.initForm();
  }

  initForm() {
    this.form = this.fb.group({
      total_planned: [this.projectBudget.total_planned],
      total_forecast: [this.projectBudget.total_forecast],
      total_fact: [this.projectBudget.total_fact],
      effect_plan: [this.projectBudget.effect_plan],
      effect_forecast: [this.projectBudget.effect_forecast],
      effect_fact: [this.projectBudget.effect_fact],
      irr_plan: [this.projectBudget.irr_plan],
      irr_forecast: [this.projectBudget.irr_forecast],
      irr_fact: [this.projectBudget.irr_fact],
      npv_plan: [this.projectBudget.npv_plan],
      npv_forecast: [this.projectBudget.npv_forecast],
      npv_fact: [this.projectBudget.npv_fact],
    });

    if (this.projectBudget.approved) {
      this.form.controls.effect_plan.disable();
      this.form.controls.irr_plan.disable();
      this.form.controls.npv_plan.disable();
    }
  }

  save() {
    firstValueFrom(this.financeService.financeProjectBudgetsPartialUpdate({ id: this.projectBudget.id, data: this.form.value })).then(
      (pb) => this.ref.close(pb)
    );
  }
}
