import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import * as moment from 'moment';
import { ProjectBudget } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-budget',
  templateUrl: './project-budget.component.html',
  styleUrls: ['./project-budget.component.scss'],
})
export class ProjectBudgetComponent implements OnInit {
  public projectBudget: ProjectBudget;
  public projectBudgetForm: FormGroup;

  constructor(
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly dynamicDialogConfig: DynamicDialogConfig,
    private readonly formBuilder: FormBuilder
  ) {}

  static checkDatesValidator(group: FormGroup) {
    return moment(group.controls.period_start.value, 'DD.MM.YYYY') < moment(group.controls.period_end.value, 'DD.MM.YYYY')
      ? null
      : { period: true };
  }

  ngOnInit(): void {
    this.projectBudget = this.dynamicDialogConfig.data.pb;
    this.initPbForm(this.projectBudget);
  }

  initPbForm(pb: ProjectBudget) {
    this.projectBudgetForm = this.formBuilder.group(
      {
        id: [pb.id, Validators.required],
        project: [pb.project, Validators.required],
        code: [pb.code],
        period_start: [moment(pb.period_start).format('DD.MM.YYYY'), Validators.required],
        period_end: [moment(pb.period_end).format('DD.MM.YYYY'), Validators.required],
        total_planned: [pb.total_planned, Validators.required],
        total_planned_currency: [pb.total_planned_currency, Validators.required],
      },
      { validators: ProjectBudgetComponent.checkDatesValidator }
    );
  }

  save() {
    this.dynamicDialogRef.close(this.projectBudgetForm.value);
  }
}
