import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ProjectBudgetType } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-finance-add-budget-type',
  templateUrl: './project-finance-add-budget-type.component.html',
  styleUrls: ['./project-finance-add-budget-type.component.scss'],
})
export class ProjectFinanceAddBudgetTypeComponent implements OnInit {
  public choosableTypes: Array<any>;
  public code: string;

  public bt: ProjectBudgetType;
  public btForm: FormGroup;
  public currency: string;
  private projectId: number;
  private totalBudget: number;

  constructor(public ref: DynamicDialogRef, public config: DynamicDialogConfig, private formBuilder: FormBuilder) {}

  get maxPlanned() {
    return this.totalBudget - this.config.data.projectBudgetTypes.reduce((total, cv) => total + Number(cv.total_planned), 0);
  }

  ngOnInit(): void {
    const exTypes: Array<number> = [];
    this.bt = this.config.data.bt;
    this.projectId = this.config.data.projectId;
    this.currency = this.config.data.currency || 'RUB';
    this.totalBudget = this.config.data.totalBudget;
    this.config.data.projectBudgetTypes.map((pbt) => exTypes.push(pbt.budget_type));
    this.choosableTypes = this.config.data.budgetTypesIds
      .filter((bt) => !exTypes.includes(bt.id))
      .map((bt) => ({ id: bt.id, name: bt.rel_category.name + ' - ' + bt.name, code: bt.code }));
    this.initForm(this.bt);
  }

  save(): void {
    // this.ref.close({budget_type: this.chosenType, code: this.code});
    this.ref.close(this.btForm.value);
  }

  cancel(): void {
    this.ref.close();
  }

  initForm(bt: ProjectBudgetType) {
    this.btForm = this.formBuilder.group({
      id: [bt?.id],
      project: [this.projectId, Validators.required],
      budget_type: [bt?.budget_type, Validators.required],
      code: [bt?.code],
      total_planned: [bt?.total_planned, Validators.compose([Validators.min(0), Validators.max(this.maxPlanned)])],
    });

    this.btForm.controls.budget_type.valueChanges.subscribe((newBt) => {
      if (!this.btForm.controls.code.value) {
        const chosenBt = this.choosableTypes.find((ct) => ct.id === newBt);
        this.btForm.controls.code.setValue(chosenBt.code);
      }
    });
  }

  planInput($event) {
    this.btForm.controls.total_planned.setValue($event.fn);
  }

  maxPlan() {
    this.btForm.controls.total_planned.setValue(this.maxPlanned);
  }
}
