import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Observable, of } from 'rxjs';
import { DialogService } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import { PermissionObjectType, UserPermissionsService } from '@core-project/services/user-permissions.service';
import { map, switchMap } from 'rxjs/operators';
import { FinanceService, ProjectsService } from 'data-access/sdk/api2';
import { Project, ProjectBudget } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-finance',
  templateUrl: './project-finance.component.html',
  styleUrls: ['./project-finance.component.scss'],
  providers: [DialogService],
})
export class ProjectFinanceComponent implements OnInit {
  public projectId: number;
  public project$: Observable<Project>;

  public switcherOptions;
  public switchSelected = 'general';

  public extensionOptions;
  public extension = 'auto';

  public perms$: Observable<Array<string>>;

  public project: Project;
  public projectBudget: ProjectBudget;

  public load$: Observable<any>;

  constructor(
    private route: ActivatedRoute,
    private projectsService: ProjectsService,
    private financeService: FinanceService,
    public dialogService: DialogService,
    public translateService: TranslateService,
    private userPermissionsService: UserPermissionsService
  ) {}

  ngOnInit() {
    this.projectId = Number(this.route.parent.snapshot.paramMap.get('projectId'));
    this.project$ = this.projectsService.projectsRead({ id: this.projectId, include: 'project_budget' });
    this.perms$ = this.userPermissionsService.getUserObjectPermissions(PermissionObjectType.project, this.projectId);
    this.switcherOptions = [
      { name: this.translateService.instant('FINANCE.SWITCH.GENERAL'), value: 'general' },
      { name: this.translateService.instant('FINANCE.SWITCH.TABLE'), value: 'table' },
      { name: this.translateService.instant('FINANCE.SWITCH.EXPENSES'), value: 'expenses' },
    ];

    this.extensionOptions = [
      { name: this.translateService.instant('FINANCE.EXTENSIONS.NONE'), value: 'none' },
      { name: this.translateService.instant('FINANCE.EXTENSIONS.AUTO'), value: 'auto' },
      { name: this.translateService.instant('FINANCE.EXTENSIONS.THOUSANDS'), value: 'thousands' },
      { name: this.translateService.instant('FINANCE.EXTENSIONS.MILLIONS'), value: 'millions' },
      { name: this.translateService.instant('FINANCE.EXTENSIONS.BILLIONS'), value: 'billions' },
    ];
    this.load$ = this.loadData$();
  }

  private loadData$(): Observable<any> {
    return this.projectsService
      .projectsRead({
        id: this.projectId,
        include: 'project_budget',
      })
      .pipe(
        switchMap((project: Project) => {
          this.project = project;
          // return this.financeService.financeProjectBudgetsList(String(this.projectId));
          return of(project.rel_project_budget);
        }),
        switchMap((pbs) => {
          this.projectBudget = pbs;
          if (!this.projectBudget) {
            return this.financeService
              .financeProjectBudgetsCreate({
                project: this.projectId,
              })
              .pipe(
                map((pb) => {
                  this.projectBudget = pb;
                  return this.loadData$();
                })
              );
          }
          return of(this.projectBudget);
        })
      );
  }
}
