import {Component, OnInit} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {TranslateService} from '@ngx-translate/core';
import {Actions, ofActionSuccessful, Store} from '@ngxs/store';
import {firstValueFrom} from 'rxjs';
import {takeUntil} from 'rxjs/operators';
import {DialogService} from 'primeng/dynamicdialog';

import {ChoicefieldInterface} from '@core-project/interfaces';

import {Issue, IssuesService, Project, ProjectsService} from 'data-access/sdk/api2';
import {IssuesActions} from 'data-access/store';
import {Unsubscriber} from 'utils/shared-services';

@Component({
  selector: 'app-project-risks',
  templateUrl: './project-risks.component.html',
  styleUrls: ['./project-risks.component.scss'],
  providers: [DialogService, Unsubscriber],
})
export class ProjectRisksComponent implements OnInit {
  private projectId: number;

  private readonly updateObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(IssuesActions.UpdateIssue, IssuesActions.DeleteIssue, IssuesActions.CreateIssue),
    )
    .subscribe(() => {
      this.loadData();
    });

  project: Project;
  issues: Array<Issue>;
  onlyActive = true;
  selectOptions: ChoicefieldInterface[];

  constructor(
    private readonly activatedRoute: ActivatedRoute,
    private readonly projectService: ProjectsService,
    private readonly issuesService: IssuesService,
    private readonly translateService: TranslateService,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
  ) {
  }

  ngOnInit() {
    this.selectOptions = [
      {label: this.translateService.instant('ISSUES.SHOW_ONLY_ACTIVE'), value: true},
      {label: this.translateService.instant('ISSUES.SHOW_ALL'), value: false},
    ];
    this.projectId = Number(this.activatedRoute.parent.snapshot.paramMap.get('projectId'));
    this.projectService.projectsRead({id: this.projectId}).subscribe((project) => (this.project = project));
    this.loadData();

  }

  addIssue() {
    this.store.dispatch(new IssuesActions.ShowCreateIssueInSidebar(this.projectId)).subscribe();
  }

  private loadData(): void {
    firstValueFrom(this.issuesService.issuesIssuesList({project: this.projectId.toString(), include: 'domains, works__to_user'})).then(
      (issues) => {
        this.issues = issues;
      },
    );
  }
}
