import {NgModule} from '@angular/core';
import {RouterModule, Routes} from '@angular/router';
import {ProjectComponent} from './project.component';
import {AuthGuard} from '@core-project/auth/guards';
import {ProjectTasksComponent} from './project-tasks/project-tasks.component';
import {ProjectCalendarComponent} from './project-calendar/project-calendar.component';
import {ProjectStakeholdersComponent} from './project-stakeholders/project-stakeholders.component';
import {ProjectTeamComponent} from './project-team/project-team.component';
import {ProjectProcurementsComponent} from './project-procurements/project-procurements.component';
import {ProjectFinanceComponent} from './project-finance/project-finance.component';
import {ProjectRisksComponent} from './project-risks/project-risks.component';
import {ProjectDocumentsComponent} from './project-documents/project-documents.component';
import {ProjectPlanComponent} from './project-plan/project-plan.component';
import {getProjectPermissionsGuard, resetProjectPermissionsGuard} from 'data-access/store';

const projectRoutes: Routes = [
  {
    path: 'project/:projectId',
    component: ProjectComponent,
    canActivate: [AuthGuard, getProjectPermissionsGuard],
    canDeactivate: [resetProjectPermissionsGuard],
    children: [
      {path: 'details', loadChildren: () => import('./project-details/project-details.module').then(m => m.ProjectDetailsModule)},
      {path: 'plan', component: ProjectPlanComponent},
      {path: 'tasks', component: ProjectTasksComponent},
      {path: 'documents', component: ProjectDocumentsComponent},
      {path: 'issues', component: ProjectRisksComponent},
      {path: 'finance', component: ProjectFinanceComponent},
      {path: 'procurements', component: ProjectProcurementsComponent},
      {
        path: 'team',
        component: ProjectTeamComponent,
        children: [
          {
            path: '',
            pathMatch: 'full',
            redirectTo: 'structure',
          },
        ],
      },
      {path: 'stakeholders', component: ProjectStakeholdersComponent},
      {path: 'calendar', component: ProjectCalendarComponent},
    ],
  },
];

@NgModule({
  declarations: [],
  imports: [RouterModule.forChild(projectRoutes)],
  exports: [RouterModule],
})
export class ProjectRoutingModule {
}
