import {Component, OnInit} from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import {MenuItem} from 'primeng/api';
import {ActivatedRoute} from '@angular/router';
import {Project, ProjectsService} from 'data-access/sdk/api2';
import {Store} from '@ngxs/store';
import {GlobalActions} from 'data-access/store';
import {RouterStateService} from './services/router-state.service';

@Component({
  selector: 'app-project',
  templateUrl: './project.component.html',
  styleUrls: ['./project.component.scss'],
})
export class ProjectComponent implements OnInit {
  projectId: number;
  project: Project;

  constructor(
    private readonly route: ActivatedRoute,
    private readonly projectsService: ProjectsService,
    private readonly translateService: TranslateService,
    private readonly store: Store,
    private routerStateService: RouterStateService,
  ) {
  }

  ngOnInit() {
    this.routerStateService.params$ = this.route.params;

    this.route.params.subscribe((params) => {
      this.projectId = params.projectId;
      this.projectsService.projectsRead({id: this.projectId}).subscribe((project) => {
        this.project = project;

        this.store.dispatch(new GlobalActions.SetAppTitle({key: this.project.name}));

        const items: MenuItem[] = [
          {id: 'info', label: 'Информация', routerLink: ['/project', this.project.id, 'details'], icon: 'info-icon'},
          {
            id: 'tasks',
            label: this.translateService.instant('PROJECT.WORKS'),
            routerLink: ['/project', this.project.id, 'tasks'],
            icon: 'works',
          },
          {
            id: 'plan',
            label: this.translateService.instant('GANTT.HEADER'),
            routerLink: ['/project', this.project.id, 'plan'],
            icon: 'plans',
          },
          {
            id: 'docs',
            label: this.translateService.instant('DOCUMENTS.HEADER'),
            routerLink: ['/project', this.project.id, 'documents'],
            icon: 'documents',
          },
          {
            id: 'risks',
            label: this.translateService.instant('ISSUES.HEADER_PROBLEMS'),
            routerLink: ['/project', this.project.id, 'issues'],
            icon: 'problems',
          },
          {
            id: 'finance',
            label: this.translateService.instant('FINANCE.PROJECT.BUDGET'),
            routerLink: ['/project', this.project.id, 'finance'],
            icon: 'budget',
          },
          {
            id: 'team',
            label: this.translateService.instant('PROJECT.TEAM.HEADER'),
            routerLink: ['/project', this.project.id, 'team'],
            icon: 'contacts',
          },
        ];

        this.store.dispatch(new GlobalActions.SetTopMenuItems(items));
      });
    });
  }

  ngOnDestroy(): void {
    this.store.dispatch(new GlobalActions.SetTopMenuItems([]));
  }
}
