import {NgModule} from '@angular/core';
import {CommonModule} from '@angular/common';
import {ScrollingModule} from '@angular/cdk/scrolling';
import {TranslateModule} from '@ngx-translate/core';
import {RouterModule} from '@angular/router';

// primeng
import {FieldsetModule} from 'primeng/fieldset';
import {OverlayPanelModule} from 'primeng/overlaypanel';
import {CalendarModule} from 'primeng/calendar';
import {SpinnerModule} from 'primeng/spinner';
import {DialogModule} from 'primeng/dialog';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {TabMenuModule} from 'primeng/tabmenu';
import {TreeModule} from 'primeng/tree';
import {CardModule} from 'primeng/card';
import {DividerModule} from 'primeng/divider';
import {RippleModule} from 'primeng/ripple';
import {DragDropModule} from 'primeng/dragdrop';
import {InputTextareaModule} from 'primeng/inputtextarea';
import {SliderModule} from 'primeng/slider';
import {InputNumberModule} from 'primeng/inputnumber';
import {MultiSelectModule} from 'primeng/multiselect';
import {SelectButtonModule} from 'primeng/selectbutton';
import {InputSwitchModule} from 'primeng/inputswitch';
import {TableModule} from 'primeng/table';
import {ButtonModule} from 'primeng/button';
import {DynamicDialogModule} from 'primeng/dynamicdialog';
import {DropdownModule} from 'primeng/dropdown';
import {InputTextModule} from 'primeng/inputtext';
import {ScrollPanelModule} from 'primeng/scrollpanel';
import {BadgeModule} from 'primeng/badge';
import {KnobModule} from 'primeng/knob';
import {TimelineModule} from 'primeng/timeline';
import {AccordionModule} from 'primeng/accordion';
import {EditorModule} from 'primeng/editor';
import {SkeletonModule} from 'primeng/skeleton';

// modules
import {ProjectRoutingModule} from './project-routing.module';
import {NewsfeedModule} from '../newsfeed/newsfeed.module';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {ProjectFormModule} from '@core-project/modules/forms/project-form/project-form.module';
import {ProjectViewFormModule} from '@core-project/modules/forms/project-view-form/project-view-form.module';
import {ProjectMetaFormModule} from '@core-project/modules/forms/project-meta-form/project-meta-form.module';
import {ProjectExpenseFormModule} from '@core-project/modules/forms/project-expense-form/project-expense-form.module';
import {PipesModule} from '@core-project/pipes/pipes.module';
import {ProjectMetaValuesFormModule} from '@core-project/modules/forms/project-meta-values-form/project-meta-values-form.module';
import {ProjectMemberFormModule} from '@core-project/modules/forms/project-member-form/project-member-form.module';
import {ProjectStateFormModule} from '@core-project/modules/forms/project-state-form/project-state-form.module';
import {SharedComponentsModule} from '@core-project/modules/shared-components/shared-components.module';
import {DirectivesModule} from '@core-project/directives/directives.module';

// components
import {ProjectComponent} from './project.component';
import {ProjectDetailsOldComponent} from './project-details-old/project-details-old.component';
import {ProjectTasksComponent} from './project-tasks/project-tasks.component';
import {ProjectPlanComponent} from './project-plan/project-plan.component';
import {ProjectDocumentsComponent} from './project-documents/project-documents.component';
import {ProjectRisksComponent} from './project-risks/project-risks.component';
import {ProjectFinanceComponent} from './project-finance/project-finance.component';
import {ProjectProcurementsComponent} from './project-procurements/project-procurements.component';
import {ProjectTeamComponent} from './project-team/project-team.component';
import {ProjectStakeholdersComponent} from './project-stakeholders/project-stakeholders.component';
import {ProjectCalendarComponent} from './project-calendar/project-calendar.component';
import {ProjectFinanceTableComponent} from './project-finance/project-finance-table/project-finance-table.component';
import {ProjectFinanceAddBudgetTypeComponent} from './project-finance/project-finance-add-budget-type/project-finance-add-budget-type.component';
import {ProjectBudgetItemComponent} from './project-finance/project-budget-item/project-budget-item.component';
import {ProjectBudgetComponent} from './project-finance/project-budget/project-budget.component';
import {ProjectExpensesComponent} from './project-finance/project-expenses/project-expenses.component';
import {ProjectFinanceGeneralComponent} from './project-finance/project-finance-general/project-finance-general.component';
import {ProjectMonthInfoComponent} from './project-finance/project-month-info/project-month-info.component';
import {ProjectBudgetApprovalComponent} from './project-finance/project-budget-approval/project-budget-approval.component';
import {ProjectBudgetParamsComponent} from './project-finance/project-budget-params/project-budget-params.component';

import {ConfirmOverlaySComponent} from '@core-project/modules/b-kit/confirm-delete/confirm-overlay-s.component';
import {StringInfoOverlaysModule} from 'features/string-info-overlays';
import {HasPermissionDirective} from 'utils/shared-directives';
import {ProjectDetailsModule} from './project-details/project-details.module';
import {RouterStateService} from '@core-project/containers/project/services/router-state.service';
import { KanbanBoardComponent } from "../../../../../../libs/ui/kanban-board/src/lib/components/kanban-board/kanban-board.component";

const primeNgModules = [
  FieldsetModule,
  OverlayPanelModule,
  CalendarModule,
  SpinnerModule,
  DialogModule,
  ConfirmDialogModule,
  TabMenuModule,
  TreeModule,
  CardModule,
  DividerModule,
  RippleModule,
  DragDropModule,
  InputTextareaModule,
  SliderModule,
  InputNumberModule,
  MultiSelectModule,
  SelectButtonModule,
  InputSwitchModule,
  TableModule,
  ButtonModule,
  DynamicDialogModule,
  DropdownModule,
  InputTextModule,
  ScrollPanelModule,
  BadgeModule,
  KnobModule,
  TimelineModule,
  AccordionModule,
  EditorModule,
  SkeletonModule,
];

@NgModule({
    declarations: [
        ProjectComponent,
        ProjectDetailsOldComponent,
        ProjectTasksComponent,
        ProjectPlanComponent,
        ProjectDocumentsComponent,
        ProjectRisksComponent,
        ProjectFinanceComponent,
        ProjectProcurementsComponent,
        ProjectTeamComponent,
        ProjectStakeholdersComponent,
        ProjectCalendarComponent,
        ProjectFinanceTableComponent,
        ProjectFinanceAddBudgetTypeComponent,
        ProjectBudgetItemComponent,
        ProjectBudgetComponent,
        ProjectExpensesComponent,
        ProjectFinanceGeneralComponent,
        ProjectMonthInfoComponent,
        ProjectBudgetApprovalComponent,
        ProjectBudgetParamsComponent,
    ],
    providers: [RouterStateService],
    imports: [
        ...primeNgModules,
        CommonModule,
        RouterModule,
        TranslateModule,
        ScrollingModule,
        FormsModule,
        ReactiveFormsModule,
        PipesModule,
        DirectivesModule,
        SharedComponentsModule,
        NewsfeedModule,
        ProjectRoutingModule,
        ProjectFormModule,
        ProjectViewFormModule,
        ProjectMetaFormModule,
        ProjectExpenseFormModule,
        ProjectMetaValuesFormModule,
        ProjectMemberFormModule,
        ProjectStateFormModule,
        ConfirmOverlaySComponent,
        HasPermissionDirective,
        StringInfoOverlaysModule,
        ProjectDetailsModule,
        KanbanBoardComponent
    ]
})
export class ProjectModule {
}
