import {NgModule} from '@angular/core';
import {CommonModule} from '@angular/common';
import {ProjectsComponent} from './projects.component';
import {ProjectsRoutingModule} from './projects-routing.module';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {ProjectFormModule} from '@core-project/modules/forms/project-form/project-form.module';

import {DirectivesModule} from '@core-project/directives/directives.module';
import {ProjectsRoadmapComponent} from './projects-roadmap/projects-roadmap.component';
import {ProjectsStructureComponent} from './projects-structure/projects-structure.component';
import {RouterModule} from '@angular/router';
import {TranslateModule} from '@ngx-translate/core';
import {SharedComponentsModule} from '@core-project/modules/shared-components/shared-components.module';
import {PipesModule} from '@core-project/pipes/pipes.module';

// primeng
import {TableModule} from 'primeng/table';
import {ButtonModule} from 'primeng/button';
import {DialogModule} from 'primeng/dialog';
import {DropdownModule} from 'primeng/dropdown';
import {FieldsetModule} from 'primeng/fieldset';
import {CalendarModule} from 'primeng/calendar';
import {InputMaskModule} from 'primeng/inputmask';
import {InputSwitchModule} from 'primeng/inputswitch';
import {InputTextModule} from 'primeng/inputtext';
import {ListboxModule} from 'primeng/listbox';
import {MultiSelectModule} from 'primeng/multiselect';
import {OverlayPanelModule} from 'primeng/overlaypanel';
import {SpinnerModule} from 'primeng/spinner';
import {TooltipModule} from 'primeng/tooltip';
import {TriStateCheckboxModule} from 'primeng/tristatecheckbox';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {RippleModule} from 'primeng/ripple';
import {SidebarModule} from 'primeng/sidebar';
import {BadgeModule} from 'primeng/badge';
import {InputNumberModule} from 'primeng/inputnumber';
import {SplitButtonModule} from 'primeng/splitbutton';
import {CheckboxModule} from 'primeng/checkbox';
import {AccordionModule} from 'primeng/accordion';
import {SkeletonModule} from 'primeng/skeleton';
import {InputTextareaModule} from 'primeng/inputtextarea';
import {ScrollPanelModule} from 'primeng/scrollpanel';
import {TableResizeTrDirective} from '@core-project/modules/ui-kit/table-resize/table-resize-tr.directive';
import {HasPermissionDirective} from 'utils/shared-directives';
import {UiCustomCalendarModule} from 'ui/custom-calendar';

const primeNgModules = [
  TableModule,
  ButtonModule,
  DialogModule,
  DropdownModule,
  FieldsetModule,
  CalendarModule,
  InputMaskModule,
  InputSwitchModule,
  InputTextModule,
  ListboxModule,
  MultiSelectModule,
  OverlayPanelModule,
  SpinnerModule,
  TooltipModule,
  TriStateCheckboxModule,
  ConfirmDialogModule,
  RippleModule,
  SidebarModule,
  BadgeModule,
  InputNumberModule,
  SplitButtonModule,
  CheckboxModule,
  AccordionModule,
  SkeletonModule,
  InputTextareaModule,
  ScrollPanelModule,
];

@NgModule({
  declarations: [ProjectsComponent, ProjectsRoadmapComponent, ProjectsStructureComponent],
  imports: [
    ...primeNgModules,
    CommonModule,
    ProjectsRoutingModule,
    ReactiveFormsModule,
    RouterModule,
    ProjectFormModule,
    DirectivesModule,
    FormsModule,
    TranslateModule,
    SharedComponentsModule,
    PipesModule,
    TableResizeTrDirective,
    HasPermissionDirective,
    UiCustomCalendarModule,
  ],
})
export class ProjectsModule {
}
