import * as moment from 'moment/moment';

export namespace Dates {
  export const startQuarterMapper = {
    1: 3,
    2: 2,
    0: 1,
  };

  export const endQuarterMapper = {
    1: 1,
    2: 2,
    0: 3,
  };
}

export function getQuarterByMonth(month: number): number {
  if (month > 12 || month < 0) {
    return 0;
  }
  if (month <= 3) {
    return 1;
  }
  if (month >= 4 && month <= 6) {
    return 2;
  }
  if (month >= 7 && month <= 9) {
    return 3;
  }
  if (month >= 10 && month <= 12) {
    return 4;
  }
}

export function getMonthFromDate(date: moment.Moment | string) {
  if (typeof date === 'string') {
    date = moment(date);
  }
  return date.month();
}

export function getYearFromDate(date: moment.Moment | string) {
  if (typeof date === 'string') {
    date = moment(date);
  }
  return date.year();
}

export function jsDateToDbString(date: Date): string {
  if (date === null || date === undefined) {
    return null;
  }
  return momentToDbString(moment(date));
}

export function dbStringToJsDate(date: string): Date {
  if (date === null || date === undefined || date === '') {
    return null;
  }
  return dbStringToMoment(date).toDate();
}

export function dbStringToMoment(date: string): moment.Moment {
  if (date === null || date === undefined || date === '') {
    return null;
  }
  return moment(date, 'YYYY-MM-DD');
}

export function momentToDbString(date: moment.Moment): string {
  if (date === null || date === undefined) {
    return null;
  }
  return date.format('YYYY-MM-DD');
}
