import { gantt } from '../../assets/dhtmlxgantt/dhtmlxgantt';
import * as moment from 'moment/moment';

export function getGanttIdFromUserId(id: number): string {
  return 'user_' + id.toString();
}

export function getGanttIdFromDepartmentId(id: number): string {
  return 'dep_' + id.toString();
}

export function getGanttIdFromProjectId(id: number): string {
  return 'project_' + id.toString();
}

export function getGanttIdFromProjectGroupId(id: number): string {
  return 'pg_' + id.toString();
}

export const defaultScales = {
  day: [
    {
      unit: 'week',
      step: 1,
      format: (date) => {
        const endDate = gantt.date.add(date, 6, 'day');
        return moment(date).format('D MMM') + ' - ' + moment(endDate).format('D MMM');
      },
    },
    { unit: 'day', step: 1, format: '%j' },
  ],
  week: [
    { unit: 'month', step: 1, format: (date) => moment(date).format('MMMM YYYY') },
    {
      unit: 'week',
      format: (date) => {
        const endDate = gantt.date.add(date, 6, 'day');
        return moment(date).format('D') + ' - ' + moment(endDate).format('D');
      },
    },
  ],
  month: [
    { unit: 'quarter', step: 1, format: (date) => 'Q' + moment(date).format('Q YYYY') },
    { unit: 'month', step: 1, format: (date) => moment(date).format('MMMM') },
  ],
  quarter: [
    { unit: 'year', step: 1, format: '%Y' },
    { unit: 'quarter', step: 1, format: (date) => 'Q' + moment(date).format('Q') },
  ],
  year: [{ unit: 'year', step: 1, format: '%Y' }],
};

export const templateScales = {
  day: [
    {
      unit: 'week',
      step: 1,
      format: 'W%W',
    },
    { unit: 'day', step: 1, format: (date) => moment(date).format('DDD') },
  ],
  week: [
    { unit: 'month', step: 1, format: (date) => moment(date).format('M') },
    {
      unit: 'week',
      step: 1,
      format: 'W%W',
    },
  ],
  month: [
    { unit: 'quarter', step: 1, format: (date) => 'Q' + moment(date).format('Q') },
    { unit: 'month', step: 1, format: (date) => moment(date).format('M') },
  ],
};

export function getGanttTreeIcon(icon: string): string {
  return `<div class="gantt_tree_icon text-center"><i class="${icon}"></i></div>`;
}
