import { Component, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { OverlayPanel, OverlayPanelModule } from 'primeng/overlaypanel';
import { Observable } from 'rxjs';
import { ButtonModule } from 'primeng/button';
import { RippleModule } from 'primeng/ripple';
import { InputTextModule } from 'primeng/inputtext';
import { AbstractControl, FormControl, ReactiveFormsModule, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';

@Component({
  selector: 'app-confirm-delete',
  templateUrl: './confirm-overlay-s.component.html',
  styleUrls: ['./confirm-overlay-s.component.scss'],
  standalone: true,
  imports: [ButtonModule, RippleModule, InputTextModule, OverlayPanelModule, ReactiveFormsModule, TranslateModule],
})
export class ConfirmOverlaySComponent {
  @Input() confirmWord!: string;
  @Output() onSuccess: EventEmitter<void> = new EventEmitter<void>();

  @ViewChild('overlayPanel') overlayPanel: OverlayPanel;

  confirmWordFC: FormControl<string> = new FormControl<string>('', [Validators.required, this.validator()]);

  toggle(event: any, target?: any) {
    this.overlayPanel.toggle(event, target);
  }

  clickSuccess() {
    this.overlayPanel.hide();
    this.onSuccess.emit();
  }

  private validator(): ValidatorFn {
    return (control: AbstractControl): ValidationErrors | null => {
      if (control.value !== this.confirmWord) {
        return { wrongWord: true };
      }

      return null;
    };
  }
}
