import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { CompaniesService } from 'data-access/sdk/api2';
import { Company } from 'data-access/sdk/api2';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { firstValueFrom } from 'rxjs';

@Component({
  selector: 'app-company-form',
  templateUrl: './company-form.component.html',
  styleUrls: ['./company-form.component.scss'],
})
export class CompanyFormComponent implements OnInit {
  private company: Company;
  public form: FormGroup;
  public saveLoading = false;

  constructor(
    private ref: DynamicDialogRef,
    private config: DynamicDialogConfig,
    private companiesService: CompaniesService,
    private fb: FormBuilder
  ) {}

  ngOnInit() {
    this.company = this.config.data.company;
    this.initForm();
  }

  private initForm() {
    this.form = this.fb.group({
      name: ['', Validators.required],
      full_name: [''],
      postal_address: [''],
      legal_address: [''],
    });

    this.form.patchValue(this.company);
  }

  public save() {
    this.saveLoading = true;
    firstValueFrom(this.companiesService.companiesPartialUpdate({ id: this.company.id, data: this.form.value }))
      .then((c) => this.ref.close(c))
      .catch((_c) => (this.saveLoading = false));
  }
}
