import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Dashboard } from 'data-access/sdk/api2';
import { DashboardsService } from 'data-access/sdk/api2';
import { firstValueFrom } from 'rxjs';

@Component({
  selector: 'app-dashboard-form',
  templateUrl: './dashboard-form.component.html',
  styleUrls: ['./dashboard-form.component.scss'],
})
export class DashboardFormComponent implements OnInit {
  private dashboard: Dashboard;
  public form: FormGroup;

  constructor(
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef,
    private fb: FormBuilder,
    private dashboardsService: DashboardsService
  ) {}

  ngOnInit() {
    this.dashboard = this.config.data.dashboard;
    this.initForm();
  }

  private initForm() {
    this.form = this.fb.group({
      name: [this.dashboard?.name, Validators.required],
    });
  }

  public save() {
    if (this.form.valid) {
      const obs$ = this.dashboard?.id
        ? this.dashboardsService.dashboardsDashboardsPartialUpdate({ id: this.dashboard.id, data: this.form.value })
        : this.dashboardsService.dashboardsDashboardsCreate(this.form.value);

      firstValueFrom(obs$).then((d) => this.ref.close(d));
    }
  }
}
