import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DepartmentsService } from 'data-access/sdk/api2';
import { Department } from 'data-access/sdk/api2';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { CompaniesService } from 'data-access/sdk/api2';
import { AuthenticationService } from '@core-project/services/authentication.service';

@Component({
  selector: 'app-department-form',
  templateUrl: './department-form.component.html',
  styleUrls: ['./department-form.component.scss'],
})
export class DepartmentFormComponent implements OnInit {
  form: FormGroup;
  department: Department;
  departmentChoices$: Observable<Array<Department>>;

  constructor(
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef,
    private formBuilder: FormBuilder,
    private companiesService: CompaniesService,
    private authenticationService: AuthenticationService
  ) {}

  ngOnInit(): void {
    this.department = this.config.data.department;
    this.getDepartments();
    // this.departmentChoices$ = this.departmentsService.departmentsList({});
    this.initForm();
  }

  initForm() {
    this.form = this.formBuilder.group({
      name: [this.department?.name, Validators.required],
      description: [this.department?.description],
      parent_department: [this.department?.parent_department],
    });
  }

  getDepartments() {
    const filterIds = [];
    if (this.department) {
      filterIds.push(this.department.id, ...this.department.subdepartments);
    }

    this.departmentChoices$ = this.companiesService.companiesDepartmentTree(this.authenticationService.user.company).pipe(
      map((deps) =>
        deps
          .filter((dep) => {
            if (!this.department) {
              return true;
            } else {
              if (filterIds.includes(dep.id)) {
                filterIds.push(...dep.subdepartments);
                return false;
              } else {
                return true;
              }
            }
          })
          .map((dep) => {
            dep.name = '-'.repeat(dep.level) + ' ' + dep.name;
            return dep;
          })
      )
    );
  }

  save() {
    this.ref.close(this.form.value);
  }
}
