import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup } from '@angular/forms';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { FileHandle } from '@core-project/directives/file-drop.directive';
import { firstValueFrom, Observable } from 'rxjs';
import { DocumentFile, User } from 'data-access/sdk/api2';
import { DocumentsService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-document-file-form',
  templateUrl: './document-file-form.component.html',
  styleUrls: ['./document-file-form.component.scss'],
})
export class DocumentFileFormComponent implements OnInit {
  public documentId: number;
  public docFile: DocumentFile;

  public documentFileForm: FormGroup;
  public user: User;
  public selectedFile;

  constructor(
    private authenticationService: AuthenticationService,
    private config: DynamicDialogConfig,
    private formBuilder: FormBuilder,
    private documentsService: DocumentsService,
    public ref: DynamicDialogRef
  ) {}

  ngOnInit(): void {
    this.documentId = this.config.data.documentId;
    this.docFile = this.config.data.docFile;
    if (this.config.data.files && this.config.data.files.length) {
      this.selectedFile = this.config.data.files[0].file;
    }
    this.initForm();
  }

  public filesDropped(files: FileHandle[]): void {
    this.selectedFile = files[0]?.file;
  }

  public selectFile(event, fileUpload?) {
    this.selectedFile = event.currentFiles[0];
    fileUpload.clear();
  }

  public save() {
    let action: Observable<any>;
    if (this.docFile) {
      action = this.documentsService.documentsFilesPartialUpdate({
        id: this.docFile.id,
        data: this.documentFileForm.value,
      });
    } else {
      action = this.documentsService.documentsFilesCreate({
        document: this.documentId,
        comment: this.documentFileForm.value.comment,
        docFile: this.selectedFile,
      });
    }
    firstValueFrom(action).then((_) => this.ref.close());
  }

  private initForm() {
    this.documentFileForm = this.formBuilder.group({
      comment: [this.docFile?.comment || ''],
    });
  }
}
