import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DocumentsService } from 'data-access/sdk/api2';
import { TranslateService } from '@ngx-translate/core';
import { MessageService } from 'primeng/api';
import { DocumentMetaField } from 'data-access/sdk/api2';
import { ChoicefieldInterface } from '@core-project/interfaces';

@Component({
  selector: 'app-document-meta-form',
  templateUrl: './document-meta-form.component.html',
  styleUrls: ['./document-meta-form.component.scss'],
})
export class DocumentMetaFormComponent implements OnInit {
  meta: DocumentMetaField;
  form: FormGroup;
  metaTypeChoices: Array<ChoicefieldInterface>;
  showChoices: boolean;

  constructor(
    private config: DynamicDialogConfig,
    private formBuilder: FormBuilder,
    private ref: DynamicDialogRef,
    private documentsService: DocumentsService,
    private translateService: TranslateService,
    private messageService: MessageService
  ) {}

  ngOnInit(): void {
    this.meta = this.config.data.meta;
    this.metaTypeChoices = [
      { value: 'DT', label: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_TYPES.DT') },
      { value: 'INT', label: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_TYPES.INT') },
      { value: 'CH', label: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_TYPES.CH') },
      { value: 'CHS', label: this.translateService.instant('KB.METHODOLOGY.DOCUMENTS.META_TYPES.CHS') },
    ];
    this.initForm();
  }

  initForm() {
    this.form = this.formBuilder.group({
      name: [this.meta?.name, Validators.required],
      meta_type: [this.meta?.meta_type, Validators.required],
      in_name: [this.meta?.in_name || false, Validators.required],
      in_name_prefix: [this.meta?.in_name_prefix || ''],
      required: [this.meta?.required || false, Validators.required],
      placeholder: [this.meta?.placeholder || ''],
      choices: [this.meta?.choices || []],
    });
    this.inNameToggle();
    this.showChoicesToggle();
    if (this.meta) {
      this.form.controls.meta_type.disable();
    }
  }

  save() {
    this.ref.close(this.form.value);
  }

  inNameToggle() {
    if (this.form.controls.in_name.value) {
      this.form.controls.in_name_prefix.enable();
    } else {
      this.form.controls.in_name_prefix.disable();
    }
  }

  showChoicesToggle() {
    this.showChoices = this.form.controls.meta_type.value === 'CHS';
  }
}
