import { Component, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { BudgetCategory } from 'data-access/sdk/api2';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { FinanceService } from 'data-access/sdk/api2';
import { BudgetType } from 'data-access/sdk/api2';

@Component({
  selector: 'app-budget-type-form',
  templateUrl: './budget-type-form.component.html',
  styleUrls: ['./budget-type-form.component.scss'],
})
export class BudgetTypeFormComponent implements OnInit {
  public categories$: Observable<Array<BudgetCategory>>;
  private bType: BudgetType;
  public form: FormGroup;

  constructor(
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef,
    private formBuilder: FormBuilder,
    private financeService: FinanceService
  ) {}

  ngOnInit(): void {
    this.categories$ = this.financeService.financeBudgetCategoriesList();
    this.bType = this.config.data.bType;
    this.initForm();
  }

  initForm() {
    this.form = this.formBuilder.group({
      name: [this.bType?.name, Validators.required],
      category: [this.bType?.category, Validators.required],
      code: [this.bType?.code],
    });
  }

  save() {
    this.ref.close(this.form.value);
  }
}
