import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { IssuesService } from 'data-access/sdk/api2';
import { IssueDomain } from 'data-access/sdk/api2';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { firstValueFrom, Observable } from 'rxjs';

@Component({
  selector: 'app-issue-domain-form',
  templateUrl: './issue-domain-form.component.html',
  styleUrls: ['./issue-domain-form.component.scss'],
})
export class IssueDomainFormComponent implements OnInit {
  private domain: IssueDomain;
  public form: FormGroup;

  constructor(
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef,
    private issuesService: IssuesService,
    private fb: FormBuilder
  ) {}

  ngOnInit() {
    this.domain = this.config.data.domain;
    this.initForm();
  }

  initForm() {
    this.form = this.fb.group({
      name: [this.domain?.name, Validators.required],
      color: [this.domain?.color, Validators.required],
    });
  }

  save() {
    const obs$: Observable<IssueDomain> = this.domain
      ? this.issuesService.issuesDomainsPartialUpdate({ id: this.domain.id, data: this.form.value })
      : this.issuesService.issuesDomainsCreate(this.form.value);
    firstValueFrom(obs$).then((domain) => this.ref.close(domain));
  }
}
