import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { NewsService } from 'data-access/sdk/api2';
import { NewsEntry } from 'data-access/sdk/api2';
import { firstValueFrom } from 'rxjs';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-news-form',
  templateUrl: './news-form.component.html',
  styleUrls: ['./news-form.component.scss'],
})
export class NewsFormComponent implements OnInit {
  newsEntry: NewsEntry;
  projectId: number;
  departmentId: number;
  form: FormGroup;
  saving: boolean;

  constructor(
    private formBuilder: FormBuilder,
    private config: DynamicDialogConfig,
    private ref: DynamicDialogRef,
    private newsService: NewsService,
    private translateService: TranslateService
  ) {}

  ngOnInit() {
    this.newsEntry = this.config.data.newsEntry;
    this.projectId = this.config.data.projectId;
    this.departmentId = this.config.data.departmentId;

    this.initForm();
  }

  save() {
    this.saving = true;
    const data = this.form.value;
    const obs$ = data.id ? this.newsService.newsPartialUpdate({ id: data.id, data }) : this.newsService.newsCreate(data);
    firstValueFrom(obs$).then((_r) => this.ref.close());
  }

  private initForm() {
    this.form = this.formBuilder.group({
      id: [null],
      title: ['', Validators.required],
      text: ['', Validators.required],
      project: [this.projectId],
      department: [this.departmentId],
    });

    if (this.newsEntry) {
      this.form.patchValue(this.newsEntry);
    }
  }
}
