import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ProjectGroup } from 'data-access/sdk/api2';
import { CompaniesService } from 'data-access/sdk/api2';
import { map } from 'rxjs/operators';
import { BehaviorSubject, Observable } from 'rxjs';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';

@Component({
  selector: 'app-project-group-form',
  templateUrl: './project-group-form.component.html',
  styleUrls: ['./project-group-form.component.scss'],
})
export class ProjectGroupFormComponent implements OnInit {
  formGroup: FormGroup;
  projectGroupsChoicesSubject: BehaviorSubject<ProjectGroup[]> = new BehaviorSubject([]);
  projectGroupsChoices$: Observable<ProjectGroup[]> = this.projectGroupsChoicesSubject.asObservable();

  private projectGroup: ProjectGroup;

  constructor(
    private formBuilder: FormBuilder,
    private companiesService: CompaniesService,
    private authenticationService: AuthenticationService,
    private dialogConfig: DynamicDialogConfig,
    private dynamicDialogRef: DynamicDialogRef
  ) {}

  ngOnInit() {
    this.projectGroup = this.dialogConfig.data?.group;
    this.getProjectGroups();
    this.initForm();
  }

  initForm() {
    this.formGroup = this.formBuilder.group({
      name: [this.projectGroup?.name, Validators.required],
      parent_group: [this.projectGroup?.parent_group],
    });
  }

  save() {
    this.dynamicDialogRef.close(this.formGroup.value);
  }

  private getProjectGroups() {
    const filterIds = [];

    if (this.projectGroup) {
      filterIds.push(this.projectGroup.id, ...this.projectGroup.subgroups);
    }

    this.companiesService
      .companiesProjectGroupTree(this.authenticationService.user.company)
      .pipe(
        map((pgs) =>
          pgs
            .filter((pg) => {
              if (!this.projectGroup) {
                return true;
              } else {
                if (filterIds.includes(pg.id)) {
                  filterIds.push(...pg.subgroups);
                  return false;
                } else {
                  return true;
                }
              }
            })
            .map((pg) => {
              pg.name = '-'.repeat(pg.level) + ' ' + pg.name;
              return pg;
            })
        )
      )
      .subscribe((projectGroups) => {
        this.projectGroupsChoicesSubject.next(projectGroups);
      });
  }
}
