import { Component, OnInit } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';

import { ProjectCategoriesService } from 'data-access/sdk/api2';
import { ProjectCategory, ProjectType } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-type-form',
  templateUrl: './project-type-form.component.html',
  styleUrls: ['./project-type-form.component.scss'],
})
export class ProjectTypeFormComponent implements OnInit {
  private readonly categoriesSubject: BehaviorSubject<ProjectCategory[]> = new BehaviorSubject<ProjectCategory[]>([]);
  
  projectCategory: ProjectType | ProjectCategory | undefined;
  formGroup: FormGroup = this.formBuilder.group({
    name: ['', Validators.required],
    category: [null],
    about: [''],
  });

  readonly categories$: Observable<ProjectCategory[]> = this.categoriesSubject.asObservable();

  constructor(
    private readonly formBuilder: FormBuilder,
    private readonly dialogConfig: DynamicDialogConfig,
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly projectCategoriesService: ProjectCategoriesService
  ) {}

  ngOnInit(): void {
    this.projectCategoriesService.projectCategoriesList().subscribe((categories: ProjectCategory[]) => {
      this.categoriesSubject.next(categories);
    });

    this.projectCategory = this.dialogConfig.data?.cat;
    this.formGroup.patchValue(this.projectCategory);
  }

  save() {
    this.dynamicDialogRef.close(this.formGroup.value);
  }
}
