import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { ProjectView } from 'data-access/sdk/api2';

@Component({
  selector: 'app-project-view-form',
  templateUrl: './project-view-form.component.html',
  styleUrls: ['./project-view-form.component.scss'],
})
export class ProjectViewFormComponent implements OnInit {
  view: ProjectView;
  canBePublic: boolean;
  viewForm: FormGroup;

  constructor(private formBuilder: FormBuilder, private dialogRef: DynamicDialogRef, private dialogConfig: DynamicDialogConfig) {}

  ngOnInit() {
    this.view = this.dialogConfig.data.view;
    this.canBePublic = this.dialogConfig.data.canBePublic;
    this.initForm();
  }

  initForm() {
    this.viewForm = this.formBuilder.group({
      name: [this.view?.name, Validators.required],
      public: [this.view?.public || false, Validators.required],
    });
    if (!this.canBePublic) {
      this.viewForm.controls.public.disable();
    }
  }

  save() {
    this.dialogRef.close(this.viewForm.value);
  }
}
