import {Injectable} from '@angular/core';
import {UserRole} from 'data-access/sdk/api2';

export enum ResponsiblePrefixEnum {
  User = 'user_',
  Department = 'department_',
}

@Injectable({
  providedIn: 'root',
})
export class ResponsiblePrefixService {

  constructor() {
  }

  mapResponsible(role: UserRole | undefined): string {
    if (role?.user) {
      return this.addPrefix(role.user, ResponsiblePrefixEnum.User);
    }

    if (role?.department) {
      return this.addPrefix(role.department, ResponsiblePrefixEnum.Department);
    }

    return '';
  }

  getPrefix(value: string): ResponsiblePrefixEnum {
    if (value.startsWith(ResponsiblePrefixEnum.User)) {
      return ResponsiblePrefixEnum.User;
    }

    if (value.startsWith(ResponsiblePrefixEnum.Department)) {
      return ResponsiblePrefixEnum.Department;
    }
  }

  deletePrefix(value: string): number {
    if (value.startsWith(ResponsiblePrefixEnum.User)) {
      return Number(value.substring(ResponsiblePrefixEnum.User.length));
    }

    if (value.startsWith(ResponsiblePrefixEnum.Department)) {
      return Number(value.substring(ResponsiblePrefixEnum.Department.length));
    }
  }

  addPrefix(name: number | undefined, prefix: ResponsiblePrefixEnum): string | undefined {
    if (!name) {
      return undefined;
    }
    return prefix + name;
  }
}
