import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { FileContainerService } from '@core-project/services/file-container.service';
import { TranslateService } from '@ngx-translate/core';
import { Document, DocumentFile } from 'data-access/sdk/api2';

interface SaveAsItem {
  label: string;
  filename: string;
}

@Component({
  selector: 'app-document-save-button',
  templateUrl: './document-save-button.component.html',
  styleUrls: ['./document-save-button.component.scss'],
})
export class DocumentSaveButtonComponent implements OnInit {
  @ViewChild('op') panel;

  @Input() document: Document;
  @Input() documentFile: DocumentFile;

  saveAsItems: SaveAsItem[];

  constructor(private fileContainerService: FileContainerService, private ts: TranslateService) {}

  ngOnInit(): void {
    this.saveAsItems = [
      { label: this.ts.instant('DOCUMENTS.SAVE_BTN.AUTO_NAME'), filename: this.document.auto_name },
      { label: this.ts.instant('DOCUMENTS.SAVE_BTN.DOC_NAME'), filename: this.document.name },
      { label: this.ts.instant('DOCUMENTS.SAVE_BTN.FILE_NAME'), filename: this.documentFile.original_name },
    ];
  }

  public download(name: string) {
    this.fileContainerService.downloadFile(this.documentFile, name);
    this.panel.hide();
  }
}
