import { Component, Input, OnInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { CRUDEvent } from 'data-access/sdk/api2';

@Component({
  selector: 'app-history-record',
  templateUrl: './history-record.component.html',
  styleUrls: ['./history-record.component.scss'],
})
export class HistoryRecordComponent implements OnInit {
  @Input() h: CRUDEvent;
  @Input() style: any;
  @Input() styleClass: string;
  show = true;
  modelName: string;
  action: string;

  supported_event_types = [1, 2, 3];
  supported_models = ['work', 'issue'];
  supported_fields = {
    work: ['start_fc', 'end_fc', 'status'],
    issue: ['status'],
  };
  translated_fields = {
    work: ['status'],
    issue: ['status'],
  };

  constructor(private ts: TranslateService) {}

  ngOnInit(): void {
    this.modelName = this.h.content_type.model;
    if (!this.supported_event_types.includes(this.h.event_type) || !this.supported_models.includes(this.modelName)) {
      this.show = false;
      return;
    }

    const tsModelKey = `HISTORY.${this.modelName.toUpperCase()}`;
    switch (this.h.event_type) {
      // OBJECT CREATE
      case 1: {
        this.action = this.ts.instant(`${tsModelKey}.CREATE_VERB`) + this.ts.instant(`${tsModelKey}.NAME`);
        break;
      }

      // OBJECT CHANGE
      case 2: {
        const changes = JSON.parse(this.h.changed_fields);

        const actions: string[] = [];
        Object.keys(changes).forEach((ch) => {
          if (!this.supported_fields[this.modelName].includes(ch)) {
            return;
          }
          const oldValue = changes[ch][0];
          const newValue = changes[ch][1];

          const newDisplayValue = this.translated_fields[this.modelName].includes(ch)
            ? this.ts.instant(`${tsModelKey}.FIELDS_TS.${ch.toUpperCase()}.${newValue}`)
            : newValue;

          let action = '';
          if (oldValue === '' || oldValue === 'None') {
            action =
              this.ts.instant('HISTORY.CHANGE_TYPE.SET') +
              this.ts.instant(`${tsModelKey}.FIELDS.${ch.toUpperCase()}`) +
              `"${newDisplayValue}"`;
          } else {
            action =
              this.ts.instant('HISTORY.CHANGE_TYPE.EDIT') +
              this.ts.instant(`${tsModelKey}.FIELDS.${ch.toUpperCase()}`) +
              this.ts.instant('HISTORY.CH_TO') +
              `"${newDisplayValue}"`;
          }
          actions.push(action);
        });
        if (actions.length === 0) {
          this.show = false;
          return;
        }
        this.action = actions.join(', ');
        break;
      }

      // OBJECT REMOVE
      case 3: {
        this.action = this.ts.instant('HISTORY.EVENT_TYPE.' + String(this.h.event_type)) + this.ts.instant(`${tsModelKey}.NAME`);
        break;
      }
    }
  }
}
