import { AfterContentInit, Component, ContentChildren, Input, QueryList, TemplateRef } from '@angular/core';
import { PrimeTemplate } from 'primeng/api';

@Component({
  selector: 'app-icon-text-item',
  template: `
    <div [class]="styleClass" [ngStyle]="style" [ngClass]="{ 'mt-2 flex flex-row column-gap-2 row-gap-1': true }">
      <ng-container *ngTemplateOutlet="iconTemplate || defaultIcon"></ng-container>
      <ng-template #defaultIcon>
        <div class="flex align-items-center justify-content-center" style="width: 1.5rem">
          <ng-container *ngTemplateOutlet="iconContentTemplate || defaultIconContent"></ng-container>
          <ng-template #defaultIconContent>
            <i *ngIf="icon" [class]="icon" [ngClass]="{ 'text-color-secondary ': true }"></i>
          </ng-template>
        </div>
      </ng-template>

      <ng-container *ngTemplateOutlet="descriptionTemplate || defaultDescription"></ng-container>
      <ng-template #defaultDescription>
        <div class="flex align-items-center">
          <ng-container *ngTemplateOutlet="descriptionContentTemplate || defaultDescriptionContent"></ng-container>
          <ng-template #defaultDescriptionContent>
            <span class="text-color-secondary ">{{ description }}:</span>
          </ng-template>
        </div>
      </ng-template>

      <ng-container *ngTemplateOutlet="itemTemplate || defaultItem"></ng-container>
      <ng-template #defaultItem>
        <div class="flex flex-grow-1 align-items-center">
          <ng-container *ngTemplateOutlet="itemContentTemplate || defaultItemContent"></ng-container>
          <ng-template #defaultItemContent>
            <span>{{ item }}</span>
          </ng-template>
        </div>
      </ng-template>
    </div>
  `,
})
export class IconTextItemComponent implements AfterContentInit {
  @Input() icon: string;
  @Input() description: string;
  @Input() item: string;
  @Input() style: any;
  @Input() styleClass: string;
  @Input() editable: boolean;

  @ContentChildren(PrimeTemplate) templates: QueryList<any>;

  iconTemplate: TemplateRef<any>;
  iconContentTemplate: TemplateRef<any>;
  descriptionTemplate: TemplateRef<any>;
  descriptionContentTemplate: TemplateRef<any>;
  itemTemplate: TemplateRef<any>;
  itemContentTemplate: TemplateRef<any>;

  ngAfterContentInit() {
    this.templates.forEach((template) => {
      switch (template.getType()) {
        case 'icon':
          this.iconTemplate = template.template;
          break;
        case 'iconContent':
          this.iconContentTemplate = template.template;
          break;
        case 'description':
          this.descriptionTemplate = template.template;
          break;
        case 'descriptionContent':
          this.descriptionContentTemplate = template.template;
          break;
        case 'item':
          this.itemTemplate = template.template;
          break;
        case 'itemContent':
          this.itemContentTemplate = template.template;
          break;
        default:
          this.itemContentTemplate = template.template;
      }
    });
  }
}
