import { Component, EventEmitter, Input, Output } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { ProjectEnums } from '@core-project/enums/project';
import IndicatorEnum = ProjectEnums.IndicatorEnum;

@Component({
  selector: 'app-indicator-editor',
  template: `
    <div [class]="styleClass">
      <span *ngFor="let ind of indicators" [ngClass]="{ 'opacity-20': indicator !== ind, 'cursor-pointer': !disabled }">
        <app-project-indicator
          [indicator]="ind"
          [icon]="icon"
          styleClass="inline-flex"
          (click)="changeIndicator(ind)"
        ></app-project-indicator>
      </span>
    </div>
  `,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: IndicatorEditorComponent,
    },
  ],
})
export class IndicatorEditorComponent implements ControlValueAccessor {
  @Input() styleClass: any;
  @Input() icon = 'fa-solid fa-circle';

  @Output() onChange: EventEmitter<string> = new EventEmitter<IndicatorEnum>();

  touched = false;
  disabled = false;

  indicator: IndicatorEnum;

  indicators = [IndicatorEnum.RED, IndicatorEnum.AMBER, IndicatorEnum.GREEN, IndicatorEnum.NA];

  onChangeFn = (indicator) => {};
  onTouchedFn = () => {};

  writeValue(indicator: IndicatorEnum) {
    this.indicator = indicator;
  }

  registerOnChange(onChange: any) {
    this.onChangeFn = onChange;
  }

  registerOnTouched(onTouched: any) {
    this.onTouchedFn = onTouched;
  }

  markAsTouched() {
    if (!this.touched) {
      this.onTouchedFn();
      this.touched = true;
    }
  }

  setDisabledState(disabled: boolean) {
    this.disabled = disabled;
  }

  changeIndicator(indicator: IndicatorEnum) {
    if (this.disabled) return;
    this.indicator = indicator;
    this.onChangeFn(indicator);
    this.onChange.emit(indicator);
  }
}
