import { Component, Input, OnInit } from '@angular/core';
import { IndicatorConstraints } from '@core-project/interfaces/indicator-constraints';

@Component({
  selector: 'app-indicator-knob',
  template: `
    <p-knob
      [min]="0"
      [max]="valueNum > 100 ? valueNum : 100"
      [readonly]="true"
      [valueTemplate]="valueNum > 300 ? '>300%' : '{value}%'"
      [valueColor]="color"
      [ngModel]="valueNum"
    ></p-knob>
  `,
})
export class IndicatorKnobComponent implements OnInit {
  @Input() value: string | number;
  @Input() constraints: IndicatorConstraints;

  public color = 'green';
  public valueNum: number;

  ngOnInit() {
    if (typeof this.value === 'string') {
      this.valueNum = Number(this.value);
    } else {
      this.valueNum = this.value;
    }
    if (this.constraints) {
      this.color = this.getColor(this.valueNum, this.constraints);
    }
  }

  getColor(value: number, constraints: IndicatorConstraints): string {
    if (constraints.redGte && value >= constraints.redGte) {
      return 'red';
    }
    if (constraints.redLte && value <= constraints.redLte) {
      return 'red';
    }
    if (constraints.amberGte && value >= constraints.amberGte) {
      return 'orange';
    }
    if (constraints.amberLte && value <= constraints.amberLte) {
      return 'orange';
    }
    return 'green';
  }
}
