import {
  AfterContentInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ContentChildren,
  EventEmitter,
  Input,
  Output,
  QueryList,
  TemplateRef,
  ViewEncapsulation,
} from '@angular/core';
import { PrimeTemplate } from 'primeng/api';

@Component({
  selector: 'app-inplace',
  templateUrl: './inplace.component.html',
  styleUrls: ['./inplace.component.scss'],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class InplaceComponent implements AfterContentInit {
  @Input() active: boolean;
  @Input() closable = true;
  @Input() disabled: boolean;
  @Input() preventClick: boolean;
  @Input() style: any;
  @Input() styleClass: string;
  @Input() closeIcon = 'fa fa-check';
  @Input() closeLabel: string;

  @ContentChildren(PrimeTemplate) templates: QueryList<any>;

  @Output() onActivate: EventEmitter<any> = new EventEmitter();
  @Output() onDeactivate: EventEmitter<any> = new EventEmitter();

  hover: boolean;
  displayTemplate: TemplateRef<any>;
  contentTemplate: TemplateRef<any>;

  constructor(public cd: ChangeDetectorRef) {}

  ngAfterContentInit() {
    this.templates.forEach((item) => {
      switch (item.getType()) {
        case 'display':
          this.displayTemplate = item.template;
          break;

        case 'content':
          this.contentTemplate = item.template;
          break;
      }
    });
  }

  onActivateClick(event) {
    if (!this.preventClick) {
      this.activate(event);
    }
  }

  onDeactivateClick(event) {
    if (!this.preventClick) {
      this.deactivate(event);
    }
  }

  activate(event?: Event) {
    if (!this.disabled) {
      this.active = true;
      this.onActivate.emit(event);
      this.cd.markForCheck();
    }
  }

  deactivate(event?: Event) {
    if (!this.disabled) {
      this.active = false;
      this.hover = false;
      this.onDeactivate.emit(event);
      this.cd.markForCheck();
    }
  }

  onKeydown(event: KeyboardEvent) {
    if (event.which === 13) {
      this.activate(event);
      event.preventDefault();
    }
  }
}
