import { Component, Input, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { ProjectState } from 'data-access/sdk/api2';
import { ProjectEnums } from '@core-project/enums/project';
import IndicatorEnum = ProjectEnums.IndicatorEnum;
import * as moment from 'moment';

interface IconedIndicator {
  icon: string;
  indicator: IndicatorEnum;
}

const iconToIndicatorMapping = {
  works_indicator: 'fa-regular fa-calendar-range',
  budget_indicator: 'fa-regular fa-ruble-sign',
  issues_indicator: 'fa-regular fa-fire',
  procurements_indicator: 'fa-regular fa-cart-shopping',
};

@Component({
  selector: 'app-project-indicator-bar',
  template: `
    <app-project-indicator
      *ngFor="let ind of indicators"
      [icon]="ind.icon"
      [sizeRem]="sizeRem"
      [indicator]="ind.indicator"
    ></app-project-indicator>
  `,
})
export class ProjectIndicatorBarComponent implements OnChanges {
  @Input() projectState: ProjectState;
  @Input() validate = false;
  @Input() sizeRem = 1;

  public indicators: IconedIndicator[];

  ngOnChanges(changes: SimpleChanges) {
    let invalidated = !this.projectState;
    if (this.validate && this.projectState) {
      invalidated = moment.duration(moment(this.projectState.date).diff(moment())).days() > 7;
    }

    this.indicators = [];

    Object.keys(iconToIndicatorMapping).forEach((ind) => {
      this.indicators.push({
        icon: iconToIndicatorMapping[ind],
        indicator: invalidated ? IndicatorEnum.NA : this.projectState[ind],
      });
    });
  }
}
