import { Component, Input, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { ProjectEnums } from '@core-project/enums/project';
import { Project } from 'data-access/sdk/api2';
import { ProjectTypeStagesService } from 'data-access/sdk/api2';
import StatusEnum = ProjectEnums.StatusEnum;

interface Stage {
  name: string;
  styleClass: string;
}

@Component({
  selector: 'app-project-stages',
  template: `
    <div class="grid grid-nogutter">
      <div class="col" *ngFor="let stage of stages">
        <div [class]="stage.styleClass" [ngClass]="{ 'h-3rem border-2 flex justify-content-center align-items-center': true }">
          <span class="text-white font-bold">{{ stage.name }}</span>
        </div>
      </div>
    </div>
  `,
})
export class ProjectStagesComponent implements OnChanges {
  @Input() project: Project;

  stages: Stage[] = [];

  constructor(private projectTypeStagesService: ProjectTypeStagesService) {}

  ngOnChanges(changes: SimpleChanges) {
    this.createStages();
  }

  private createStages() {
    this.projectTypeStagesService
      .projectTypeStagesList({ projectType: this.project.project_type.toString(), include: 'stage' })
      .subscribe((ptss) => {
        this.stages = [];
        ptss.forEach((pts) => {
          const color = this.getColorByStatus(pts.rel_stage.status);
          const opacity = pts.stage === this.project.stage ? '' : ' opacity-30';

          this.stages.push({
            name: pts.rel_stage.name,
            styleClass: `border-${color}-700 bg-${color}-500` + opacity,
          });
        });
      });
  }

  private getColorByStatus(status: StatusEnum): string {
    switch (status) {
      case StatusEnum.FUT:
        return 'blue';
      case StatusEnum.ACT:
        return 'green';
      case StatusEnum.CLS:
        return 'gray';
    }
  }
}
