import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig } from 'primeng/dynamicdialog';
import { ProjectStatesService } from 'data-access/sdk/api2';
import { ProjectState } from 'data-access/sdk/api2';
import { Observable } from 'rxjs';

@Component({
  selector: 'app-project-state-history',
  template: `
    <p-table [value]="states$ | async">
      <ng-template pTemplate="header">
        <tr>
          <th></th>
          <th>{{ 'PROJECT_STATE.STATE' | translate }}</th>
          <th>{{ 'PROJECT_STATE.USER' | translate }}</th>
          <th>{{ 'PROJECT_STATE.DATE' | translate }}</th>
        </tr>
      </ng-template>

      <ng-template pTemplate="body" let-state>
        <tr>
          <td><app-project-indicator [indicator]="state.indicator"></app-project-indicator></td>
          <td [innerHTML]="state.state"></td>
          <td><lib-user-string [user]="state.rel_author"></lib-user-string></td>
          <td>{{ state.date | date : 'shortDate' }}</td>
        </tr>
      </ng-template>
    </p-table>
  `,
})
export class ProjectStateHistoryComponent implements OnInit {
  private projectId: number;
  public states$: Observable<ProjectState[]>;

  constructor(private config: DynamicDialogConfig, private projectStatesService: ProjectStatesService) {}

  ngOnInit() {
    this.projectId = this.config.data.projectId;
    if (this.projectId) {
      this.states$ = this.projectStatesService.projectStatesList({ project: this.projectId.toString(), include: 'author' });
    }
  }
}
