import { Component, Input, OnInit } from '@angular/core';
import { ProjectStatesService } from 'data-access/sdk/api2';
import { ProjectState } from 'data-access/sdk/api2';
import { Observable } from 'rxjs';
import { ProjectEnums } from '@core-project/enums/project';

@Component({
  selector: 'app-project-state',
  template: `
    <ng-container *ngIf="state$ | async as state; else loading">
      <ng-container *ngIf="state; else noState">
        <app-project-indicator-bar [projectState]="state"></app-project-indicator-bar>
        <span [innerHTML]="state.state"></span>

        <div class="text-sm font-italic text-gray-500 mt-2" *ngIf="state">
          <lib-user-string [user]="state.rel_author"></lib-user-string> {{ state.date | date : 'shortDate' }}
        </div>
      </ng-container>

      <ng-template #noState>
        <app-project-indicator-bar [projectState]="state"></app-project-indicator-bar>
        {{ 'PROJECT.DETAILS.NO_STATE' | translate }}
      </ng-template>
    </ng-container>

    <ng-template #loading>
      <app-project-indicator-bar [projectState]="null"></app-project-indicator-bar>
      {{ 'PROJECT.DETAILS.NO_STATE' | translate }}
    </ng-template>
  `,
})
export class ProjectStateComponent implements OnInit {
  @Input() projectId: number;
  public state$: Observable<ProjectState>;
  public indicatorEnum = ProjectEnums.IndicatorEnum;

  constructor(private projectStatesService: ProjectStatesService) {}

  ngOnInit() {
    this.state$ = this.projectStatesService.projectStatesCurrent(this.projectId.toString());
  }
}
