import {Component, Input, OnChanges, OnInit, SimpleChanges} from '@angular/core';
import {ProjectEnums} from '@core-project/enums/project';

@Component({
  selector: 'app-project-status',
  templateUrl: './project-status.component.html',
  styleUrls: ['./project-status.component.scss'],
})
export class ProjectStatusComponent implements OnInit, OnChanges {
  @Input() status: ProjectEnums.StatusEnum;
  @Input() showText = true;
  @Input() styleClass: string;
  @Input() textStyle: 'long' | 'short' = 'long';

  icon: string;
  color: string;
  translateString: string;

  constructor() {
  }

  ngOnInit(): void {
    this.getColorAndIcon();
  }

  ngOnChanges(changes: SimpleChanges) {
    this.getColorAndIcon();
    this.translateString = this.textStyle === 'short' ? 'PROJECTS.STATUS.' : 'PROJECT_STATUS.';
  }

  getColorAndIcon() {
    switch (this.status) {
      case ProjectEnums.StatusEnum.FUT: {
        this.icon = 'fa-light fa-flux-capacitor';
        this.color = 'text-blue-500';
        break;
      }
      case ProjectEnums.StatusEnum.ACT: {
        this.icon = 'fa-solid fa-play';
        this.color = 'text-green-500';
        break;
      }
      case ProjectEnums.StatusEnum.CLS: {
        this.icon = 'fa-solid fa-stop';
        this.color = 'text-gray-500';
        break;
      }
      case ProjectEnums.StatusEnum.CNC: {
        this.icon = 'fa-solid fa-ban';
        this.color = 'text-red-500';
        break;
      }
      case ProjectEnums.StatusEnum.PSP: {
        this.icon = 'fa-solid fa-pause';
        this.color = 'text-purple-500';
        break;
      }
    }
  }
}
