import {Component, Input, OnInit} from '@angular/core';
import {ReportPlugin, ReportPluginOption, ReportPluginOptionsService} from 'data-access/sdk/api2';
import {Observable} from 'rxjs';
import {ReportPluginOptionEnums} from '@core-project/enums/report-plugins';
import {ReportDownloaderService} from '@core-project/services/report-downloader.service';
import {Store} from '@ngxs/store';
import {RightSidebarActions} from 'data-access/store';

@Component({
  selector: 'app-report-config',
  templateUrl: './report-config.component.html',
  styleUrls: ['./report-config.component.scss'],
})
export class ReportConfigComponent implements OnInit {
  @Input() reportPlugin: ReportPlugin;

  componentEnum = ReportPluginOptionEnums.ComponentEnum;
  options$: Observable<ReportPluginOption[]>;
  params = {};

  constructor(
    private reportPluginOptionsService: ReportPluginOptionsService,
    private reportDownloaderService: ReportDownloaderService,
    private readonly store: Store,
  ) {
  }

  ngOnInit() {
    this.store.dispatch(new RightSidebarActions.SetHeader(this.reportPlugin.name));
    this.loadParams();
    this.options$ = this.reportPluginOptionsService.reportPluginOptionsList({reportPlugin: this.reportPlugin.id.toString()});
  }

  getReport() {
    this.saveParams();
    this.reportDownloaderService.getReport(this.reportPlugin, this.params);
    this.store.dispatch(new RightSidebarActions.Close());
  }

  private saveParams() {
    localStorage.setItem('reportParams_' + this.reportPlugin.module_name, JSON.stringify(this.params));
  }

  private loadParams() {
    const paramsStr = localStorage.getItem('reportParams_' + this.reportPlugin.module_name);
    if (paramsStr) {
      this.params = JSON.parse(paramsStr);
    }
  }
}
