import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { LoadingComponent } from './loading/loading.component';
import { TabMenuComponent } from './tab-menu/tab-menu.component';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { GridTableComponent } from './grid-table/grid-table.component';
import { TranslateModule } from '@ngx-translate/core';
import { UserTagComponent } from './user-tag/user-tag.component';
import { HistoryRecordComponent } from './history-record/history-record.component';
import { DocumentsStructureComponent } from './documents/documents-structure/documents-structure.component';
import { DocumentInfoComponent } from './documents/document-info/document-info.component';
import { DirectivesModule } from '@core-project/directives/directives.module';
import { ContactTagComponent } from './contact-tag/contact-tag.component';
import { UnderConstructionComponent } from './under-construction/under-construction.component';
import { ExtensionSwitcherComponent } from './extension-switcher/extension-switcher.component';
import { DocumentFormsModule } from '@core-project/modules/forms/document-forms/document-forms.module';
import { DateFilterComponent } from './date-filter/date-filter.component';
import { ProjectStatusComponent } from './project-status/project-status.component';
import { NewsfeedComponent } from './news/newsfeed/newsfeed.component';
import { NewsEntryComponent } from './news/news-entry/news-entry.component';
import { DocumentSaveButtonComponent } from './documents/document-save-button/document-save-button.component';
import { EditorComponent } from './editor/editor.component';
import { InplaceComponent } from './inplace/inplace.component';
import { CommentComponent } from './comment/comment.component';
import { ActionsComponent } from './actions/actions.component';
import { IssueFireComponent } from '@core-project/modules/shared-components/issues/issue-fire/issue-fire.component';
import { IssueStatusComponent } from '@core-project/modules/shared-components/issues/issue-status/issue-status.component';
import { IssueRakeComponent } from '@core-project/modules/shared-components/issues/issue-rake/issue-rake.component';
import { IssuesTableComponent } from '@core-project/modules/shared-components/issues/issues-table/issues-table.component';
import { IssueDomainTagComponent } from '@core-project/modules/shared-components/issues/issue-domain-tag/issue-domain-tag.component';
import { KanbanBoardComponent } from '@core-project/modules/shared-components/works/kanban-board/kanban-board.component';
import { KanbanBoardDialogComponent } from '@core-project/modules/shared-components/works/kanban-board-dialog/kanban-board-dialog.component';
import { WorkCardComponent } from '@core-project/modules/shared-components/works/work-card/work-card.component';
import { WorkDangerComponent } from '@core-project/modules/shared-components/works/work-danger/work-danger.component';
import { WorkDateComponent } from '@core-project/modules/shared-components/works/work-date-tag/work-date.component';
import { WorkPeriodComponent } from '@core-project/modules/shared-components/works/work-period/work-period.component';
import { WorkStatusComponent } from '@core-project/modules/shared-components/works/work-status/work-status.component';
import { WorkStringComponent } from '@core-project/modules/shared-components/works/work-string/work-string.component';
import { WorkSymbolComponent } from '@core-project/modules/shared-components/works/work-symbol/work-symbol.component';
import { WorksTableComponent } from '@core-project/modules/shared-components/works/works-table/works-table.component';
import { GanttChartComponent } from './works/gantt-chart/gantt-chart.component';
import { WorkFormComponent } from '@core-project/modules/shared-components/works/work-form/work-form.component';
import { GanttTooltipComponent } from './works/gantt-tooltip/gantt-tooltip.component';
import { GanttScaleComponent } from './works/gantt-scale/gantt-scale.component';
import { GanttAutoscheduleComponent } from './works/gantt-autoschedule/gantt-autoschedule.component';
import { GanttViewSwitcherComponent } from './works/gantt-view-switcher/gantt-view-switcher.component';
import { WorkToUserSelectorComponent } from './works/work-to-user-selector/work-to-user-selector.component';
import { WorklinkInfoComponent } from './works/worklink-info/worklink-info.component';
import { WorkTemplateFormComponent } from './works/work-template-form/work-template-form.component';
import { WorklinktemplateInfoComponent } from './works/worklinktemplate-info/worklinktemplate-info.component';
import { WorksFromTemplateFormComponent } from './works/works-from-template-form/works-from-template-form.component';
import { WidgetComponent } from './widget/widget.component';
import { CdkDrag, CdkDragHandle } from '@angular/cdk/drag-drop';
import { ProjectIndicatorComponent } from './project-indicator/project-indicator.component';
import { IndicatorKnobComponent } from './indicator-knob/indicator-knob.component';
import { ProjectStagesComponent } from './project-stages/project-stages.component';
import { OrganizationStringComponent } from './organization-string/organization-string.component';
import { HelpButtonComponent } from './help-button/help-button.component';
import { HelpWindowComponent } from './help-window/help-window.component';
import { IndicatorEditorComponent } from './indicator-editor/indicator-editor.component';
import { ProjectStateHistoryComponent } from './project-state-history/project-state-history.component';
import { ProjectStateComponent } from './project-state/project-state.component';
import { ProjectIndicatorBarComponent } from './project-indicator-bar/project-indicator-bar.component';
import { ProjectStatusReportComponent } from './project-status-report/project-status-report.component';
import { IconTextItemComponent } from './icon-text-item/icon-text-item.component';
import { PipesModule } from '@core-project/pipes/pipes.module';
import { ProjectTreeSelectComponent } from './project-tree-select/project-tree-select.component';
import { ReportConfigComponent } from './report-config/report-config.component';
import { UiCustomCalendarModule } from 'ui/custom-calendar';
import { StringInfoOverlaysModule } from 'features/string-info-overlays';

// primeng
import { TabMenuModule } from 'primeng/tabmenu';
import { RatingModule } from 'primeng/rating';
import { ChipModule } from 'primeng/chip';
import { ScrollPanelModule } from 'primeng/scrollpanel';
import { CardModule } from 'primeng/card';
import { DragDropModule } from 'primeng/dragdrop';
import { ButtonModule } from 'primeng/button';
import { TableModule } from 'primeng/table';
import { ConfirmDialogModule } from 'primeng/confirmdialog';
import { RippleModule } from 'primeng/ripple';
import { OverlayPanelModule } from 'primeng/overlaypanel';
import { DropdownModule } from 'primeng/dropdown';
import { MultiSelectModule } from 'primeng/multiselect';
import { ListboxModule } from 'primeng/listbox';
import { SelectButtonModule } from 'primeng/selectbutton';
import { CalendarModule } from 'primeng/calendar';
import { FieldsetModule } from 'primeng/fieldset';
import { EditorModule } from 'primeng/editor';
import { AvatarModule } from 'primeng/avatar';
import { ImageModule } from 'primeng/image';
import { SkeletonModule } from 'primeng/skeleton';
import { TagModule } from 'primeng/tag';
import { InputNumberModule } from 'primeng/inputnumber';
import { InputTextareaModule } from 'primeng/inputtextarea';
import { InputTextModule } from 'primeng/inputtext';
import { InputSwitchModule } from 'primeng/inputswitch';
import { CheckboxModule } from 'primeng/checkbox';
import { ProgressSpinnerModule } from 'primeng/progressspinner';
import { KnobModule } from 'primeng/knob';
import { TreeTableModule } from 'primeng/treetable';
import { HasPermissionDirective } from 'utils/shared-directives';
import { UiMilestonesTimelineComponent } from 'ui/milestones-timeline';

const primeNgModules = [
  TabMenuModule,
  RatingModule,
  ChipModule,
  ScrollPanelModule,
  CardModule,
  DragDropModule,
  ButtonModule,
  TableModule,
  ConfirmDialogModule,
  RippleModule,
  OverlayPanelModule,
  DropdownModule,
  MultiSelectModule,
  ListboxModule,
  SelectButtonModule,
  CalendarModule,
  FieldsetModule,
  EditorModule,
  AvatarModule,
  ImageModule,
  SkeletonModule,
  TagModule,
  InputNumberModule,
  InputTextareaModule,
  InputTextModule,
  InputSwitchModule,
  CheckboxModule,
  ProgressSpinnerModule,
  KnobModule,
  TreeTableModule,
];

@NgModule({
  declarations: [
    LoadingComponent,
    TabMenuComponent,
    GridTableComponent,
    UserTagComponent,
    HistoryRecordComponent,
    DocumentsStructureComponent,
    DocumentInfoComponent,
    ContactTagComponent,
    UnderConstructionComponent,
    ExtensionSwitcherComponent,
    DateFilterComponent,
    ProjectStatusComponent,
    NewsfeedComponent,
    NewsEntryComponent,
    DocumentSaveButtonComponent,
    EditorComponent,
    InplaceComponent,
    CommentComponent,
    ActionsComponent,
    IssueFireComponent,
    IssueStatusComponent,
    IssueRakeComponent,
    IssuesTableComponent,
    IssueDomainTagComponent,
    KanbanBoardComponent,
    KanbanBoardDialogComponent,
    WorkCardComponent,
    WorkDangerComponent,
    WorkDateComponent,
    WorkPeriodComponent,
    WorkStatusComponent,
    WorkStringComponent,
    WorkSymbolComponent,
    WorksTableComponent,
    WorkFormComponent,
    GanttChartComponent,
    GanttTooltipComponent,
    GanttScaleComponent,
    GanttAutoscheduleComponent,
    GanttViewSwitcherComponent,
    WorkToUserSelectorComponent,
    WorklinkInfoComponent,
    WorkTemplateFormComponent,
    WorklinktemplateInfoComponent,
    WorksFromTemplateFormComponent,
    WidgetComponent,
    ProjectIndicatorComponent,
    IndicatorKnobComponent,
    ProjectStagesComponent,
    OrganizationStringComponent,
    HelpButtonComponent,
    HelpWindowComponent,
    IndicatorEditorComponent,
    ProjectStateHistoryComponent,
    ProjectStateComponent,
    ProjectIndicatorBarComponent,
    ProjectStatusReportComponent,
    IconTextItemComponent,
    ProjectTreeSelectComponent,
    ReportConfigComponent,
  ],
  exports: [
    LoadingComponent,
    TabMenuComponent,
    GridTableComponent,
    DocumentsStructureComponent,
    DocumentInfoComponent,
    ContactTagComponent,
    UnderConstructionComponent,
    ExtensionSwitcherComponent,
    DateFilterComponent,
    ProjectStatusComponent,
    NewsfeedComponent,
    EditorComponent,
    InplaceComponent,
    HistoryRecordComponent,
    CommentComponent,
    ActionsComponent,
    UserTagComponent,
    WorksTableComponent,
    KanbanBoardComponent,
    WorkSymbolComponent,
    IssueFireComponent,
    IssuesTableComponent,
    GanttChartComponent,
    GanttScaleComponent,
    GanttAutoscheduleComponent,
    GanttViewSwitcherComponent,
    WidgetComponent,
    ProjectIndicatorComponent,
    ProjectIndicatorComponent,
    IndicatorKnobComponent,
    ProjectStagesComponent,
    OrganizationStringComponent,
    HelpButtonComponent,
    IndicatorEditorComponent,
    ProjectIndicatorBarComponent,
    IconTextItemComponent,
    WorkStatusComponent,
  ],
  imports: [
    ...primeNgModules,
    CommonModule,
    FormsModule,
    TranslateModule,
    DirectivesModule,
    DocumentFormsModule,
    ReactiveFormsModule,
    CdkDrag,
    CdkDragHandle,
    PipesModule,
    HasPermissionDirective,
    UiCustomCalendarModule,
    StringInfoOverlaysModule,
    UiMilestonesTimelineComponent,
  ],
})
export class SharedComponentsModule {}
