import {
  Component,
  EventEmitter,
  Input,
  OnDestroy,
  OnInit,
  Output,
  Renderer2,
  ViewChild,
  ViewContainerRef,
  ViewEncapsulation,
} from '@angular/core';
import { UserWidget } from 'data-access/sdk/api2';
import { WidgetParams } from '@core-project/interfaces';
import { widgetsParams } from '@core-project/helpers/widget';
import * as _ from 'lodash-es';
import { WidgetContainerDirective } from 'utils/shared-directives';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-widget',
  templateUrl: './widget.component.html',
  styleUrls: ['./widget.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class WidgetComponent implements OnInit, OnDestroy {
  @Input() parent: HTMLElement;
  @Input() userWidget: UserWidget;

  @Input() editable = false;

  @Output() onChange = new EventEmitter<UserWidget>();
  @Output() onClose = new EventEmitter();

  @ViewChild(WidgetContainerDirective, { static: true }) contentContainer: WidgetContainerDirective;

  loading: boolean;
  widgetParams: WidgetParams;

  private dataChSub: Subscription;

  constructor() {}

  ngOnInit() {
    this.widgetParams = _.find(widgetsParams, ['name', this.userWidget.widget]);
    this.renderComponent();
  }

  ngOnDestroy() {
    this.dataChSub?.unsubscribe();
  }

  endDrag(pos: any) {
    this.userWidget.position_x = pos.x;
    this.userWidget.position_y = pos.y;
    this.onChange.emit(this.userWidget);
  }

  close(event: Event) {
    event.stopPropagation();
    this.onClose.emit();
  }

  private dataChange(newData: any) {
    // console.log('DATA CHANGE', newData);
    this.userWidget.properties = newData;
    this.onChange.emit(this.userWidget);
  }

  private renderComponent() {
    const componentRef = this.contentContainer.viewContainerRef.createComponent(this.widgetParams.component);
    // console.log('RENDER', this.userWidget);
    if (this.userWidget.properties) {
      Object.keys(this.userWidget.properties).forEach((k) => {
        componentRef.setInput(k, this.userWidget.properties[k]);
        // componentRef.instance[k] = this.userWidget.properties[k];
      });
    }

    this.dataChSub = componentRef.instance.onChange.subscribe((ch) => this.dataChange(ch));
  }
}
