import { Component, ElementRef, Input, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { Work } from 'data-access/sdk/api2';
import { WorkEnums } from '@core-project/enums';
import { TranslateService } from '@ngx-translate/core';
import { getColorByStatus, getIconByStatus } from '@core-project/helpers/works';

@Component({
  selector: 'app-work-status',
  template: `
    <span [class]="styleClass" [ngStyle]="style">
      <span [class]="color">
        <i *ngIf="showIcon" [class]="icon" [pTooltip]="text"></i>
        <span *ngIf="showLabel" [ngClass]="{ 'ml-2': showIcon }">{{ text }}</span>
      </span>
    </span>
  `,
})
export class WorkStatusComponent implements OnInit, OnChanges {
  @Input() status: WorkEnums.StatusEnum;
  @Input() style: any;
  @Input() styleClass: string;
  @Input() showLabel = true;
  @Input() showIcon = true;

  public workStatus = WorkEnums.StatusEnum;
  elementRef: ElementRef;

  public color: string;
  public icon: string;
  public text: string;

  constructor(private elRef: ElementRef, private ts: TranslateService) {
    this.elementRef = elRef;
  }

  ngOnInit() {
    this.onChanges();
  }

  ngOnChanges(changes: SimpleChanges) {
    this.onChanges();
  }

  private onChanges() {
    this.color = getColorByStatus(this.status);
    this.icon = getIconByStatus(this.status);
    this.text = this.ts.instant('WORKS.STATUS.' + this.status);
  }
}
