import { Component, OnInit } from '@angular/core';
import { WorkTemplate } from 'data-access/sdk/api2';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DialogEnums, WorkEnums } from '@core-project/enums';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import { WorktemplatesService } from 'data-access/sdk/api2';
import { SelectItem } from 'primeng/api';
import { Work } from 'data-access/sdk/api2';
import * as moment from 'moment/moment';
import { firstValueFrom, Observable } from 'rxjs';
import { ProjectEnums } from '@core-project/enums/project';
import StageEnum = ProjectEnums.StageEnum;
import { ProjectTypeStage } from 'data-access/sdk/api2';
import { ProjectTypeStagesService } from 'data-access/sdk/api2';

@Component({
  selector: 'app-work-template-form',
  templateUrl: './work-template-form.component.html',
  styleUrls: ['./work-template-form.component.scss'],
})
export class WorkTemplateFormComponent implements OnInit {
  workT: WorkTemplate;
  workTform: FormGroup;
  workTypes = WorkEnums.WorkTypeEnum;
  workTypeChoices: SelectItem[] = [];
  ptStages$: Observable<ProjectTypeStage[]>;
  private projectTypeId: number;

  constructor(
    private ref: DynamicDialogRef,
    private config: DynamicDialogConfig,
    private formBuilder: FormBuilder,
    private translateService: TranslateService,
    private workTemplatesService: WorktemplatesService,
    private projectTypeStagesService: ProjectTypeStagesService
  ) {}

  get workType() {
    return this.workTform.controls.work_type.value;
  }

  get c() {
    return this.workTform.controls;
  }

  ngOnInit() {
    this.workT = this.config.data.workTemplate;
    this.projectTypeId = this.config.data.projectTypeId;

    const allowedTypes = [WorkEnums.WorkTypeEnum.TS, WorkEnums.WorkTypeEnum.MS];
    allowedTypes.forEach((wt) =>
      this.workTypeChoices.push({
        label: this.translateService.instant('WORKS.TYPE.' + wt),
        value: wt,
      })
    );
    this.ptStages$ = this.projectTypeStagesService.projectTypeStagesList({
      projectType: this.projectTypeId.toString(),
      include: 'stage',
    });

    this.initForm(this.workT);
  }

  initForm(workT: WorkTemplate) {
    this.workTform = this.formBuilder.group({
      id: [workT?.id],
      name: [workT?.name, Validators.required],
      description: [workT?.description],
      result: [workT?.result],
      work_type: [workT?.work_type, Validators.required],
      project_type: [workT?.project_type || this.projectTypeId],
      group: [workT?.group],
      start_date: [workT?.start_date || '1970-01-01', Validators.required],
      end_date: [workT?.end_date || '1970-01-02', Validators.required],
      milestone_level: [workT?.milestone_level || 3],
      milestone_stage: [workT?.milestone_stage],
    });
  }

  save() {
    const val = this.workTform.value;
    const obs$: Observable<Work> = this.workT?.id
      ? this.workTemplatesService.worktemplatesPartialUpdate({ id: this.workT.id, data: val })
      : this.workTemplatesService.worktemplatesCreate(val);
    firstValueFrom(obs$).then((w) => this.ref.close(w));
  }

  delete() {
    if (this.workT.id) {
      firstValueFrom(this.workTemplatesService.worktemplatesDelete(this.workT.id)).then((_w) => this.ref.close(DialogEnums.ActionEnum.DEL));
    }
  }
}
