import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { User } from 'data-access/sdk/api2';
import { firstValueFrom, forkJoin, Observable } from 'rxjs';
import { Department } from 'data-access/sdk/api2';
import { TranslateService } from '@ngx-translate/core';
import { AuthenticationService } from '@core-project/services/authentication.service';
import { UsersService } from 'data-access/sdk/api2';
import { DepartmentsService } from 'data-access/sdk/api2';
import * as _ from 'lodash-es';
import { SelectItemGroup } from 'primeng/api';

@Component({
  selector: 'app-work-to-user-selector',
  templateUrl: './work-to-user-selector.component.html',
  styleUrls: ['./work-to-user-selector.component.scss'],
})
export class WorkToUserSelectorComponent implements OnInit, OnChanges {
  @Input() toUser: number;
  @Output() toUserChange = new EventEmitter<number>();
  @Input() toDepartment: number;
  @Output() toDepartmentChange = new EventEmitter<number>();
  @Input() userSourceMode: 'project' | 'department';
  private user: User;
  private userList$: Observable<Array<User>>;
  private departmentList$: Observable<Array<Department>>;
  public objChoices: SelectItemGroup[];
  public toObj: string;

  constructor(
    private translateService: TranslateService,
    private authenticationService: AuthenticationService,
    private usersService: UsersService,
    private departmentsService: DepartmentsService
  ) {}

  ngOnInit() {
    this.user = this.authenticationService.user;

    if (this.userSourceMode === 'project') {
      this.userList$ = this.usersService.usersList({});
      this.departmentList$ = this.departmentsService.departmentsList({});
    } else {
      this.userList$ = this.departmentsService.departmentsAllUsers(this.user.department);
      this.departmentList$ = this.departmentsService.departmentsSubdepartments(this.user.department);
    }

    firstValueFrom(forkJoin([this.departmentList$, this.userList$])).then((res) => {
      this.objChoices = [
        {
          label: this.translateService.instant('WORK.FORM.DEPARTMENTS'),
          items: res[0].map((d) => ({ label: d.name, value: 'dep_' + d.id.toString() })),
        },
        {
          label: this.translateService.instant('WORK.FORM.USERS'),
          items: res[1].map((u) => ({ label: u.full_name, value: 'user_' + u.id.toString() })),
        },
      ];
    });
  }

  ngOnChanges(changes: SimpleChanges) {
    this.toObj = this.getToObjId(this.toUser, this.toDepartment);
  }

  public onChange() {
    if (this.toObj) {
      const sp = _.split(this.toObj, '_', 2);
      if (sp.length === 2) {
        if (sp[0] === 'user') {
          this.toUserChange.emit(Number(sp[1]));
        }
        if (sp[0] === 'dep') {
          this.toDepartmentChange.emit(Number(sp[1]));
          this.toUserChange.emit(null);
        }
      }
    }
  }

  private getToObjId(toUser: number, toDepartment: number): string {
    if (toUser) {
      return 'user_' + toUser.toString();
    }
    if (toDepartment) {
      return 'dep_' + toDepartment.toString();
    }
    return null;
  }
}
