import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import { WorklinksService } from 'data-access/sdk/api2';
import { WorkLink } from 'data-access/sdk/api2';
import { firstValueFrom } from 'rxjs';
import { SelectItem } from 'primeng/api';
import { WorkEnums } from '@core-project/enums';
import { DialogEnums } from '@core-project/enums';

@Component({
  selector: 'app-worklink-info',
  templateUrl: './worklink-info.component.html',
  styleUrls: ['./worklink-info.component.scss'],
})
export class WorklinkInfoComponent implements OnInit {
  public workLink: WorkLink;
  public workTypesSelect: SelectItem[];

  constructor(
    private ref: DynamicDialogRef,
    private config: DynamicDialogConfig,
    private ts: TranslateService,
    private workLinksService: WorklinksService
  ) {}

  ngOnInit() {
    this.workTypesSelect = [
      { label: this.ts.instant('GANTT.LINK.TYPE.0'), value: WorkEnums.LinkTypeEnum.FTS },
      { label: this.ts.instant('GANTT.LINK.TYPE.1'), value: WorkEnums.LinkTypeEnum.STS },
      { label: this.ts.instant('GANTT.LINK.TYPE.2'), value: WorkEnums.LinkTypeEnum.FTF },
    ];

    firstValueFrom(
      this.workLinksService.worklinksRead({
        id: this.config.data.linkId,
        include: 'source__to_user, source__to_department, target__to_user, target__to_department',
      })
    ).then((wl) => (this.workLink = wl));
  }

  save() {
    firstValueFrom(this.workLinksService.worklinksPartialUpdate({ id: this.workLink.id, data: this.workLink })).then();
  }

  deleteLink() {
    firstValueFrom(this.workLinksService.worklinksDelete(this.workLink.id)).then((_wl) => this.ref.close(DialogEnums.ActionEnum.DEL));
  }
}
