import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { TranslateService } from '@ngx-translate/core';
import { WorklinktemplatesService } from 'data-access/sdk/api2';
import { SelectItem } from 'primeng/api';
import { WorkLinkTemplate } from 'data-access/sdk/api2';
import { DialogEnums, WorkEnums } from '@core-project/enums';
import { firstValueFrom } from 'rxjs';

@Component({
  selector: 'app-worklinktemplate-info',
  templateUrl: './worklinktemplate-info.component.html',
  styleUrls: ['./worklinktemplate-info.component.scss'],
})
export class WorklinktemplateInfoComponent implements OnInit {
  public workLinkTemplate: WorkLinkTemplate;
  public workTypesSelect: SelectItem[];

  constructor(
    private ref: DynamicDialogRef,
    private config: DynamicDialogConfig,
    private ts: TranslateService,
    private workLinkTemplatesService: WorklinktemplatesService
  ) {}

  ngOnInit() {
    this.workTypesSelect = [
      { label: this.ts.instant('GANTT.LINK.TYPE.0'), value: WorkEnums.LinkTypeEnum.FTS },
      { label: this.ts.instant('GANTT.LINK.TYPE.1'), value: WorkEnums.LinkTypeEnum.STS },
      { label: this.ts.instant('GANTT.LINK.TYPE.2'), value: WorkEnums.LinkTypeEnum.FTF },
    ];

    firstValueFrom(
      this.workLinkTemplatesService.worklinktemplatesRead({
        id: this.config.data.linkId,
        include: 'source, target',
      })
    ).then((wl) => (this.workLinkTemplate = wl));
  }

  save() {
    firstValueFrom(
      this.workLinkTemplatesService.worklinktemplatesPartialUpdate({ id: this.workLinkTemplate.id, data: this.workLinkTemplate })
    ).then();
  }

  deleteLink() {
    firstValueFrom(this.workLinkTemplatesService.worklinktemplatesDelete(this.workLinkTemplate.id)).then((_wl) =>
      this.ref.close(DialogEnums.ActionEnum.DEL)
    );
  }
}
