import { Component, OnInit } from '@angular/core';
import { DynamicDialogConfig, DynamicDialogRef } from 'primeng/dynamicdialog';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { jsDateToDbString } from '@core-project/helpers';
import { ConfirmationService } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-works-from-template-form',
  templateUrl: './works-from-template-form.component.html',
  styleUrls: ['./works-from-template-form.component.scss'],
})
export class WorksFromTemplateFormComponent implements OnInit {
  public form: FormGroup;

  constructor(
    private ref: DynamicDialogRef,
    private config: DynamicDialogConfig,
    private fb: FormBuilder,
    private ts: TranslateService,
    private confirmationService: ConfirmationService
  ) {}

  ngOnInit() {
    this.form = this.fb.group({
      start_date: [this.config.data.start_date || jsDateToDbString(new Date()), Validators.required],
      delete_existing_works: [false],
    });
  }

  public submit() {
    if (this.form.valid) {
      if (this.form.controls.delete_existing_works.value) {
        console.log('WTF???');
        this.confirmationService.confirm({
          header: this.ts.instant('WORKS.TMPL_FORM.DELETE_WORKS_HEADER'),
          message: this.ts.instant('WORKS.TMPL_FORM.DELETE_WORKS_MSG'),
          icon: 'pi pi-exclamation-triangle text-red-500',
          accept: () => this.ref.close(this.form.value),
        });
      } else {
        this.ref.close(this.form.value);
      }
    }
  }

  public get controls() {
    return this.form.controls;
  }
}
