import {Component, Input, OnInit} from '@angular/core';
import {IssueBase, User, UsersService} from 'data-access/sdk/api2';
import {Actions, ofActionSuccessful, Store} from '@ngxs/store';
import {IssuesActions} from 'data-access/store';
import {takeUntil} from 'rxjs/operators';
import {Unsubscriber} from 'utils/shared-services';

@Component({
  selector: 'app-issues-widget',
  templateUrl: './issues-widget.component.html',
  styleUrls: ['./issues-widget.component.scss'],
  providers: [Unsubscriber],
})
export class IssuesWidgetComponent implements OnInit {
  private readonly updateObserverSub = this.actions
    .pipe(
      takeUntil(this.unsubscriber.destroy$),
      ofActionSuccessful(IssuesActions.UpdateIssue, IssuesActions.DeleteIssue, IssuesActions.CreateIssue),
    )
    .subscribe(() => {
      this.loadIssues();
    });

  @Input() user: User;

  issues: Array<IssueBase>;
  loading = true;


  constructor(
    private usersService: UsersService,
    private readonly store: Store,
    private readonly actions: Actions,
    private readonly unsubscriber: Unsubscriber,
  ) {
  }

  ngOnInit(): void {
    this.loadIssues();
  }

  loadIssues() {
    this.usersService.usersIssues(this.user.id)
      .subscribe((issues) => {
        this.issues = issues;
        this.loading = false;
      });
  }

  openIssue(issue: IssueBase) {
    this.store.dispatch(new IssuesActions.ShowIssueInSidebar(issue.id)).subscribe();
  }
}
