import { Component, Input, OnInit } from '@angular/core';
import { User } from 'data-access/sdk/api2';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { ProjectEnums } from '@core-project/enums/project';
import { Project } from 'data-access/sdk/api2';
import { UsersService } from 'data-access/sdk/api2';
import { UserRole } from 'data-access/sdk/api2';
import { checkEnumValue } from '@core-project/helpers';

@Component({
  selector: 'app-projects-widget',
  templateUrl: './projects-widget.component.html',
  styleUrls: ['./projects-widget.component.scss'],
})
export class ProjectsWidgetComponent implements OnInit {
  @Input() user: User;
  selectedFilter = 'active';

  allProjects: Array<Project>;
  activeProjects: Array<Project>;
  loading = true;

  constructor(private usersService: UsersService, private router: Router, private ts: TranslateService) {}

  ngOnInit(): void {
    this.loadProjects();
  }

  loadProjects() {
    this.usersService.usersProjects(this.user.id).subscribe((upr) => {
      this.allProjects = upr.projects;
      const activeStatuses = [ProjectEnums.StatusEnum.FUT, ProjectEnums.StatusEnum.ACT];
      this.activeProjects = upr.projects.filter((p) => activeStatuses.includes(checkEnumValue(ProjectEnums.StatusEnum, p.status)));
      this.loading = false;
    });
  }

  getUserRoleName(userRoles: Array<UserRole>): string {
    const uur = userRoles.find((ur) => ur.user === this.user.id);
    return uur.custom_name || uur.rel_role.name;
  }

  goToProject(projectId: number) {
    this.router.navigate(['/project', projectId, 'details']);
  }

  changeFilter(f: string) {
    this.selectedFilter = f;
  }
}
