import {Component, Input, OnInit} from '@angular/core';
import {User, UsersService, Work, WorkBase, WorksService} from 'data-access/sdk/api2';
import {DialogService} from 'primeng/dynamicdialog';
import {TranslateService} from '@ngx-translate/core';
import {firstValueFrom} from 'rxjs';
import {KanbanBoardDialogComponent} from '@core-project/modules/shared-components/works/kanban-board-dialog/kanban-board-dialog.component';
import * as _ from 'lodash-es';
import {Store} from '@ngxs/store';
import {WorksActions} from 'data-access/store';

@Component({
  selector: 'app-works-widget',
  templateUrl: './works-widget.component.html',
  styleUrls: ['./works-widget.component.scss'],
})
export class WorksWidgetComponent implements OnInit {
  @Input() user: User;

  works: Array<WorkBase>;
  loading = true;
  loadingAllWorks = false;

  constructor(
    private usersService: UsersService,
    private worksService: WorksService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private readonly store: Store,
  ) {
  }

  ngOnInit(): void {
    this.loadWorks();
  }

  loadWorks() {
    this.loading = true;
    this.usersService.usersActiveWorks(this.user.id).subscribe((works) => {
      this.works = works;
      this.loading = false;
    });
  }

  openKanban() {
    this.loadingAllWorks = true;
    firstValueFrom(this.usersService.usersWorks(this.user.id)).then((works) => {
      this.loadingAllWorks = false;
      const ref = this.dialogService.open(KanbanBoardDialogComponent, {
        header: this.translateService.instant('WIDGETS.WORKS.HEADER'),
        width: '80%',
        data: {
          works: works,
        },
      });

      firstValueFrom(ref.onClose).then((_r) => this.loadWorks());
    });
  }

  openWork(work: Work) {
    this.store.dispatch(new WorksActions.ShowWorkInSidebar(work.id)).subscribe();
    // if (r === DialogEnums.ActionEnum.DEL) {
    //   this.removeWork(work);
    // } else {
    //   this.updateWork(work);
    // }
  }

  private removeWork(work: Work) {
    _.remove(this.works, ['id', work.id]);
  }

  private updateWork(work: Work) {
    firstValueFrom(this.worksService.worksRead({id: work.id, include: 'to_user'})).then((w) => Object.assign(work, w));
  }
}
