import { formatNumber } from '@angular/common';
import { TranslateService } from '@ngx-translate/core';
import { inject } from '@angular/core';

export interface TruncatedNumber {
  num: number;
  suffix: string;
}

const extSuffixes = {
  none: { suffix: 'NON', divisor: 1 },
  thousands: { suffix: 'THO', divisor: Math.pow(10, 3) },
  millions: { suffix: 'MIL', divisor: Math.pow(10, 6) },
  billions: { suffix: 'BIL', divisor: Math.pow(10, 9) },
};

export function truncateNumber(n: number, extension: string = 'auto') {
  if (!(extension in extSuffixes) || extension === 'auto') {
    extension = detectNumExt(n);
  }
  return { num: n / extSuffixes[extension].divisor, suffix: extSuffixes[extension].suffix };
}

function detectNumExt(n: number) {
  const nstr = String(Math.abs(Math.trunc(n)));
  if (nstr.length > 9) {
    return 'billions';
  }
  if (nstr.length > 6) {
    return 'millions';
  }
  if (nstr.length > 3) {
    return 'thousands';
  }
  return 'none';
}
