import { Inject, LOCALE_ID, Pipe, PipeTransform } from '@angular/core';
import { formatNumber } from '@angular/common';
import { truncateNumber } from '@core-project/pipes/helpers';
import { TranslateService } from '@ngx-translate/core';

@Pipe({
  name: 'hnumber',
})
export class HnumberPipe implements PipeTransform {
  constructor(@Inject(LOCALE_ID) private locale: string, private translateService: TranslateService) {}

  transform(
    n: number | string,
    extension: string = 'auto',
    digitsInfo: string = '1.1-1',
    nullAsZero: boolean = true,
    zeroAsNull: boolean = false
  ): string {
    if (typeof n === 'string') {
      n = Number(n);
    }
    if ((n === null || n === undefined) && !nullAsZero) {
      return null;
    }
    if ((n === 0) && zeroAsNull) {
      return null;
    }
    const trNum = truncateNumber(n, extension);
    const suffix = extension === 'none' ? '' : '\xa0' + this.translateService.instant('SUFFIXES.' + trNum.suffix);
    return formatNumber(trNum.num, this.locale, digitsInfo) + suffix;
  }
}
