import { Injectable } from '@angular/core';
import { Subject, Observable, BehaviorSubject } from 'rxjs';
import { MenuItem } from 'primeng/api';
import * as _ from 'lodash-es';

@Injectable()
export class BreadcrumbService {
  itemsSource = new BehaviorSubject<MenuItem[]>([]);

  itemsHandler = this.itemsSource.asObservable();

  setItems(items: MenuItem[]) {
    this.itemsSource.next(items);
  }

  addItems(items: MenuItem[]) {
    const currentItems = this.itemsSource.value;
    currentItems.push(...items);
    this.setItems(currentItems);
  }

  removeItemsUpTo(key: string, includeKey: boolean = true) {
    let currentItems = this.itemsSource.value;
    let index = _.findIndex(currentItems, ['id', key]);
    if (!includeKey) {
      index += 1;
    }
    if (index !== -1) {
      currentItems = _.slice(currentItems, 0, index);
    }
    this.setItems(currentItems);
  }
}
