import { Injectable } from '@angular/core';
import { DialogService } from 'primeng/dynamicdialog';
import { first } from 'rxjs/operators';

interface ModalImageInterface {
  image: string;
  imageName: string;
}

@Injectable({
  providedIn: 'root',
})
export class DynamicDialogService {
  public currentModalImage: ModalImageInterface = {
    image: '',
    imageName: '',
  };
  public showModalImage = false;

  constructor(public dialogService: DialogService) {}

  showDialog(component, componentProps: { [key: string]: any }, width: string = '600px'): Promise<any> {
    const ref = this.dialogService.open(component, {
      data: componentProps,
      header: componentProps && componentProps.header ? componentProps.header : null,
      showHeader: componentProps && componentProps.showHeader ? componentProps.showHeader : false,
      width,
      baseZIndex: 1000000,
    });

    return ref.onClose.pipe(first()).toPromise();
  }

  public showImage($event): void {
    this.currentModalImage.image = $event.target.src;
    this.currentModalImage.imageName = $event.target.alt;
    this.showModalImage = true;
  }
}
