import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { Loader } from '../interfaces/loader';

@Injectable()
export class LoadingService {
  // private _loading = new BehaviorSubject<boolean>(false);
  // loading = this._loading.asObservable();

  private GLOBAL = 'GLOBAL';
  private loaders: { [name: string]: boolean } = { GLOBAL: false };
  private _loading = new Subject<Loader>();
  loading = this._loading.asObservable();

  register(name?: string): void {
    if (!name) {
      name = this.GLOBAL;
    }
    this.loaders[name] = this.loaders[name] || false;
    this._loading.next({ name, active: this.loaders[name] });
  }

  unregister(name?: string): void {
    if (!name) {
      name = this.GLOBAL;
    }
    this._loading.next({ name, active: false });
    delete this.loaders[name];
  }

  toggle(name?: string) {
    if (!name) {
      name = this.GLOBAL;
    }
    if (name in this.loaders) {
      if (this.loaders[name]) {
        this.dismiss(name);
      } else {
        this.present(name);
      }
    }
  }

  present(name?: string) {
    if (!name) {
      name = this.GLOBAL;
    }
    if (name in this.loaders) {
      this.loaders[name] = true;
      this._loading.next({ name, active: true });
    }
  }

  dismiss(name?: string) {
    if (!name) {
      name = this.GLOBAL;
    }
    if (name in this.loaders) {
      this.loaders[name] = false;
      this._loading.next({ name, active: false });
    }
  }
}
