import { Injectable } from '@angular/core';
import { ReportPlugin } from 'data-access/sdk/api2';
import { catchError } from 'rxjs/operators';
import * as moment from 'moment';
import { getExtensionForFormat } from '@core-project/helpers/report-plugins';
import { MessageService } from 'primeng/api';
import { TranslateService } from '@ngx-translate/core';
import { HttpClient, HttpParams } from '@angular/common/http';
import { ApiConfiguration } from 'data-access/sdk/api2';
import { saveAs } from 'file-saver';

@Injectable({
  providedIn: 'root',
})
export class ReportDownloaderService {
  private reportStarted = false;

  constructor(
    private messageService: MessageService,
    private ts: TranslateService,
    private http: HttpClient,
    protected config: ApiConfiguration
  ) {}

  public getReport(plugin: ReportPlugin, params: object = null) {
    if (this.reportStarted) {
      this.messageService.add({
        summary: this.ts.instant('REPORTS.ERROR_ALREADY_RUNNING'),
        severity: 'error',
      });
      return;
    }

    this.messageService.add({
      summary: this.ts.instant('REPORTS.STARTED'),
      detail: this.ts.instant('REPORTS.STARTED_MSG'),
      severity: 'info',
      sticky: true,
    });
    this.reportStarted = true;

    let requestParams = new HttpParams();
    if (params != null) {
      requestParams = requestParams.set('params_json', JSON.stringify(params));
    }

    return this.http
      .get(this.config.rootUrl + `/report-plugins/${encodeURIComponent(String(plugin.id))}/generate/`, {
        responseType: 'blob',
        params: requestParams,
      })
      .pipe(
        catchError((err) => {
          this.reportStarted = false;
          return err;
        })
      )
      .subscribe((resp) => {
        const now = moment().format('YYYY-MM-DD');
        const filename = now + ' ' + plugin.name + getExtensionForFormat(plugin.format);
        this.messageService.add({
          summary: this.ts.instant('REPORTS.FINISHED'),
          detail: this.ts.instant('REPORTS.FINISHED_MSG', { name: filename }),
          severity: 'success',
          sticky: true,
        });
        saveAs(resp, filename);
        this.reportStarted = false;
      });
  }
}
