import {Injectable} from '@angular/core';
import {BehaviorSubject} from 'rxjs';
import {UserSettingsEnums} from '@core-project/enums';

@Injectable({
  providedIn: 'root',
})
export class UserSettingsService {
  layout = new BehaviorSubject<UserSettingsEnums.LayoutEnum>(undefined);
  theme = new BehaviorSubject<UserSettingsEnums.ThemeEnum>(undefined);

  loadFromLS() {
    const settings = JSON.parse(localStorage.getItem('userSettings'));
    this.layout.next(settings?.layout || UserSettingsEnums.LayoutEnum.static);
    this.setLayout(this.layout.value);

    this.theme.next(settings?.theme || UserSettingsEnums.ThemeEnum.light);
    this.setTheme(this.theme.value);
  }

  saveToLS() {
    const settings = {
      layout: this.layout.value,
      theme: this.theme.value,
    };
    localStorage.setItem('userSettings', JSON.stringify(settings));
  }

  setTheme(theme: UserSettingsEnums.ThemeEnum) {
    const themeLink = document.getElementById('theme-css');
    const urlTokens = themeLink.getAttribute('href').split('/');
    if (theme === UserSettingsEnums.ThemeEnum.light) {
      urlTokens[urlTokens.length - 1] = 'theme-light.css';
      this.theme.next(UserSettingsEnums.ThemeEnum.light);
    } else {
      urlTokens[urlTokens.length - 1] = 'theme-dark.css';
      this.theme.next(UserSettingsEnums.ThemeEnum.dark);
    }

    const newURL = urlTokens.join('/');
    this.replaceLink(themeLink, newURL);
    this.saveToLS();
  }

  setLayout(layout: UserSettingsEnums.LayoutEnum) {
    this.layout.next(layout);
    this.saveToLS();
  }

  private replaceLink(linkElement, href) {
    if (this.isIE()) {
      linkElement.setAttribute('href', href);
    } else {
      const id = linkElement.getAttribute('id');
      const cloneLinkElement = linkElement.cloneNode(true);

      cloneLinkElement.setAttribute('href', href);
      cloneLinkElement.setAttribute('id', id + '-clone');

      linkElement.parentNode.insertBefore(cloneLinkElement, linkElement.nextSibling);

      cloneLinkElement.addEventListener('load', () => {
        linkElement.remove();
        cloneLinkElement.setAttribute('id', id);
      });
    }
  }

  private isIE() {
    return /(MSIE|Trident\/|Edge\/)/i.test(window.navigator.userAgent);
  }
}
