import {DoBootstrap, Injector, NgModule} from '@angular/core';
import {FormsModule} from '@angular/forms';
import {createCustomElement} from '@angular/elements';
import {BrowserModule} from '@angular/platform-browser';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {UiCustomCalendarModule} from 'ui/custom-calendar';
import {AppComponent} from './app.component';
import {StringInfoOverlaysModule} from 'features/string-info-overlays';
import {ApiModule} from 'data-access/sdk/api2';
import {TranslateLoader, TranslateModule} from '@ngx-translate/core';
import {HttpClientModule} from '@angular/common/http';
import {
  PluginAttrService,
  pluginAuthInterceptorProvider,
  pluginHttpLoaderFactory,
  translateAppInitializerProvider,
} from 'utils/plugins-shared';
import {environment} from '../environments/environment';


@NgModule({
  declarations: [
    AppComponent,
  ],
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    HttpClientModule,
    ApiModule,
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: () => pluginHttpLoaderFactory(environment),
      },
    }),
    FormsModule,
    UiCustomCalendarModule,
    StringInfoOverlaysModule,
  ],
  providers: [
    translateAppInitializerProvider,
    pluginAuthInterceptorProvider,
    PluginAttrService,
  ],
})
export class AppModule implements DoBootstrap {
  constructor(
    injector: Injector,
  ) {
    const element = createCustomElement(AppComponent, {
      injector: injector,
    });
    const elementTag = 'dw-demo-widget';

    if (customElements.get(elementTag)) {
      console.warn(`Element ${elementTag} already registered.`);
    } else {
      customElements.define(elementTag, element);
    }
  }

  ngDoBootstrap() {
    // DONT DELETE
  }
}
