import {Injectable} from '@angular/core';
import {Observable, withLatestFrom} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {CommentModel, NewCommentModel} from '../models/comment.model';
import {CommentDTO} from '../models/comment.dto';
import {CommentBaseModel} from '../../comments';
import {CommentBaseAdapterService} from './comment-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class CommentAdapterService extends BaseAdapter<CommentModel, CommentDTO> {
  constructor(
    private readonly commentBaseAdapterService: CommentBaseAdapterService,
  ) {
    super();
  }

  convert(dto: CommentDTO): Observable<CommentModel> {
    const replyTo$: Observable<CommentBaseModel | undefined> = this.createModelObs(dto.rel_reply_to, this.commentBaseAdapterService);

    return this.commentBaseAdapterService.convert(dto)
      .pipe(
        withLatestFrom(replyTo$),
        switchMap(([baseModel, replyTo]) => {
          const model: CommentModel = {
            ...baseModel,

            // relation ids
            replyToId: dto.reply_to ?? undefined,

            // relation models
            replyTo,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: CommentModel): CommentDTO {
    throw new Error('Method not implemented.');
  }

  reverseToNewModel(model: NewCommentModel): CommentDTO {
    return {
      text: model.text,
      author: model.authorId,
      reply_to: model.replyToId,
    };
  }
}
