import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {CommentBaseModel} from 'data-access/sdk/internal-api';
import {CommentBaseDTO} from '../models/comment-base.dto';
import {UserAdapterService} from '../../users/services/user-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class CommentBaseAdapterService extends BaseAdapter<CommentBaseModel, CommentBaseDTO> {
  constructor(
    private readonly userAdapterService: UserAdapterService,
  ) {
    super();
  }

  convert(dto: CommentBaseDTO): Observable<CommentBaseModel> {
    const author$ = this.createModelObs(dto.rel_author, this.userAdapterService);

    return forkJoin([
      author$,
    ])
      .pipe(
        switchMap(([author]) => {
          const model: CommentBaseModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            created: dto.created ?? undefined,
            replyToId: dto.reply_to ?? undefined, // haven't relation model in dto
            text: dto.text ?? undefined,
            translateOptions: dto.translate_options ?? undefined,
            translateString: dto.translate_string ?? undefined,

            // relation ids
            authorId: dto.author ?? undefined,
            companyId: dto.company ?? undefined,

            // relation models
            author,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: CommentBaseModel): CommentBaseDTO {
    throw new Error('Method not implemented.');
  }

}
