import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {CompanyModel} from '../models/company.model';
import {CompanyDTO} from '../models/company.dto';
import {CompanyCurrencyAdapterService} from './company-currency-adapter.service';
import {CompanyIndicatorConstraintsAdapterService} from './company-indicator-constraints-adapter.service';
import {CompanyPreferencesAdapterService} from './company-preferences-adapter.service';
import {CompanyCurrencyModel} from '../models/company-currency.model';
import {CompanyIndicatorConstraintModel} from '../models/company-indicator-constraint.model';
import {CompanyPreferencesModel} from '../models/company-preferences.model';

@Injectable({
  providedIn: 'root',
})
export class CompanyAdapterService extends BaseAdapter<CompanyModel, CompanyDTO> {

  constructor(
    private readonly companyCurrencyAdapterService: CompanyCurrencyAdapterService,
    private readonly companyIndicatorConstraintsAdapterService: CompanyIndicatorConstraintsAdapterService,
    private readonly companyPreferencesAdapterService: CompanyPreferencesAdapterService,
  ) {
    super();
  }

  convert(dto: CompanyDTO): Observable<CompanyModel> {
    const currencies$: Observable<CompanyCurrencyModel[]> = this.createModelArrObs(dto.rel_currencies, this.companyCurrencyAdapterService);

    const indicatorConstraints$: Observable<CompanyIndicatorConstraintModel | undefined> = this.createModelObs(
      dto.rel_indicator_constraints,
      this.companyIndicatorConstraintsAdapterService,
    );

    const preferences$: Observable<CompanyPreferencesModel | undefined> = this.createModelObs(
      dto.rel_preferences,
      this.companyPreferencesAdapterService,
    );

    return forkJoin([
      currencies$,
      indicatorConstraints$,
      preferences$,
    ])
      .pipe(
        switchMap(([currencies, indicatorConstraints, preferences]) => {
          const model: CompanyModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            fullName: dto.full_name ?? undefined,
            legalAddress: dto.legal_address ?? undefined,
            name: dto.name ?? undefined,
            postalAddress: dto.postal_address ?? undefined,

            // relation ids
            currenciesIds: dto.currencies ?? [],
            indicatorConstraintsId: dto.indicator_constraints ?? undefined,
            parentCompanyId: dto.parent_company ?? undefined,
            preferencesId: dto.preferences ?? undefined,

            // relation models
            currencies,
            indicatorConstraints,
            preferences,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: CompanyModel): CompanyDTO {
    throw new Error('Method not implemented.');
  }
}
