import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ContactModel} from '../models/contact.model';
import {ContactDTO} from '../models/contact.dto';
import {EmailAdapterService} from '../../emails/services/email-adapter.service';
import {TelephoneAdapterService} from '../../telephones/services/telephone-adapter.service';
import {EmailModel} from '../../emails';
import {TelephoneModel} from '../../telephones';

@Injectable({
  providedIn: 'root',
})
export class ContactAdapterService extends BaseAdapter<ContactModel, ContactDTO> {
  constructor(
    private readonly emailAdapterService: EmailAdapterService,
    private readonly telephoneAdapterService: TelephoneAdapterService,
  ) {
    super();
  }

  convert(dto: ContactDTO): Observable<ContactModel> {
    const emails$: Observable<EmailModel[]> = this.createModelArrObs(dto.rel_emails, this.emailAdapterService);
    const telephones$: Observable<TelephoneModel[]> = this.createModelArrObs(dto.rel_telephones, this.telephoneAdapterService);

    return forkJoin([
      emails$,
      telephones$,
    ])
      .pipe(
        switchMap(([emails, telephones]) => {
          const model: ContactModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            comment: dto.comment ?? undefined,
            firstName: dto.first_name ?? this.convertError('Convert error: first_name is not defined'),
            fullName: dto.full_name ?? undefined,
            lastName: dto.last_name ?? undefined,
            middleName: dto.middle_name ?? undefined,
            organizationName: dto.organization_name ?? undefined,
            photo: dto.photo ?? undefined,
            position: dto.position ?? undefined,
            shortName: dto.short_name ?? undefined,

            // relation ids
            emailsIds: dto.emails ?? [],
            telephonesIds: dto.telephones ?? [],
            organizationId: dto.organization ?? this.convertError('Convert error: organization is not defined'),

            // relation models
            emails,
            telephones,
          };

          return this.createConvertObservable(model);
        }),
      );

  }

  reverse(model: ContactModel): ContactDTO {
    throw new Error('Method not implemented.');
  }
}
