import {Injectable} from '@angular/core';
import {switchMap} from 'rxjs/operators';

import {PluginsService} from 'data-access/sdk/api2';
import {DashboardPluginAdapterService} from './dashboard-plugin-adapter.service';
import {catchError, forkJoin, of} from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class DashboardPluginsApiService {
  constructor(
    private readonly apiService: PluginsService,
    private readonly adapterService: DashboardPluginAdapterService,
  ) {
  }

  readList(include?: string) {
    return this.apiService.pluginsDashboardPluginsList({include})
      .pipe(
        switchMap((plugins) => forkJoin(
          plugins.map((plugin) => this.adapterService.convert(plugin)),
        )),
        catchError(() => of([])),
      );
  }

  read(id: number) {
    return this.apiService.pluginsDashboardPluginsRead({id})
      .pipe(
        switchMap((plugin) => this.adapterService.convert(plugin)),
      );
  }
}
