import {Injectable} from '@angular/core';
import {Observable, withLatestFrom} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {DepartmentModel} from '../models/department.model';
import {DepartmentDTO} from '../models/department.dto';
import {DepartmentBaseAdapterService} from './department-base-adapter.service';
import {UserBaseModel} from '../../users';
import {UserBaseAdapterService} from '../../users/services/user-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class DepartmentAdapterService extends BaseAdapter<DepartmentModel, DepartmentDTO> {

  constructor(
    private readonly departmentBaseAdapterService: DepartmentBaseAdapterService,
    private readonly userBaseAdapterService: UserBaseAdapterService,
  ) {
    super();
  }

  convert(dto: DepartmentDTO): Observable<DepartmentModel> {
    const employees$: Observable<UserBaseModel[]> = this.createModelArrObs(dto.rel_employees, this.userBaseAdapterService);

    return this.departmentBaseAdapterService.convert(dto)
      .pipe(
        withLatestFrom(employees$),
        switchMap(([baseModel, employees]) => {
          const model: DepartmentModel = {
            ...baseModel,
            parentDepartment: dto.parent_department ?? undefined,
            subDepartments: dto.subdepartments ?? [],

            // relation ids
            employeesIds: dto.employees ?? [],

            // relation models
            employees,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: DepartmentModel): DepartmentDTO {
    throw new Error('Method not implemented.');
  }
}
