import {Injectable} from '@angular/core';
import {switchMap} from 'rxjs/operators';
import {forkJoin, of} from 'rxjs';

import {DepartmentsService} from 'data-access/sdk/api2';

import {DepartmentAdapterService} from './department-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class DepartmentsApiService {
  constructor(
    private readonly departmentsService: DepartmentsService,
    private readonly departmentAdapterService: DepartmentAdapterService,
  ) {
  }

  readDepartments(params: DepartmentsService.DepartmentsListParams = {}) {
    return this.departmentsService.departmentsList(params)
      .pipe(
        switchMap((departments) => {
          if (departments.length > 0) {
            return forkJoin(departments.map((department) => this.departmentAdapterService.convert(department)));
          }

          return of([]);
        }),
      );
  }
}
