import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {BudgetTypeModel} from '../models/budget-type.model';
import {BudgetTypeDTO} from '../models/budget-type.dto';
import {BudgetCategoryAdapterService} from './budget-category-adapter.service';
import {BudgetCategoryModel} from '../models/budget-category.model';

@Injectable({
  providedIn: 'root',
})
export class BudgetTypeAdapterService extends BaseAdapter<BudgetTypeModel, BudgetTypeDTO> {
  constructor(
    private readonly budgetCategoryAdapterService: BudgetCategoryAdapterService,
  ) {
    super();
  }

  convert(dto: BudgetTypeDTO): Observable<BudgetTypeModel> {
    const budgetCategory$: Observable<BudgetCategoryModel | undefined> = this.createModelObs(
      dto.rel_category,
      this.budgetCategoryAdapterService,
    );

    return forkJoin([
      budgetCategory$,
    ])
      .pipe(
        switchMap(([category]) => {
          const model: BudgetTypeModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            code: dto.code ?? undefined,
            name: dto.name ?? this.convertError('Convert error: name is not defined'),
            order: dto.order ?? 0,

            // relation ids
            categoryId: dto?.id ?? 0,
            projectBudgetTypes: dto.project_budget_types ?? [],

            // relation models
            category,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: BudgetTypeModel): BudgetTypeDTO {
    throw new Error('Method not implemented.');
  }
}
