import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectBudgetTypeModel} from '../models/project-budget-type.model';
import {ProjectBudgetTypeDTO} from '../models/project-budget-type.dto';
import {BudgetStatsAdapterService} from './budget-stats-adapter.service';
import {BudgetTypeAdapterService} from './budget-type-adapter.service';
import {BudgetTypeModel} from '../models/budget-type.model';
import {BudgetStatsModel} from '../models/budget-stats.model';

@Injectable({
  providedIn: 'root',
})
export class ProjectBudgetTypeAdapterService extends BaseAdapter<ProjectBudgetTypeModel, ProjectBudgetTypeDTO> {
  constructor(
    private readonly budgetStatsAdapterService: BudgetStatsAdapterService,
    private readonly budgetTypeAdapterService: BudgetTypeAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectBudgetTypeDTO): Observable<ProjectBudgetTypeModel> {
    const budgetType$: Observable<BudgetTypeModel | undefined> = this.createModelObs(dto.rel_budget_type, this.budgetTypeAdapterService);
    const stats$: Observable<BudgetStatsModel | undefined> = this.createModelObs(dto.stats, this.budgetStatsAdapterService);

    return forkJoin([
      budgetType$,
      stats$,
    ])
      .pipe(
        switchMap(([budgetType, stats]) => {
          const model: ProjectBudgetTypeModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            code: dto.code ?? undefined,
            name: dto.name ?? this.convertError('Convert error: name is not defined'),
            stats,
            totalPlanned: dto.total_planned ?? undefined,
            totalPlannedCurrency: dto.total_planned_currency ?? undefined,

            // relation ids
            budgetTypeId: dto.budget_type,
            projectId: dto.project ?? this.convertError('Convert error: project (id) is not defined'),

            // relation models
            budgetType,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: ProjectBudgetTypeModel): ProjectBudgetTypeDTO {
    throw new Error('Method not implemented.');
  }
}
