import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {CRUDEventDTO} from '../models/crud-event.dto';
import {HistoryActionModel} from '../models/history-action.model';
import {UserAdapterService} from '../../users/services/user-adapter.service';
import {ContentTypeAdapterService} from './content-type-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class HistoryActionAdapterService extends BaseAdapter<HistoryActionModel, CRUDEventDTO> {

  constructor(
    private readonly userAdapterService: UserAdapterService,
    private readonly contentTypeAdapterService: ContentTypeAdapterService,
  ) {
    super();
  }

  convert(dto: CRUDEventDTO): Observable<HistoryActionModel> {
    const user$ = this.createModelObs(dto.user, this.userAdapterService);
    const contentType$ = this.createModelObs(dto.content_type, this.contentTypeAdapterService);

    return forkJoin([
      user$,
      contentType$,
    ])
      .pipe(
        switchMap(([user, contentType]) => {
          const model: HistoryActionModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            changedFields: dto.changed_fields ? JSON.parse(dto.changed_fields) : undefined,
            contentType,
            datetime: dto.datetime ?? this.convertError('Convert error: datetime is not defined'),
            eventType: dto.event_type,
            objectId: dto.object_id,
            objectJsonRepr: dto.object_json_repr ?? undefined,
            objectRepr: dto.object_repr ?? undefined,
            user,
            userPkAsString: dto.user_pk_as_string ?? undefined,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: HistoryActionModel): CRUDEventDTO {
    throw new Error('Method not implemented.');
  }
}
