import {Injectable} from '@angular/core';
import {map, switchMap} from 'rxjs/operators';
import {forkJoin, Observable} from 'rxjs';

import {HistoryService} from 'data-access/sdk/api2';

import {HistoryActionAdapterService} from './history-action-adapter.service';
import {HistoryActionModel} from '../models/history-action.model';

@Injectable({
  providedIn: 'root',
})
export class HistoryApiService {

  constructor(
    private readonly historyService: HistoryService,
    private readonly historyActionAdapterService: HistoryActionAdapterService,
  ) {
  }

  readIssueHistory(id: number): Observable<HistoryActionModel[]> {
    return this.historyService.historyList({contentTypeModel: 'issue', objectId: id.toString()})
      .pipe(
        switchMap((historyActionsDTO) => forkJoin(
          historyActionsDTO.map((historyActionDTO) => this.historyActionAdapterService.convert(historyActionDTO)),
        )),
        map((historyAction) => historyAction
          .sort((a, b) => {
            const dateA = new Date(a.datetime);
            const dateB = new Date(b.datetime);

            return dateA.getTime() - dateB.getTime();
          })),
      );
  }

  readWorkHistory(id: number): Observable<HistoryActionModel[]> {
    return this.historyService.historyList({contentTypeModel: 'work', objectId: id.toString()})
      .pipe(
        switchMap((historyActionsDTO) => forkJoin(
          historyActionsDTO.map((historyActionDTO) => this.historyActionAdapterService.convert(historyActionDTO)),
        )),
        map((historyAction) => historyAction
          .sort((a, b) => {
            const dateA = new Date(a.datetime);
            const dateB = new Date(b.datetime);

            return dateA.getTime() - dateB.getTime();
          }),
        ),
      );
  }
}
