import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';

import {BaseAdapter} from 'utils/shared-services';
import {IssueStatusEnum} from '../../issues';
import {IssueBaseModel} from '../models/issue-base.model';
import {IssueBaseDTO} from '../models/issue-base.dto';

@Injectable({
  providedIn: 'root',
})
export class IssueBaseAdapterService extends BaseAdapter<IssueBaseModel, IssueBaseDTO> {
  constructor() {
    super();
  }

  convert(dto: IssueBaseDTO): Observable<IssueBaseModel> {
    const model: IssueBaseModel = {
      id: dto.id ?? this.convertError('Convert error: id is not defined'),
      createdAt: dto.created_at ?? this.convertError('Convert error: created_at is not defined'),
      description: dto.description ?? undefined,
      name: dto.name ?? this.convertError('Convert error: name is not defined'),
      order: dto.order ?? 0,
      projectName: dto.project_name ?? undefined,
      resolveDate: dto.resolve_date ?? undefined,
      response: dto.response ?? undefined,
      severity: dto.severity ?? 0,
      status: this.convertIssueStatus(dto.status),
    };

    return this.createConvertObservable(model);
  }

  reverse(model: IssueBaseModel): IssueBaseDTO {
    return {
      description: model.description ?? null,
      created_at: model.createdAt,
      name: model.name,
      order: model.order,
      resolve_date: model.resolveDate ?? null,
      severity: model.severity,
      status: model.status,
    };
  }

  private convertIssueStatus(status: IssueBaseDTO['status']): IssueBaseModel['status'] {
    switch (status) {
      case 'ACT':
        return IssueStatusEnum.ACT;
      case 'SLV':
        return IssueStatusEnum.SLV;
      default:
        return undefined;
    }
  }
}
