import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectBudgetModel} from '../../finances/models/project-budget.model';
import {ProjectBudgetDTO} from '../../finances/models/project-budget.dto';
import {BudgetStatsAdapterService} from '../../finances/services/budget-stats-adapter.service';
import {BudgetStatsModel} from '../../finances/models/budget-stats.model';

@Injectable({
  providedIn: 'root',
})
export class ProjectBudgetAdapterService extends BaseAdapter<ProjectBudgetModel, ProjectBudgetDTO> {
  constructor(
    private readonly budgetStatsAdapterService: BudgetStatsAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectBudgetDTO): Observable<ProjectBudgetModel> {
    const stats$: Observable<BudgetStatsModel | undefined> = this.createModelObs(dto.stats, this.budgetStatsAdapterService);

    return forkJoin([
      stats$,
    ])
      .pipe(
        switchMap(([stats]) => {
          const model: ProjectBudgetModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            isApproved: dto.approved ?? false,
            approvedAt: dto.approved_at ?? undefined,
            approvedBy: dto.approved_by ?? undefined,
            code: dto.code ?? undefined,
            effectFact: dto.effect_fact ?? undefined,
            effectForecast: dto.effect_forecast ?? undefined,
            effectPlan: dto.effect_plan ?? undefined,
            irrFact: dto.irr_fact ?? undefined,
            irrForecast: dto.irr_forecast ?? undefined,
            irrPlan: dto.irr_plan ?? undefined,
            npvFact: dto.npv_fact ?? undefined,
            npvForecast: dto.npv_forecast ?? undefined,
            npvPlan: dto.npv_plan ?? undefined,
            periodEnd: dto.period_end ?? undefined,
            periodStart: dto.period_start ?? undefined,
            stats,
            totalPlanned: dto.total_planned ?? undefined,
            totalPlannedCurrency: dto.total_planned_currency ?? undefined,

            // relation ids
            projectId: dto.project ?? this.convertError('Convert error: project (id) is not defined'),
          };
          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: ProjectBudgetModel): ProjectBudgetDTO {
    throw new Error('Method not implemented.');
  }
}
