import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';

import {BaseAdapter} from 'utils/shared-services';

import {WorkBaseModel, WorkModel, WorkStatusEnum} from '../../works';
import {LinkModel} from '../models/link.model';
import {LinkTypeEnum} from '../models/link-type.enum';
import {IssueBaseModel, IssueStatusEnum} from '../../issues';

@Injectable({
  providedIn: 'root',
})
export class LinksAdapterService extends BaseAdapter<LinkModel, WorkBaseModel | IssueBaseModel> {
  convert(work: WorkModel): Observable<LinkModel> {
    throw new Error('Method not implemented.');
  }

  convertFromWork(work: WorkBaseModel, isParent: boolean = false): Observable<LinkModel> {
    const model: LinkModel = {
      id: work.id,
      svgIcon: this.convertWorkIcon(work.status),
      isParent,
      name: work.name,
      type: LinkTypeEnum.Work,
      iconColor: this.convertWorkIconColor(work.status),
    };

    return this.createConvertObservable(model);
  }

  convertFromIssue(issue: IssueBaseModel, isParent: boolean = false): Observable<LinkModel> {
    const model: LinkModel = {
      id: issue.id,
      svgIcon: this.convertIssueIcon(issue.status),
      isParent,
      name: issue.name,
      type: LinkTypeEnum.Issue,
      iconColor: this.convertIssueIconColor(issue.status),
    };

    return this.createConvertObservable(model);
  }

  reverse(model: LinkModel): WorkModel {
    throw new Error('Method not implemented.');
  }

  private convertWorkIcon(status: WorkStatusEnum | undefined): string {
    switch (status) {
      case WorkStatusEnum.BL:
        return 'ci-backlog';
      case WorkStatusEnum.CR:
        return 'ci-todo';
      case WorkStatusEnum.WP:
        return 'ci-in-progress';
      case WorkStatusEnum.DN:
        return 'ci-on-check';
      case WorkStatusEnum.AC:
        return 'ci-completed';
      case WorkStatusEnum.CN:
        return 'ci-cancelled';
      case WorkStatusEnum.PS:
        return 'ci-paused';
      default:
        console.error('Unknown status: ' + status);
        // TODO #errorService
        return 'default';
    }
  }

  private convertIssueIcon(status: IssueStatusEnum | undefined): string {
    switch (status) {
      case IssueStatusEnum.ACT:
        return 'ci-fire';
      case IssueStatusEnum.SLV:
        return 'ci-fire';
      default:
        console.error('Unknown status: ' + status);
        // TODO #errorService
        return 'ci-fire';
    }
  }

  private convertWorkIconColor(status: WorkStatusEnum | undefined): string {
    switch (status) {
      case WorkStatusEnum.CR:
        return '#A871EF';
      case WorkStatusEnum.WP:
        return '#1677CC';
      case WorkStatusEnum.DN:
        return '#FB9600';
      case WorkStatusEnum.AC:
        return '#71AF74';
      case WorkStatusEnum.CN:
        return '#DA4A4A';
      default:
        console.error('Unknown status: ' + status);
        // TODO #errorService
        return 'var(--uikit-text-secondary)';
    }
  }

  private convertIssueIconColor(status: IssueStatusEnum | undefined): string {
    switch (status) {
      case IssueStatusEnum.ACT:
        return '#DA4A4A';
      case IssueStatusEnum.SLV:
        return '#71AF74';
      default:
        console.error('Unknown status: ' + status);
        // TODO #errorService
        return 'var(--uikit-text-secondary)';
    }
  }
}
