import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {map, switchMap} from 'rxjs/operators';

import {NotificationsService} from 'data-access/sdk/api2';

import {NotificationAdapterService} from './notification-adapter.service';
import {NotificationDTO} from '../models/notification.dto';
import {NewNotificationModel, NotificationModel} from '../models/notification.model';
import {NotificationsCountResponseInterface} from '../models/notifications-count-response.interface';

@Injectable({
  providedIn: 'root',
})
export class NotificationApiService {
  constructor(
    private readonly notificationAdapterService: NotificationAdapterService,
    private readonly notificationsService: NotificationsService,
  ) {
  }

  getNotifications(pageIndex: number, pageSize: number): Observable<NotificationModel[]> {
    return this.notificationsService.notificationsList({limit: pageSize, offset: pageIndex * pageSize})
      .pipe(
        switchMap((notifications) => forkJoin([
          ...notifications.results.map((notificationDTO) => this.notificationAdapterService.convert(notificationDTO)),
        ])),
      );
  }

  getNewNotificationsCount(): Observable<number> {
    return this.notificationsService.notificationsUnreadCount()
      .pipe(
        // adapter from SDK (undefined can't be in reality)
        map((notifications) => ({
          unread: notifications.unread ?? 0,
        }) as NotificationsCountResponseInterface),
        map((notifications) => notifications.unread),
      );
  }

  markAsRead(id: number): Observable<unknown> {
    return this.notificationsService.notificationsMarkAsRead(id);
  }

  markAllAsRead(): Observable<unknown> {
    return this.notificationsService.notificationsMarkAllAsRead();
  }

  sendTestNotification(notification: NewNotificationModel): Observable<unknown> {
    const notificationDTO: NotificationDTO = this.notificationAdapterService.reverse(notification);
    return this.notificationsService.notificationsTest(notificationDTO);
  }
}
