import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {PartnerModel} from '../models/partner.model';
import {PartnerDTO} from '../models/partner.dto';
import {CompanyAdapterService} from '../../companies/services/company-adapter.service';
import {ContactAdapterService} from '../../contacts/services/contact-adapter.service';
import {EmailAdapterService} from '../../emails/services/email-adapter.service';
import {TelephoneAdapterService} from '../../telephones/services/telephone-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class PartnerAdapterService extends BaseAdapter<PartnerModel, PartnerDTO> {
  constructor(
    private readonly companyAdapterService: CompanyAdapterService,
    private readonly contactAdapterService: ContactAdapterService,
    private readonly emailAdapterService: EmailAdapterService,
    private readonly telephoneAdapterService: TelephoneAdapterService,
  ) {
    super();
  }

  convert(dto: PartnerDTO): Observable<PartnerModel> {
    const company$: Observable<PartnerModel['company']> = this.createModelObs(dto.rel_company, this.companyAdapterService);
    const contacts$: Observable<PartnerModel['contacts']> = this.createModelArrObs(
      dto.rel_contacts,
      this.contactAdapterService,
    );
    const emails$: Observable<PartnerModel['emails']> = this.createModelArrObs(
      dto.rel_emails,
      this.emailAdapterService,
    );
    const telephones$: Observable<PartnerModel['telephones']> = this.createModelArrObs(
      dto.rel_telephones,
      this.telephoneAdapterService,
    );

    return forkJoin([
      company$,
      contacts$,
      emails$,
      telephones$,
    ])
      .pipe(
        switchMap(([company, contacts, emails, telephones]) => {
          const model: PartnerModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            details: dto.details ?? undefined,
            fullName: dto.full_name ?? undefined,
            legalAddress: dto.legal_address ?? undefined,
            name: dto.name ?? this.convertError('Convert error: name is not defined'),
            postalAddress: dto.postal_address ?? undefined,

            // relation ids
            companyId: dto.company ?? undefined,
            contactsIds: dto.contacts ?? [],
            emailsIds: dto.emails ?? [],
            telephonesIds: dto.telephones ?? [],

            // relation models
            company,
            contacts,
            emails,
            telephones,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: PartnerModel): PartnerDTO {
    throw new Error('Method not implemented.');
  }
}
