import {Injectable} from '@angular/core';
import {switchMap} from 'rxjs/operators';
import {catchError, forkJoin, of} from 'rxjs';

import {PluginsService} from 'data-access/sdk/api2';
import {PluginAdapterService} from './plugin-adapter.service';
import {NewPluginInterface, PluginModel} from '../models/plugin.model';

@Injectable({
  providedIn: 'root',
})
export class PluginsApiService {
  constructor(
    private readonly apiService: PluginsService,
    private readonly adapterService: PluginAdapterService,
  ) {
  }

  create(plugin: NewPluginInterface) {
    const data = {
      name: plugin.name,
      pkgFile: plugin.file,
      enabled: plugin.isEnabled,
    };
    return this.apiService.pluginsDashboardPluginsCreate(data);
  }

  read(id: number, include?: string) {
    return this.apiService.pluginsDashboardPluginsRead({id, include})
      .pipe(
        switchMap((plugin) => this.adapterService.convert(plugin)),
      );
  }

  readList(include?: string) {
    return this.apiService.pluginsDashboardPluginsList({include})
      .pipe(
        switchMap((plugins) => forkJoin(
          plugins.map((plugin) => this.adapterService.convert(plugin)),
        )),
        catchError(() => of([])),
      );
  }

  delete(id: number) {
    return this.apiService.pluginsDashboardPluginsDelete(id);
  }

  update(plugin: PluginModel) {
    const data = {
      id: plugin.id,
      name: plugin.name,
      enabled: plugin.isEnabled,
      order: plugin.order,
      pkgFile: plugin.file,
    };
    return this.apiService.pluginsDashboardPluginsPartialUpdate(data);
  }

  changePluginStatus(plugin: PluginModel) {
    const data = {id: plugin.id, name: plugin.name, enabled: plugin.isEnabled};
    return this.apiService.pluginsDashboardPluginsPartialUpdate(data);
  }
}
