import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectMetaValueModel} from '../models/project-meta-value.model';
import {ProjectMetaValueDTO} from '../models/project-meta-value.dto';
import {ProjectMetaAdapterService} from './project-meta-adapter.service';
import {ProjectMetaModel} from '../models/project-meta.model';

@Injectable({
  providedIn: 'root',
})
export class ProjectMetaValueAdapterService extends BaseAdapter<ProjectMetaValueModel, ProjectMetaValueDTO> {
  constructor(
    private readonly projectMetaAdapterService: ProjectMetaAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectMetaValueDTO): Observable<ProjectMetaValueModel> {
    const meta$: Observable<ProjectMetaModel | undefined> = this.createModelObs(
      dto.rel_meta,
      this.projectMetaAdapterService,
    );

    return forkJoin([
      meta$,
    ])
      .pipe(
        switchMap(([meta]) => {
          const model: ProjectMetaValueModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            created: dto.created ?? undefined,
            project: dto.project ?? undefined,
            updated: dto.updated ?? undefined,
            value_CH: dto.value_CH ?? undefined,
            value_CHS: dto.value_CHS ?? undefined,
            value_DT: dto.value_DT ?? undefined,
            value_FLAG: dto.value_FLAG ?? undefined,
            value_FLT: dto.value_FLT ?? undefined,
            value_INT: dto.value_INT ?? undefined,
            value_MCHS: dto.value_MCHS ?? [],
            value_MONEY: dto.value_MONEY ?? undefined,
            value_MONEY_currency: dto.value_MONEY_currency ?? undefined,
            value_TXT: dto.value_TXT ?? undefined,
            value_URL: dto.value_URL ?? undefined,

            // relation ids
            metaId: dto.meta ?? undefined,

            // relation models
            meta,
          };

          return this.createConvertObservable(model);
        }),
      );


  }

  reverse(model: ProjectMetaValueModel): ProjectMetaValueDTO {
    throw new Error('Method not implemented.');
  }
}
