import { Injectable } from '@angular/core';
import { Observable, forkJoin, of, switchMap } from 'rxjs';

import { BaseAdapter } from 'utils/shared-services';

import { ProjectMilestoneDTO } from '../models/project-milestone.dto';
import { ProjectMilestoneModel } from '../models/project-milestone.model';
import { ProjectMilestoneIndicatorEnum } from '../models/project-milestone-indicator.enum';
import { ProjectMilestoneStatusEnum } from '../models/project-milestone-status.enum';
import { DepartmentModel } from '../../departments';
import { DepartmentAdapterService } from '../../departments/services/department-adapter.service';
import { UserModel } from '../../users';
import { UserAdapterService } from '../../users/services/user-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class ProjectMilestoneAdapterService extends BaseAdapter<ProjectMilestoneModel, ProjectMilestoneDTO> {
  constructor(private readonly userAdapter: UserAdapterService, private readonly departmentAdapter: DepartmentAdapterService) {
    super();
  }
  override convert(dto: ProjectMilestoneDTO): Observable<ProjectMilestoneModel> {
    const toUser$: Observable<UserModel | undefined> = dto.rel_to_user ? this.userAdapter.convert(dto.rel_to_user) : of(undefined);
    const toDepartment$: Observable<DepartmentModel | undefined> = dto.rel_to_department
      ? this.departmentAdapter.convert(dto.rel_to_department)
      : of(undefined);

    return forkJoin([toUser$, toDepartment$]).pipe(
      switchMap(([toUser, toDepartment]) => {
        const model: ProjectMilestoneModel = {
          id: dto.id ?? this.convertError('Convert error: id is not defined'),
          endEff: dto.end_eff ?? this.convertError('Convert error: end_eff is not defined'),
          indicator: this.convertIndicator(dto.indicator),
          name: dto.name ?? this.convertError('Convert error: name is not defined'),
          overdueDaysBase: dto.overdue_days_base ?? undefined,
          status: this.convertStatus(dto.status),
          toDepartment,
          toUser,
        };

        return this.createConvertObservable(model);
      })
    );
  }
  override reverse(model: ProjectMilestoneModel): ProjectMilestoneDTO {
    throw new Error('Method not implemented.');
  }

  private convertIndicator(indicator: ProjectMilestoneModel['indicator'] | undefined): ProjectMilestoneIndicatorEnum | undefined {
    switch (indicator) {
      case 'RED':
        return ProjectMilestoneIndicatorEnum.RED;
      case 'AMBER':
        return ProjectMilestoneIndicatorEnum.AMBER;
      case 'GREEN':
        return ProjectMilestoneIndicatorEnum.GREEN;
      default:
        return undefined;
    }
  }

  private convertStatus(status: ProjectMilestoneDTO['status']): ProjectMilestoneStatusEnum | undefined {
    switch (status) {
      case 'BL':
        return ProjectMilestoneStatusEnum.BL;
      case 'CR':
        return ProjectMilestoneStatusEnum.CR;
      case 'WP':
        return ProjectMilestoneStatusEnum.WP;
      case 'DN':
        return ProjectMilestoneStatusEnum.DN;
      case 'AC':
        return ProjectMilestoneStatusEnum.AC;
      case 'CN':
        return ProjectMilestoneStatusEnum.CN;
      case 'PS':
        return ProjectMilestoneStatusEnum.PS;
      default:
        return undefined;
    }
  }
}
