import { Injectable } from '@angular/core';
import { Observable, forkJoin, of, switchMap } from 'rxjs';

import { BaseAdapter } from 'utils/shared-services';

import { ProjectStageDTO } from '../models/project-stage.dto';
import { ProjectStageModel } from '../models/project-stage.model';
import { ProjectMilestoneAdapterService } from './project-milestone-adapter.service';
import { ProjectMilestoneModel } from '../models/project-milestone.model';
import { ProjectStageStateEnum } from '../models/project-stage-state.enum';

@Injectable({
  providedIn: 'root',
})
export class ProjectStagesAdapterService extends BaseAdapter<ProjectStageModel, ProjectStageDTO> {
  constructor(private readonly projectMilestoneAdapter: ProjectMilestoneAdapterService) {
    super();
  }
  override convert(dto: ProjectStageDTO): Observable<ProjectStageModel> {
    const endMilestone$: Observable<ProjectMilestoneModel | undefined> = dto.end_milestone
      ? this.projectMilestoneAdapter.convert(dto.end_milestone)
      : of(undefined);

    return forkJoin([endMilestone$]).pipe(
      switchMap(([endMilestone]) => {
        const model: ProjectStageModel = {
          name: dto.name ?? this.convertError('Convert error: name is not defined'),
          endMilestone,
          state: this.convertProjectStageState(dto.state) ?? this.convertError('Convert error: project state is not defined'),
        };

        return this.createConvertObservable(model);
      })
    );
  }
  override reverse(model: ProjectStageModel): ProjectStageDTO {
    throw new Error('Method not implemented.');
  }

  private convertProjectStageState(state: ProjectStageDTO['state']): ProjectStageStateEnum | undefined {
    switch (state) {
      case 'DONE':
        return ProjectStageStateEnum.DONE;
      case 'CURRENT':
        return ProjectStageStateEnum.CURRENT;
      case 'FUTURE':
        return ProjectStageStateEnum.FUTURE;
      default:
        return undefined;
    }
  }
}
