import {Injectable} from '@angular/core';
import {Observable, withLatestFrom} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectTypeModel} from '../models/project-type.model';
import {ProjectTypeDto} from '../models/project-type.dto';
import {ProjectTypeBaseAdapterService} from './project-type-base-adapter.service';
import {ProjectTypeStageAdapterService} from '../../stages/services/project-type-stage-adapter.service';
import {ProjectTypeStageModel} from '../../stages';

@Injectable({
  providedIn: 'root',
})
export class ProjectTypeAdapterService extends BaseAdapter<ProjectTypeModel, ProjectTypeDto> {
  constructor(
    private readonly projectTypeBaseAdapterService: ProjectTypeBaseAdapterService,
    private readonly projectTypeStageAdapterService: ProjectTypeStageAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectTypeDto): Observable<ProjectTypeModel> {
    const projectTypeStages$: Observable<ProjectTypeStageModel[]> = this.createModelArrObs(
      dto.rel_pt_stages,
      this.projectTypeStageAdapterService,
    );

    return this.projectTypeBaseAdapterService.convert(dto)
      .pipe(
        withLatestFrom(projectTypeStages$),
        switchMap(([baseModel, projectTypeStages]) => {
          const model: ProjectTypeModel = {
            ...baseModel,

            // relations ids
            categoryId: dto.category ?? undefined,
            projectsIds: dto.projects ?? [],
            projectTypeStagesIds: dto.pt_stages ?? [],

            // relation models
            projectTypeStages,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: ProjectTypeModel): ProjectTypeDto {
    throw new Error('Method not implemented.');
  }
}
