import { Injectable } from '@angular/core';
import { switchMap } from 'rxjs/operators';
import { Observable, forkJoin, of, throwError } from 'rxjs';

import { ProjectsService } from 'data-access/sdk/api2';

import { ProjectAdapterService } from './project-adapter';
import { ProjectStagesAdapterService } from './project-stages-adapter.service';
import { ProjectStageModel } from '../models/project-stage.model';

@Injectable({
  providedIn: 'root',
})
export class ProjectsApiService {
  constructor(
    private readonly projectsService: ProjectsService,
    private readonly projectAdapterService: ProjectAdapterService,
    private readonly projectStagesAdapterService: ProjectStagesAdapterService
  ) {}

  readProjects(params: ProjectsService.ProjectsListParams = {}) {
    return this.projectsService.projectsList({ ...params }).pipe(
      switchMap((projects) => {
        if (projects.length > 0) {
          return forkJoin(projects.map((project) => this.projectAdapterService.convert(project)));
        }

        return of([]);
      })
    );
  }

  readProjectTimeline(projectId: number): Observable<ProjectStageModel[]> {
    return this.projectsService.projectsTimeline(projectId).pipe(
      switchMap(({ stages }) => {
        if (stages && stages.length > 0) {
          return forkJoin(stages.map((stage) => this.projectStagesAdapterService.convert(stage)));
        }

        return throwError(() => new Error('No stages found in response'));
      })
    );
  }
}

