import {Injectable} from '@angular/core';
import {forkJoin, Observable} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {UserRoleModel} from '../models/user-role.model';
import {UserRoleDTO} from '../models/user-role.dto';
import {RoleBaseAdapterService} from './role-base-adapter.service';
import {RoleBaseModel} from '../models/role-base.model';
import {DepartmentBaseModel, UserBaseModel} from 'data-access/sdk/internal-api';
import {DepartmentBaseAdapterService} from '../../departments/services/department-base-adapter.service';
import {UserBaseAdapterService} from '../../users/services/user-base-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class UserRoleAdapterService extends BaseAdapter<UserRoleModel, UserRoleDTO> {
  constructor(
    private readonly roleBaseAdapterService: RoleBaseAdapterService,
    private readonly departmentBaseAdapterService: DepartmentBaseAdapterService,
    private readonly userBaseAdapterService: UserBaseAdapterService,
  ) {
    super();
  }

  convert(dto: UserRoleDTO): Observable<UserRoleModel> {
    const department$: Observable<DepartmentBaseModel | undefined> = this.createModelObs(
      dto.rel_department,
      this.departmentBaseAdapterService,
    );
    const role$: Observable<RoleBaseModel | undefined> = this.createModelObs(dto.rel_role, this.roleBaseAdapterService);
    const user$: Observable<UserBaseModel | undefined> = this.createModelObs(dto.rel_user, this.userBaseAdapterService);

    return forkJoin([
      department$,
      role$,
      user$,
    ])
      .pipe(
        switchMap(([department, role, user]) => {
          const model: UserRoleModel = {
            id: dto.id ?? this.convertError('Convert error: id is not defined'),
            createdAt: dto.created_at ?? undefined,
            customName: dto.custom_name ?? undefined,
            updatedAt: dto.updated_at ?? undefined,

            // ids
            companyId: dto.company ?? undefined,
            departmentId: dto.department ?? undefined,
            roleId: dto.role ?? undefined,
            userId: dto.user ?? undefined,
            projectId: dto.project ?? undefined,
            projectGroupId: dto.project_group ?? undefined,

            // models
            department,
            role,
            user,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: UserRoleModel): UserRoleDTO {
    throw new Error('Method not implemented.');
  }
}
