import {Injectable} from '@angular/core';
import {catchError, forkJoin, of} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {ProjectType, ProjectTypeStagesService} from 'data-access/sdk/api2';

import {ProjectTypeStageDTO, ProjectTypeStageModel} from '../models';
import {ProjectTypeStageAdapterService} from './project-type-stage-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class ProjectStagesApiService {
  constructor(
    private readonly projectTypeStagesService: ProjectTypeStagesService,
    private readonly projectTypeStageAdapterService: ProjectTypeStageAdapterService,
  ) {
  }

  readProjectStages(projectTypeId: ProjectType['id']) {
    return this.projectTypeStagesService
      .projectTypeStagesList({projectType: String(projectTypeId), include: 'stage'})
      .pipe(
        switchMap((projectStages) => forkJoin(
          projectStages.map((stage) => this.projectTypeStageAdapterService.convert(stage)),
        )),
        catchError(() => of([])),
      );
  }

  updateProjectStages(projectStages: ProjectTypeStageModel[]) {
    const projectStageDTOs = projectStages
      .map((stage) => this.projectTypeStageAdapterService.reverse(stage));

    return forkJoin(
      projectStageDTOs
        .map((stage) => this.projectTypeStagesService
          .projectTypeStagesPartialUpdateResponse({
            id: stage.id,
            data: stage,
          })),
    );
  }

  deleteProjectStage(id: number, replaceWithPtStage: number) {
    return this.projectTypeStagesService.projectTypeStagesDelete({id, replaceWithPtStage});
  }

  createProjectStage(projectTypeId: ProjectTypeStageDTO['project_type'], projectStageId: ProjectTypeStageDTO['stage']) {
    return this.projectTypeStagesService.projectTypeStagesCreate({project_type: projectTypeId, stage: projectStageId});
  }
}
