import {Injectable} from '@angular/core';
import {forkJoin, of} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {ProjectTypeStageDTO, ProjectTypeStageModel} from '../models';
import {StageAdapterService} from './stage-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class ProjectTypeStageAdapterService extends BaseAdapter<ProjectTypeStageModel, ProjectTypeStageDTO> {
  constructor(
    private readonly stageAdapterService: StageAdapterService,
  ) {
    super();
  }

  convert(dto: ProjectTypeStageDTO) {
    return forkJoin([
      (dto.rel_stage ? this.stageAdapterService.convert(dto.rel_stage) : of(undefined)),
    ])
      .pipe(
        switchMap(([stage]) => {
            const model: ProjectTypeStageModel = {
              id: dto.id ?? this.convertError('Convert error: id is not defined'),
              order: dto.order ?? this.convertError('Convert error: order is not defined'),

              // relations ids
              projectTypeId: dto.project_type ?? this.convertError('Convert error: project_type is not defined'),
              stageId: dto.stage ?? this.convertError('Convert error: stage is not defined'),

              // relation models
              stage,
            };

            return this.createConvertObservable(model);
          },
        ),
      );
  }

  reverse(model: ProjectTypeStageModel) {
    return {
      id: model.id,
      order: model.order,
      project_type: model.projectTypeId,
      stage: model.stageId,
    };
  }
}
