import {Injectable} from '@angular/core';
import {ProjectEnums} from '@core-project/enums/project';

import {ProjectTypeStageModel, StageModel} from '../models';

@Injectable({
  providedIn: 'root',
})
export class StageOrderService {
  // from US
  private statusOrder = [ProjectEnums.StatusEnum.FUT, ProjectEnums.StatusEnum.ACT, ProjectEnums.StatusEnum.CLS];

  reIndexOrderStages(stages: StageModel[]): StageModel[] {
    return stages.map((stage, index: number) => {
      stage.order = index + 1;
      return stage;
    });
  }

  sortStages(stages: StageModel[]) {
    return stages
      .sort((a, b) => a.order - b.order)
      .sort((a, b) => {
        const aOrder = this.statusOrder.indexOf(a.status);
        const bOrder = this.statusOrder.indexOf(b.status);
        return aOrder - bOrder;
      });
  }

  reIndexOrderProjectStages(projectStages: ProjectTypeStageModel[]) {
    return projectStages.map((projectStage, index: number) => {
      projectStage.order = index + 1;
      return projectStage;
    });
  }

  sortProjectStages(projectStages: ProjectTypeStageModel[]) {
    return projectStages
      .sort((a, b) => a.order - b.order)
      .sort((a, b) => {
        const aOrder = a.stage ? this.statusOrder.indexOf(a.stage.status) : 0;
        const bOrder = b.stage ? this.statusOrder.indexOf(b.stage.status) : 0;
        return aOrder - bOrder;
      });
  }
}
