import {Injectable} from '@angular/core';
import {map, switchMap} from 'rxjs/operators';

import {StagesService} from 'data-access/sdk/api2';

import {StageDTO, StageModel} from '../models';
import {StageAdapterService} from './stage-adapter.service';
import {forkJoin} from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class StagesApiService {
  constructor(
    private readonly stagesService: StagesService,
    private readonly adapterService: StageAdapterService,
  ) {
  }

  createStage(stage: Pick<StageDTO, 'name' | 'status'>) {
    return this.stagesService.stagesCreate(stage);
  }

  readStage(id: number, include?: string) {
    return this.stagesService.stagesRead({id, include})
      .pipe(switchMap((stage) => this.adapterService.convert(stage)));
  }

  readStages(include?: string) {
    return this.stagesService.stagesList(include)
      .pipe(
        switchMap((stages) => forkJoin(
          stages.map((stage) => this.adapterService.convert(stage)),
        )),
      );
  }

  deleteStage(id: number, replaceWithStage: number) {
    return this.stagesService.stagesDelete({id, replaceWithStage});
  }

  updateStage(stage: StageModel) {
    const stageDTO = this.adapterService.reverse(stage);
    return this.stagesService.stagesUpdate({id: stageDTO.id as number, data: stageDTO});
  }
}
