import {Injectable} from '@angular/core';
import {Observable, withLatestFrom} from 'rxjs';
import {switchMap} from 'rxjs/operators';

import {BaseAdapter} from 'utils/shared-services';

import {UserModel} from '../models/user.model';
import {UserDTO} from '../models/user.dto';
import {UserBaseAdapterService} from './user-base-adapter.service';
import {CompanyAdapterService} from '../../companies/services/company-adapter.service';
import {DepartmentBaseAdapterService} from '../../departments/services/department-base-adapter.service';
import {TelephoneAdapterService} from '../../telephones/services/telephone-adapter.service';
import {CompanyModel} from '../../companies';
import {DepartmentBaseModel} from '../../departments';
import {TelephoneModel} from '../../telephones';

@Injectable({
  providedIn: 'root',
})
export class UserAdapterService extends BaseAdapter<UserModel, UserDTO> {

  constructor(
    private readonly userBaseAdapterService: UserBaseAdapterService,
    private readonly companyAdapterService: CompanyAdapterService,
    private readonly departmentBaseAdapterService: DepartmentBaseAdapterService,
    private readonly telephoneAdapterService: TelephoneAdapterService,
  ) {
    super();
  }

  convert(dto: UserDTO): Observable<UserModel> {
    const company$: Observable<CompanyModel | undefined> = this.createModelObs(dto.rel_company, this.companyAdapterService);
    const department$: Observable<DepartmentBaseModel | undefined> = this.createModelObs(
      dto.rel_department,
      this.departmentBaseAdapterService,
    );
    const telephones$: Observable<TelephoneModel[]> = this.createModelArrObs(dto.rel_telephones, this.telephoneAdapterService);

    return this.userBaseAdapterService.convert(dto)
      .pipe(
        withLatestFrom(
          company$,
          department$,
          telephones$,
        ),
        switchMap(([userBase, company, department, telephones]) => {
          const model: UserModel = {
            ...userBase,

            // relation ids
            companyId: dto.company ?? undefined,
            departmentId: dto.department ?? undefined,
            telephoneIds: dto.telephones ?? [],

            // relation models
            company,
            department,
            telephones,
          };

          return this.createConvertObservable(model);
        }),
      );
  }

  reverse(model: UserModel): UserDTO {
    throw new Error('Method not implemented.');
  }

}
