import {Injectable} from '@angular/core';
import {switchMap} from 'rxjs/operators';
import {forkJoin, of} from 'rxjs';

import {UsersService} from 'data-access/sdk/api2';

import {UserAdapterService} from './user-adapter.service';

@Injectable({
  providedIn: 'root',
})
export class UserApiService {
  constructor(
    private readonly usersService: UsersService,
    private readonly userAdapterService: UserAdapterService,
  ) {
  }

  readUsers(params: UsersService.UsersListParams = {}) {
    return this.usersService.usersList(params)
      .pipe(
        switchMap((users) => {
          if (users.length > 0) {
            return forkJoin(users.map((user) => this.userAdapterService.convert(user)));
          }

          return of([]);
        }),
      );
  }
}
