import * as moment from 'moment';

import { WorkStatusEnum } from '../models/work-status.enum';
import { WorkTypeEnum } from '../models/work-type.enum';

export function startBaseHint(start?: string, status?: WorkStatusEnum) {
  const now = moment().startOf('day');

  if (start && status && [WorkStatusEnum.BL, WorkStatusEnum.CR].includes(status) && now.isAfter(start)) {
    return {
      text: 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.START_BASE_WARNING',
      color: 'orange',
    };
  }

  return { text: undefined, color: undefined };
}

export function getEndEffHint(end?: string, status?: WorkStatusEnum, type?: WorkTypeEnum) {
  const now = moment().startOf('day');

  if (!type) {
    console.error('Work type is not defined');
    
  }

  if (end && status && [WorkStatusEnum.BL, WorkStatusEnum.CR, WorkStatusEnum.WP].includes(status) && now.isAfter(end)) {
    const text =
      type === WorkTypeEnum.MS
        ? 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.END_BASE_MS_WARNING'
        : 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.END_BASE_TASK_WARNING';

    return {
      text,
      color: 'orange',
    };
  }

  return { text: undefined, color: undefined };
}

export function getDeadlineHint(end?: string, deadline?: string, status?: WorkStatusEnum) {
  const now = moment().startOf('day');

  if (status && [WorkStatusEnum.BL, WorkStatusEnum.CR, WorkStatusEnum.WP].includes(status)) {
    if (deadline && now.isAfter(deadline)) {
      return {
        text: 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.DEADLINE_ERROR',
        color: 'red',
      };
    } else if (end && deadline && moment(end).isAfter(deadline)) {
      return {
        text: 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.DEADLINE_WARNING',
        color: 'orange',
      };
    }
  }

  return { text: undefined, color: undefined };
}

export function getDurationHint(end?: string, start?: string) {
  if (start && end && moment(start).isAfter(end)) {
    return {
      text: 'LIBS.FEATURES.WORKS.WORK_CARD_FORM.FORM_FIELDS.DURATION_ERROR',
      color: 'red',
    };
  }

  return { text: undefined, color: undefined };
}
