import { createSelector, Selector } from '@ngxs/store';

import { GLOBAL_RULES_STATE_TOKEN } from './global-rules.state';
import { PermissionModelsNames } from './models/permission.enum';
import { GlobalRulesStateModel } from './global-rules-state.model';

export class GlobalRulesSelectors {
  static canRead(modelName: PermissionModelsNames) {
    return createSelector(
      [GLOBAL_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canRead || false
    );
  }

  static canCreate(modelName: PermissionModelsNames) {
    return createSelector(
      [GLOBAL_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canCreate || false
    );
  }

  static canUpdate(modelName: PermissionModelsNames) {
    return createSelector(
      [GLOBAL_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canUpdate || false
    );
  }

  static canNotUpdate(modelName: PermissionModelsNames) {
    return createSelector(
      [GLOBAL_RULES_STATE_TOKEN],
      (state) => !state.permissions.find((p) => p.modelName === modelName)?.canUpdate || false
    );
  }

  static canDelete(modelName: PermissionModelsNames) {
    return createSelector(
      [GLOBAL_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canDelete || false
    );
  }

  static customRule(customPermission: string, modelName: PermissionModelsNames) {
    return createSelector([GLOBAL_RULES_STATE_TOKEN], (state) =>
      (state.permissions.find((p) => p.modelName === modelName) || undefined)?.getCustomPermission(customPermission)
    );
  }

  @Selector([GLOBAL_RULES_STATE_TOKEN])
  static permissionsInited(state: GlobalRulesStateModel) {
    return state.inited;
  }
}
