import {Injectable} from '@angular/core';
import {Action, State, StateContext, StateToken} from '@ngxs/store';
import {of, tap} from 'rxjs';

import {AuthService} from 'data-access/sdk/api2';

import {GlobalRulesStateModel} from './global-rules-state.model';
import {GlobalRulesActions} from './global-rules.actions';
import {Permission} from './models/permission.model';

export const GLOBAL_RULES_STATE_TOKEN = new StateToken<GlobalRulesStateModel>('globalRules');

@State({
  name: GLOBAL_RULES_STATE_TOKEN,
  defaults: {
    permissions: [],
    inited: false,
  },
})
@Injectable()
export class GlobalRulesState {
  constructor(
    private readonly authService: AuthService,
  ) {
  }

  @Action(GlobalRulesActions.UpdatePermissions)
  updatePermissions(ctx: StateContext<GlobalRulesStateModel>, {}: GlobalRulesActions.UpdatePermissions) {
    return this.authService.authUsersPermissions({})
      .pipe(
        tap(({permissions}) => {
          const permissionsForCreate: string[] = permissions
              ?.filter((permission) => typeof permission === 'string') as string[]
            || [];

          if (!permissionsForCreate || permissionsForCreate.length === 0) {
            console.error('No global permissions for update found!');
            return of(undefined);
          }

          const permissionModels = Permission.createPermissions(permissionsForCreate);
          ctx.patchState({permissions: permissionModels});

          if (!ctx.getState().inited) {
            ctx.patchState({inited: true});
          }

          return of(undefined);
        }),
      );
  }

  @Action(GlobalRulesActions.ResetPermissions)
  resetPermissions(ctx: StateContext<GlobalRulesStateModel>) {
    ctx.patchState({permissions: []});

    if (!ctx.getState().inited) {
      ctx.patchState({inited: true});
    }
  }
}
