import { PermissionModelsNames, PermissionTypes } from './permission.enum';
import {IPermission} from './permission.interface';

export class Permission implements IPermission {
  readonly modelName: PermissionModelsNames;
  private _canRead = false;
  private _canCreate = false;
  private _canUpdate = false;
  private _canDelete = false;

  private permissions: { [key: string]: boolean | undefined } = {} as any;

  private readonly devMode = false;
  private readonly devValue = false;

  constructor(rawName: string) {
    this.modelName = this.getModelName(rawName);
  }

  get canRead() {
    if (this.devMode) {
      return this.devValue;
    }
    return this._canRead;
  }

  get canCreate() {
    if (this.devMode) {
      return this.devValue;
    }
    return this._canCreate;
  }

  get canUpdate() {
    if (this.devMode) {
      return this.devValue;
    }
    return this._canUpdate;
  }

  get canDelete() {
    if (this.devMode) {
      return this.devValue;
    }
    return this._canDelete;
  }

  static repairRuleName(permission: PermissionTypes, model: PermissionModelsNames) {
    return `${permission}_${model}`;
  }

  static createPermissions(permissions: string[]): Permission[] {
    return permissions.reduce((acc, permissionRawName) => {
      const permissionType: PermissionTypes = permissionRawName.split('_')[0] as PermissionTypes;
      const modelName: PermissionModelsNames = permissionRawName.split('_')[1] as unknown as PermissionModelsNames;
      const permissionModel: Permission | undefined = acc.find((p) => p.modelName === modelName);
      if (permissionModel) {
        permissionModel.setPermissions({ [permissionType]: true });
      } else {
        const newPermission = new Permission(permissionRawName);
        newPermission.setPermissions({ [permissionType]: true });
        acc.push(newPermission);
      }

      return acc;
    }, [] as Permission[]);
  }

  getCustomPermission(customPermissionKey: string): boolean {
    return this.permissions[customPermissionKey] || false;
  }

  setPermissions(permissions: {
    [key in PermissionTypes | string]?: boolean;
  }) {
    if (typeof permissions['view'] !== 'undefined') {
      this._canRead = permissions['view'];
      delete permissions['view'];
    }
    if (typeof permissions['add'] !== 'undefined') {
      this._canCreate = permissions['add'];
      delete permissions['add'];
    }
    if (typeof permissions['change'] !== 'undefined') {
      this._canUpdate = permissions['change'];
      delete permissions['change'];
    }
    if (typeof permissions['delete'] !== 'undefined') {
      this._canDelete = permissions['delete'];
      delete permissions['delete'];
    }

    // custom permissions
    if (Object.keys(permissions).length > 0) {
      Object.keys(permissions).forEach((permissionKey) => {
        this.permissions[permissionKey] = permissions[permissionKey];
      });
    }
  }

  private getModelName(rawName: string) {
    return rawName.split('_')[1] as unknown as PermissionModelsNames;
  }
}
