import {Injectable} from '@angular/core';
import {ActivatedRouteSnapshot, RouterStateSnapshot} from '@angular/router';
import {Action, Actions, ofActionSuccessful, State, StateContext, StateToken, Store} from '@ngxs/store';
import {RouterNavigation} from '@ngxs/router-plugin';
import {switchMap} from 'rxjs/operators';
import {of} from 'rxjs';


import {GlobalStateModel} from './global-state.model';
import {GlobalActions} from './global.actions';
import {GlobalRulesActions} from '../app-global-rules';
import {DATA_APP_TITLE_KEY, DATA_APP_TITLE_VALUES, IAppTitle} from './index';

export const GLOBAL_STATE_TOKEN = new StateToken<GlobalStateModel>('global');

@State({
  name: GLOBAL_STATE_TOKEN,
  defaults: {
    user: undefined,
    appTitle: undefined,
    topMenuItems: [],
  },
})
@Injectable()
export class GlobalState {
  title$ = this.actions$.pipe(
    ofActionSuccessful(RouterNavigation<RouterStateSnapshot>),
    switchMap((action: RouterNavigation) => {
      const data = this.getTitleRecurse(action.routerState);

      if (data) {
        return this.store.dispatch(new GlobalActions.SetAppTitle(data));
      }

      return of(action);
      // данная логика была убрана в пользу старой реализации
      // return this.store.dispatch(new GlobalActions.ResetAppTitle());
    }),
  ).subscribe();

  constructor(
    private readonly actions$: Actions,
    private readonly store: Store,
  ) {
  }

  @Action(GlobalActions.SetTopMenuItems)
  setTopMenuItems(ctx: StateContext<GlobalStateModel>, {items}: GlobalActions.SetTopMenuItems) {
    ctx.patchState({topMenuItems: items});
  }

  @Action(GlobalActions.SetUser)
  setUser(ctx: StateContext<GlobalStateModel>, {user}: GlobalActions.SetUser) {
    ctx.patchState({user});

    return ctx.dispatch(new GlobalRulesActions.UpdatePermissions());
  }

  @Action(GlobalActions.ResetUser)
  resetUser(ctx: StateContext<GlobalStateModel>) {
    ctx.patchState({user: undefined});

    return ctx.dispatch(new GlobalRulesActions.ResetPermissions());
  }

  @Action(GlobalActions.SetAppTitle)
  setAppTitle(ctx: StateContext<GlobalStateModel>, {appTitle}: GlobalActions.SetAppTitle) {
    ctx.patchState({appTitle});
  }

  @Action(GlobalActions.ResetAppTitle)
  resetAppTitle(ctx: StateContext<GlobalStateModel>) {
    ctx.patchState({appTitle: undefined});
  }

  private getTitleRecurse(snapshot: RouterStateSnapshot | ActivatedRouteSnapshot): IAppTitle | undefined {
    const activatedRoute = snapshot.root
      ? (snapshot as RouterStateSnapshot).root.firstChild
      : (snapshot as ActivatedRouteSnapshot).firstChild;

    if (activatedRoute?.data[DATA_APP_TITLE_KEY]) {
      return this.getTitleRecurse(activatedRoute)
        ?? {key: activatedRoute.data[DATA_APP_TITLE_KEY], values: activatedRoute.data[DATA_APP_TITLE_VALUES]};
    } else {
      return undefined;
    }
  }
}
