import {Action, State, StateContext} from '@ngxs/store';
import {Injectable} from '@angular/core';
import {finalize, tap} from 'rxjs';
import {map} from 'rxjs/operators';
import {CustomDashboardsActions} from './custom-dashboards.actions';
import {DashboardPluginsApiService, DashboardPluginModel, PluginsOrderService} from 'data-access/sdk/internal-api';

export interface CustomDashboardsStateModel {
  plugins: DashboardPluginModel[];
  isListLoading: boolean;
}

@State<CustomDashboardsStateModel>({
  name: 'customDashboards',
  defaults: {
    plugins: [],
    isListLoading: false,
  },
})
@Injectable()
export class CustomDashboardsState {

  constructor(
    private readonly apiService: DashboardPluginsApiService,
    private readonly orderService: PluginsOrderService,
  ) {
  }

  @Action(CustomDashboardsActions.ReadList)
  readList(ctx: StateContext<CustomDashboardsStateModel>) {
    this.setListLoading(true, ctx);

    return this.apiService.readList()
      .pipe(
        finalize(() => this.setListLoading(false, ctx)),
        map((plugins) => this.orderService.reIndexOrder(this.orderService.sort(plugins))),
        tap((plugins) => {
          ctx.patchState({plugins});
        }),
      );
  }

  private setListLoading(loading: boolean, ctx: StateContext<CustomDashboardsStateModel>) {
    ctx.patchState({isListLoading: loading});
  }
}
