import {Injectable} from '@angular/core';
import {Action, State, StateContext} from '@ngxs/store';
import {finalize, of, tap} from 'rxjs';
import {DashboardPluginsApiService} from 'data-access/sdk/internal-api';
import {DashboardPluginActions} from './dashboard-plugin.actions';
import {DashboardPluginStateModel} from './dashboard-plugin-state.model';
import {GlobalActions} from '../app-global';

@State<DashboardPluginStateModel>({
  name: 'plugin',
  defaults: {
    plugin: undefined,
    isLoading: false,
  },
})
@Injectable()
export class DashboardPluginState {
  constructor(
    private readonly apiService: DashboardPluginsApiService,
  ) {
  }

  @Action(DashboardPluginActions.Read)
  read(ctx: StateContext<DashboardPluginStateModel>, {id}: DashboardPluginActions.Read) {
    this.setLoading(true, ctx);

    return this.apiService.read(id)
      .pipe(
        finalize(() => this.setLoading(false, ctx)),
        tap((plugin) => {
          ctx.patchState({plugin});
        }),
        tap((plugin) => {
          ctx.dispatch(new GlobalActions.SetAppTitle({key: plugin.name}));
        })
      );
  }

  @Action(DashboardPluginActions.Reset)
  reset(ctx: StateContext<DashboardPluginStateModel>) {
    ctx.patchState({plugin: undefined});
    return of(undefined);
  }

  private setLoading(loading: boolean, ctx: StateContext<DashboardPluginStateModel>) {
    ctx.patchState({isLoading: loading});
  }
}
