import { CanActivateFn, CanDeactivateFn } from '@angular/router';
import { inject } from '@angular/core';
import { Store } from '@ngxs/store';
import { map } from 'rxjs/operators';

import {ProjectRulesActions} from './project-rules.actions';

export const getProjectPermissionsGuard: CanActivateFn = (route, state) => {
  const projectId: number | undefined = route.params['projectId'] ? Number(route.params['projectId']) : undefined;

  if (projectId) {
    const store: Store = inject(Store);

    return store.dispatch(new ProjectRulesActions.UpdatePermissions(projectId)).pipe(map(() => true));
  }
  return true;
};

export const resetProjectPermissionsGuard: CanDeactivateFn<unknown> = (component, currentRoute, currentState, nextState) => {
  const projectId: number | undefined = currentRoute.params['projectId'] ? Number(currentRoute.params['projectId']) : undefined;

  if (projectId) {
    const store: Store = inject(Store);

    return store.dispatch(new ProjectRulesActions.ResetPermissions());
  }
  return true;
};
