import {createSelector, Selector} from '@ngxs/store';

import {PROJECT_RULES_STATE_TOKEN} from './project-rules.state';
import {ProjectRulesStateModel} from './project-rules-state.model';
import {PermissionModelsNames} from '../app-global-rules/models/permission.enum';
import {GLOBAL_RULES_STATE_TOKEN} from '../app-global-rules';

export class ProjectRulesSelectors {
  static canRead(modelName: PermissionModelsNames) {
    return createSelector(
      [PROJECT_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canRead || false,
    );
  }

  static canCreate(modelName: PermissionModelsNames) {
    return createSelector(
      [PROJECT_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canCreate || false,
    );
  }

  static canUpdate(modelName: PermissionModelsNames) {
    return createSelector(
      [PROJECT_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canUpdate || false,
    );
  }

  static canNotUpdate(modelName: PermissionModelsNames) {
    return createSelector(
      [PROJECT_RULES_STATE_TOKEN],
      (state) => !state.permissions.find((p) => p.modelName === modelName)?.canUpdate || false,
    );
  }

  static canDelete(modelName: PermissionModelsNames) {
    return createSelector(
      [PROJECT_RULES_STATE_TOKEN],
      (state) => state.permissions.find((p) => p.modelName === modelName)?.canDelete || false,
    );
  }

  static customRule(customPermission: string, modelName: PermissionModelsNames) {
    return createSelector([GLOBAL_RULES_STATE_TOKEN], (state) =>
      (state.permissions.find((p) => p.modelName === modelName) || undefined)?.getCustomPermission(customPermission),
    );
  }

  @Selector([PROJECT_RULES_STATE_TOKEN])
  static projectId(state: ProjectRulesStateModel) {
    return state.projectId;
  }
}
