import {Injectable} from '@angular/core';
import {Action, State, StateContext, StateToken} from '@ngxs/store';
import {of, tap} from 'rxjs';

import {AuthService} from 'data-access/sdk/api2';

import {ProjectRulesStateModel} from './project-rules-state.model';
import {ProjectRulesActions} from './project-rules.actions';
import {Permission} from '../app-global-rules/models/permission.model';

export const PROJECT_RULES_STATE_TOKEN = new StateToken<ProjectRulesStateModel>('projectRules');

@State({
  name: PROJECT_RULES_STATE_TOKEN,
  defaults: {
    projectId: undefined,
    permissions: [],
  },
})
@Injectable()
export class ProjectRulesState {
  constructor(private readonly authService: AuthService) {
  }

  @Action(ProjectRulesActions.UpdatePermissions)
  updatePermissions(ctx: StateContext<ProjectRulesStateModel>, {projectId}: ProjectRulesActions.UpdatePermissions) {
    return this.authService.authUsersPermissions({project: projectId})
      .pipe(
        tap(({permissions}) => {
          const permissionsForCreate: string[] | undefined = permissions
            ?.filter((permission) => typeof permission === 'string') as string[] | undefined;

          if (!permissionsForCreate || permissionsForCreate.length === 0) {
            console.error('No project permissions for update found!');
            return of(undefined);
          }

          const permissionModels = Permission.createPermissions(permissionsForCreate);
          ctx.patchState({
            projectId,
            permissions: permissionModels,
          });

          return of(undefined);
        }),
      );
  }

  @Action(ProjectRulesActions.ResetPermissions)
  resetPermissions(ctx: StateContext<ProjectRulesStateModel>) {
    ctx.setState({
      projectId: undefined,
      permissions: [],
    });
  }
}
