import {of} from 'rxjs';
import {Action, State, StateContext} from '@ngxs/store';
import {Injectable} from '@angular/core';
import {catchError, switchMap} from 'rxjs/operators';

import {WebsocketsApiService, WebsocketsResponseType, WebsocketsTypesEnum} from 'data-access/sdk/internal-api';

import {WebsocketsActions} from './websockets.actions';
import {token, WebsocketsModel} from './websockets.model';
import {NotificationsActions} from '../notifications';

@State<WebsocketsModel>({
  name: token,
  defaults: {},
})
@Injectable()
export class WebsocketsState {
  constructor(
    private readonly apiService: WebsocketsApiService,
  ) {
  }

  @Action(WebsocketsActions.Connect)
  activateConnection(ctx: StateContext<WebsocketsModel>, {url}: WebsocketsActions.Connect) {
    const connect$ = this.apiService.connect(url, 'Token ' + localStorage.getItem('token'));
    return connect$.pipe(
      catchError((err) => {
        console.error(err);
        // TODO message
        return ctx.dispatch(new WebsocketsActions.Disconnect()).pipe(switchMap(() => connect$));
      }),
      switchMap((response: WebsocketsResponseType) => {
        if (response.type === WebsocketsTypesEnum.NOTIFICATION) {
          return ctx.dispatch(new NotificationsActions.AddToQueue(response.data));
        }

        return of(undefined);
      }),
    );
  }

  @Action(WebsocketsActions.Disconnect)
  deactivateConnection(ctx: StateContext<WebsocketsModel>) {
    this.apiService.disconnect();

    return of(undefined);
  }
}
