import { AfterViewInit, Component, ViewChild, ChangeDetectorRef } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { forkJoin } from 'rxjs';
import { InputTextareaModule } from 'primeng/inputtextarea';
import { ButtonModule } from 'primeng/button';
import { CalendarModule } from 'primeng/calendar';

import { ProjectStageModel, ProjectStagesAdapterService } from 'data-access/sdk/internal-api';
import { UiMilestoneTimelineContainerComponent, UiMilestonesTimelineComponent } from 'ui/milestones-timeline';
import { ProjectStage } from 'data-access/sdk/api2';

@Component({
  selector: 'lib-debug-milestone-timeline',
  standalone: true,
  templateUrl: './debug-milestone-timeline.component.html',
  styleUrls: ['./debug-milestone-timeline.component.scss'],
  imports: [
    CommonModule,
    FormsModule,
    ButtonModule,
    CalendarModule,
    UiMilestonesTimelineComponent,
    InputTextareaModule,
    UiMilestoneTimelineContainerComponent,
  ],
})
export class DebugMilestoneTimelineComponent implements AfterViewInit {
  @ViewChild('timeline') timeline!: UiMilestonesTimelineComponent;
  @ViewChild('timelineContainer') timelineContainer!: UiMilestoneTimelineContainerComponent;

  stagesMock = `[
      {
        "name": "Идея",
        "state": "DONE",
        "end_milestone": {
          "id": 568,
          "name": "идея-poc",
          "end_eff": "2024-01-18",
          "deadline": null,
          "overdue_days_base": 0,
          "indicator": "GREEN",
          "status": "AC",
          "to_user": null,
          "to_department": null,
          "rel_to_user": null,
          "rel_to_department": null
        }
      },
      {
        "name": "PoC",
        "state": "CURRENT",
        "end_milestone": {
          "id": 569,
          "name": "poc-mvp",
          "end_eff": "2024-02-02",
          "deadline": null,
          "overdue_days_base": 0,
          "indicator": "GREEN",
          "status": "BL",
          "to_user": null,
          "to_department": null,
          "rel_to_user": null,
          "rel_to_department": null
        }
      },
      {
        "name": "MVP",
        "state": "FUTURE",
        "end_milestone": {
          "id": 570,
          "name": "mvp-развитие",
          "end_eff": "2024-02-14",
          "deadline": null,
          "overdue_days_base": 0,
          "indicator": "GREEN",
          "status": "BL",
          "to_user": null,
          "to_department": null,
          "rel_to_user": null,
          "rel_to_department": null
        }
      },
      {
        "name": "Развитие",
        "state": "FUTURE",
        "end_milestone": {
          "id": 571,
          "name": "развитие-тирах",
          "end_eff": "2024-03-11",
          "deadline": "2024-03-01",
          "overdue_days_base": 10,
          "indicator": "AMBER",
          "status": "BL",
          "to_user": null,
          "to_department": null,
          "rel_to_user": null,
          "rel_to_department": null
        }
      },
      {
        "name": "Тираж",
        "state": "FUTURE",
        "end_milestone": {
          "id": 572,
          "name": "тираж-архив",
          "end_eff": "2024-05-23",
          "deadline": "2024-04-16",
          "overdue_days_base": 37,
          "indicator": "RED",
          "status": "BL",
          "to_user": null,
          "to_department": null,
          "rel_to_user": null,
          "rel_to_department": null
        }
      },
      {
        "name": "Архив",
        "state": "FUTURE",
        "end_milestone": null
      }
    ]`;

  stagesControl: string = this.stagesMock;
  dateControl = new Date('2024-05-01');
  currentMarkOffsetControl = 0;

  handledStages: ProjectStageModel[] = [];

  constructor(private readonly projectStagesAdapter: ProjectStagesAdapterService, private readonly changeDetectorRef: ChangeDetectorRef) {}

  ngAfterViewInit(): void {
    this.setTimeline();
  }

  setTimeline() {
    try {
      const stages: ProjectStage[] = JSON.parse(this.stagesControl);

      forkJoin(stages.map((projectStage) => this.projectStagesAdapter.convert(projectStage))).subscribe(
        (projectStages: ProjectStageModel[]) => {
          this.handledStages = projectStages;
          this.timeline.render();
          this.timelineContainer.timeline.render();

          this.changeDetectorRef.detectChanges();
        }
      );
    } catch (error) {
      console.error(error);
    }
  }
}
