import {ChangeDetectionStrategy, Component} from '@angular/core';
import {Select, Store} from '@ngxs/store';

import {GlobalSelectors, NotificationsActions} from 'data-access/store';
import {MenuItem} from 'primeng/api';
import {AbstractControl, FormControl, FormGroup} from '@angular/forms';
import {NewNotificationModel, NotificationModel, NotificationSeverityEnum, NotificationTypeEnum} from 'data-access/sdk/internal-api';
import {Observable} from 'rxjs';

interface NotificationFormGroup {
  recipientId: AbstractControl<NotificationModel['recipientId']>;
  severity: AbstractControl<NotificationModel['severity']>;
  nameTemplate: AbstractControl<string>;
  nameParams: AbstractControl<string>;
  descriptionTemplate: AbstractControl<string>;
  descriptionParams: AbstractControl<string>;
  urlType: AbstractControl<NotificationModel['urlType']> | AbstractControl<undefined>;
  urlParams: AbstractControl<string> | AbstractControl<undefined>;
}

@Component({
  selector: 'lib-debug-notifications',
  templateUrl: './debug-notifications.component.html',
  styleUrls: ['./debug-notifications.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DebugNotificationsComponent {
  @Select(GlobalSelectors.userId)
  userId$!: Observable<number | undefined>;

  items: MenuItem[] = [
    {
      label: 'Notifications',
      routerLink: ['/', 'devtools', 'notifications'],
    },
  ];

  home: MenuItem = {
    label: 'Devtools',
    routerLink: ['/', 'devtools'],
  };

  severity: { name: string; value: NotificationSeverityEnum }[] = [
    {name: 'Info', value: NotificationSeverityEnum.Info},
    {name: 'Success', value: NotificationSeverityEnum.Success},
    {name: 'Warn', value: NotificationSeverityEnum.Warn},
    {name: 'Error', value: NotificationSeverityEnum.Error},
  ];

  urlTypes: { name: string; value: NotificationModel['urlType'] }[] = [
    {name: 'Work', value: NotificationTypeEnum.Work},
    {name: 'Issue', value: NotificationTypeEnum.Issue},
    {name: 'ProjectInfo', value: NotificationTypeEnum.ProjectInfo},
    {name: 'ProjectPlan', value: NotificationTypeEnum.ProjectPlan},
    {name: 'None', value: undefined},
  ];

  notificationForm = new FormGroup<NotificationFormGroup>({
    recipientId: new FormControl(this.store.selectSnapshot(GlobalSelectors.userId) ?? 0, {nonNullable: true}),
    severity: new FormControl(NotificationSeverityEnum.Info, {nonNullable: true}),
    nameTemplate: new FormControl('Name template', {nonNullable: true}),
    nameParams: new FormControl('{}', {nonNullable: true}),
    descriptionTemplate: new FormControl('Description template', {nonNullable: true}),
    descriptionParams: new FormControl('{}', {nonNullable: true}),
    urlType: new FormControl(undefined, {nonNullable: true}),
    urlParams: new FormControl('{}', {nonNullable: true}),
  });

  constructor(
    private readonly store: Store,
  ) {
  }

  sendNotification() {
    const notification: NewNotificationModel = {
      recipientId: this.notificationForm.controls.recipientId.value,
      severity: this.notificationForm.controls.severity.value,
      nameTemplate: this.notificationForm.controls.nameTemplate.value,
      nameParams: JSON.parse(this.notificationForm.controls.nameParams.value || '{}'),
      descriptionTemplate: this.notificationForm.controls.descriptionTemplate.value,
      descriptionParams: JSON.parse(this.notificationForm.controls.descriptionParams.value || '{}'),
      urlType: this.notificationForm.controls.urlType.value,
      urlParams: JSON.parse(this.notificationForm.controls.urlParams.value || '{}'),
    };

    this.store.dispatch(new NotificationsActions.SendDebugNotification(notification));
  }
}
