import {ChangeDetectionStrategy, Component} from '@angular/core';
import {Store} from '@ngxs/store';
import {RightSidebarActions} from 'data-access/store';
import {MenuItem} from 'primeng/api';
import {
  getIssueCreateRouterLink,
  getIssueRouterLink,
  getWorkCreateRouterLink,
  getWorkRouterLink,
  OutletsEnum,
} from 'utils/core-project-routing-utils';
import {Router} from '@angular/router';

enum RightSidebarsRoutesKeysEnum {
  CreateIssue = 'CreateIssue',
  Issue = 'Issue',
  CreateWork = 'CreateWork',
  Work = 'Work',
}

interface RightSidebarsRoutesInterface {
  key: RightSidebarsRoutesKeysEnum;
  name: string;
  fn: (...args: any[]) => void;
}

@Component({
  selector: 'lib-debug-sidebars',
  templateUrl: './debug-sidebars.component.html',
  styleUrls: ['./debug-sidebars.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DebugSidebarsComponent {
  items: MenuItem[] = [
    {
      label: 'Sidebars',
      routerLink: ['/', 'devtools', 'sidebars'],
    },
  ];

  home: MenuItem = {
    label: 'Devtools',
    routerLink: ['/', 'devtools'],
  };

  rightSidebarsRoutes: RightSidebarsRoutesInterface[] = [
    {key: RightSidebarsRoutesKeysEnum.CreateIssue, name: 'Create issue', fn: () => this.router.navigate(getIssueCreateRouterLink())},
    {key: RightSidebarsRoutesKeysEnum.Issue, name: 'Go to issue by id', fn: (id: string) => this.router.navigate(getIssueRouterLink(id))},
    {key: RightSidebarsRoutesKeysEnum.CreateWork, name: 'Create work', fn: () => this.router.navigate(getWorkCreateRouterLink())},
    {key: RightSidebarsRoutesKeysEnum.Work, name: 'Go to work by id', fn: (id: string) => this.router.navigate(getWorkRouterLink(id))},
  ];
  rightSidebarEntityId?: string;
  rightSidebarEntityType?: RightSidebarsRoutesInterface;

  constructor(
    private readonly store: Store,
    private readonly router: Router,
  ) {
  }

  openRightSidebar() {
    this.store.dispatch(new RightSidebarActions.Open());
  }

  closeRightSidebar() {
    this.store.dispatch(new RightSidebarActions.Close());
  }

  navigateRightSidebarOutlet() {
    if (this.rightSidebarEntityType?.key
      && [RightSidebarsRoutesKeysEnum.Issue, RightSidebarsRoutesKeysEnum.Work]
        .some((v) => this.rightSidebarEntityType?.key === v)
    ) {
      if (this.rightSidebarEntityId) {
        this.rightSidebarEntityType.fn(this.rightSidebarEntityId);
      } else {
        console.error('Issue id is required');
      }
      return;
    }

    if (this.rightSidebarEntityType?.key
      && [RightSidebarsRoutesKeysEnum.CreateIssue, RightSidebarsRoutesKeysEnum.CreateWork]
        .some((v) => this.rightSidebarEntityType?.key === v)
    ) {
      this.rightSidebarEntityType.fn();
      return;
    }

    console.error('Route not found. Check code.');
  }

  clearRightSidebarRoute() {
    this.router.navigate([{outlets: {[OutletsEnum.RIGHT_SIDEBAR]: null}}], {replaceUrl: true});
  }
}
