import { Injectable, Signal } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { SelectItem, SelectItemGroup } from 'primeng/api';

import { UserItemsClass, ProjectItemsClass, IssueDomainItemsClass, EntityModel } from 'ui/forms-ui';

import { CreateIssueBaseRelationsFormModel } from '../interfaces/create-issue-base-relations-form.model';
import { IssueFormModel } from '../interfaces/issue-form.model';
import { IssueFormGroupInterface } from '../interfaces/issue-form-group.interface';

@Injectable({
  providedIn: 'root',
})
export class IssueCardFormService {
  constructor() {}

  createIssueFormRelationsHandler(
    formGroup: FormGroup<IssueFormGroupInterface>,
    responsibleOptions: Signal<SelectItemGroup[]>,
    workGroupOptions: Signal<SelectItemGroup[]>,
    projectOptions: Signal<SelectItemGroup[]>,
    domainsOptions: Signal<SelectItemGroup[]>,
    model?: IssueFormModel,
    createRelations?: CreateIssueBaseRelationsFormModel,
  ): Function {
    return () => {
      const isCreate = !formGroup.controls.id.value;

      const responsibleIG = responsibleOptions().find((group) => group.label === UserItemsClass.labelKey);
      const workGroupIG = workGroupOptions().find((group) => group.label === UserItemsClass.labelKey);
      const projectsIG = projectOptions().find((group) => group.label === ProjectItemsClass.labelKey);
      const issueDomainsIG = domainsOptions().find((group) => group.label === IssueDomainItemsClass.labelKey);

      // create relations

      if (isCreate && createRelations?.projectId && projectsIG?.items && projectsIG.items.length > 0) {
        const project: EntityModel | undefined = projectsIG?.items
          .find((item) => item.value.id === createRelations.projectId)?.value;
        formGroup.controls.project.setValue(project ?? null);
      }

      if (isCreate && createRelations?.authorId && workGroupIG?.items && workGroupIG.items.length > 0) {
        const user: EntityModel | undefined = workGroupIG.items
          .find((item) => item.value.id === createRelations.authorId)?.value;
        formGroup.controls.users.setValue(user ? [user] : []);
      }

      if (isCreate && createRelations?.responsibleId && responsibleIG?.items && responsibleIG.items.length > 0) {
        const responsible: EntityModel | undefined = responsibleIG.items
          .find((item) => item.value.id === createRelations.responsibleId)?.value;
        formGroup.controls.responsible.setValue(responsible ?? null);
      }

      if (isCreate) {
        return;
      }

      // model relations

      if (model && issueDomainsIG?.items && issueDomainsIG.items.length > 0) {
        formGroup.controls.domains.setValue(this.getEntities(model.domainIds, issueDomainsIG));
      }

      if (model && responsibleIG?.items && responsibleIG.items.length > 0) {
        formGroup.controls.users.setValue(this.getEntities(model.userIds, responsibleIG));
      }

      if (model && model.projectId && projectsIG?.items && projectsIG.items.length > 0) {
        formGroup.controls.project.setValue(this.getEntity(model.projectId, projectsIG.items) ?? null);
      }

      if (model && model.responsibleId && responsibleIG?.items && responsibleIG.items.length > 0) {
        formGroup.controls.responsible.setValue(this.getEntity(model.responsibleId, responsibleIG.items) ?? null);
      }

    };
  }

  protected getEntities(ids: number[], selectItemGroup: SelectItemGroup): EntityModel[] {
    return ids
      .map((id) => this.getEntity(id, selectItemGroup.items))
      .filter((entity) => !!entity) as EntityModel[];
  }

  protected getEntity(id: number, items: SelectItem<EntityModel>[]): EntityModel | undefined {
    return items.find((item) => item.value.id === id)?.value;
  }
}
