import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';

import {BaseAdapter} from 'utils/shared-services';
import {IssueModel, NewIssueModel} from 'data-access/sdk/internal-api';

import {IssueFormModel} from '../interfaces/issue-form.model';

@Injectable({
  providedIn: 'root',
})
export class IssueFormAdapterService extends BaseAdapter<IssueFormModel, Partial<IssueModel>> {
  convert(issue: IssueModel): Observable<IssueFormModel> {
    const model: IssueFormModel = {
      id: issue.id,
      createdAt: issue.createdAt,
      description: issue.description,
      name: issue.name,
      severity: issue.severity,
      resolveDate: issue.resolveDate,

      // relation ids
      domainIds: issue.domainIds,
      userIds: issue.userIds,
      projectId: issue.projectId,
      responsibleId: issue.responsibleId,
    };

    return this.createConvertObservable(model);
  }

  reverse(model: IssueFormModel): Partial<IssueModel> {
    return {
      id: model.id,
      createdAt: model.createdAt,
      description: model.description,
      name: model.name,
      severity: model.severity,
      resolveDate: model.resolveDate,

      // relation ids
      domainIds: model.domainIds,
      userIds: model.userIds,
      projectId: model.projectId,
      responsibleId: model.responsibleId,
    };
  }

  reverseToNewModel(model: IssueFormModel): Partial<NewIssueModel> {
    return {
      description: model.description,
      name: model.name,
      severity: model.severity,
      resolveDate: model.resolveDate,

      // relation ids
      domainIds: model.domainIds,
      userIds: model.userIds,
      projectId: model.projectId,
      responsibleId: model.responsibleId,
    };
  }
}
