import {ChangeDetectionStrategy, Component, ViewChild} from '@angular/core';

import {INotificationMessage} from '../../interfaces/notification-message.interface';
import {Toast} from 'primeng/toast';
import {Store} from '@ngxs/store';
import {NotificationsActions} from 'data-access/store';

@Component({
  selector: 'lib-notification-toast',
  templateUrl: './notification-toast.component.html',
  styleUrls: ['./notification-toast.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class NotificationToastComponent {
  @ViewChild('notificationToast') notificationToast!: Toast;

  constructor(
    private readonly store: Store,
  ) {
  }

  show(message: INotificationMessage, event: MouseEvent) {
    this.closeToast(message);

    event.preventDefault();

    if (message.data) {
      this.store.dispatch(new NotificationsActions.ShowEntity(message.data));
    } else {
      console.error('Message data not found');
      // TODO #errorService
    }
  }

  closeToast(message: INotificationMessage, event?: MouseEvent) {
    if (event) {
      event.preventDefault();
      event.stopPropagation();
    }

    // close toast
    this.notificationToast.messages = this.notificationToast.messages?.filter(m => m.id !== message.id);
    this.notificationToast.onClose.emit({message});

    this.store.dispatch(new NotificationsActions.MarkAsRead(message.id))
      .subscribe();
  }
}
