import {ChangeDetectionStrategy, Component, EventEmitter, Input, Output} from '@angular/core';

import {NotificationModel} from 'data-access/sdk/internal-api';

@Component({
  selector: 'lib-notification',
  templateUrl: './notification.component.html',
  styleUrls: ['./notification.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class NotificationComponent {
  @Input() notification!: NotificationModel;

  @Output() read = new EventEmitter<void>();
  @Output() show = new EventEmitter<void>();

  getSeverityIcon(severity: NotificationModel['severity'] = this.notification.severity) {
    switch (severity) {
      case 'info':
        return 'pi pi-info-circle';
      case 'success':
        return 'pi pi-check';
      case 'warn':
        return 'pi pi-exclamation-triangle';
      case 'error':
        return 'pi pi-times';
      default:
        return 'pi pi-info-circle';
    }
  }

  getSeverityClass(severity: NotificationModel['severity'] = this.notification.severity) {
    switch (severity) {
      case 'info':
        return 'p-message-info';
      case 'success':
        return 'p-message-success';
      case 'warn':
        return 'p-message-warn';
      case 'error':
        return 'p-message-error';
      default:
        return 'p-message-info';
    }
  }

  onRead(event: MouseEvent) {
    event.stopPropagation();
    this.read.emit();
  }

  onShow(event: MouseEvent) {
    event.preventDefault();

    this.onRead(event);
    this.show.emit();
  }
}
