import {ChangeDetectionStrategy, Component, ViewChild, ViewEncapsulation} from '@angular/core';
import {OverlayPanel} from 'primeng/overlaypanel';
import {Select, Store} from '@ngxs/store';
import {NOTIFICATION_PAGE_SIZE, NotificationsActions, NotificationsSelectors} from 'data-access/store';
import {NotificationModel} from 'data-access/sdk/internal-api';
import {Observable} from 'rxjs';

@Component({
  selector: 'lib-notifications-overlay',
  templateUrl: './notifications-overlay.component.html',
  styleUrls: ['./notifications-overlay.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  encapsulation: ViewEncapsulation.None,
})
export class NotificationsOverlayComponent {
  @Select(NotificationsSelectors.notifications)
  readonly notifications$!: Observable<NotificationModel[]>;

  @Select(NotificationsSelectors.hasUnreadNotifications)
  readonly hasUnreadNotifications$!: Observable<number>;

  @ViewChild('notificationsPanel') notificationsPanel!: OverlayPanel;

  constructor(
    private readonly store: Store,
  ) {
  }

  toggle(event: MouseEvent) {
    if (this.notificationsPanel.overlayVisible) {
      this.notificationsPanel.hide();
      return;
    }

    this.notificationsPanel.show(event);
  }

  readNotification(id: NotificationModel['id']) {
    this.store.dispatch(new NotificationsActions.MarkAsRead(id))
      .subscribe();
  }

  openNotificationEntity(notification: NotificationModel) {
    this.store.dispatch(new NotificationsActions.ShowEntity(notification))
      .subscribe(() => {
        this.notificationsPanel.hide();
      });
  }

  markAllAsRead() {
    this.store.dispatch(new NotificationsActions.MarkAllAsRead())
      .subscribe(() => {
        this.notificationsPanel.hide();
      });
  }

  loadMore() {
    const numberOfNotifications = this.store.selectSnapshot(NotificationsSelectors.notificationsCount);

    this.store.dispatch(new NotificationsActions.GetNotifications(0, numberOfNotifications + NOTIFICATION_PAGE_SIZE))
      .subscribe();
  }
}
