import {ChangeDetectionStrategy, Component, EventEmitter, Input, Output, ViewChild} from '@angular/core';
import {FormControl, FormGroup, Validators} from '@angular/forms';
import {OverlayPanel} from 'primeng/overlaypanel';
import {FileSelectEvent, FileUpload} from 'primeng/fileupload';

import {NewPluginInterface} from 'data-access/sdk/internal-api';


interface CreateFG {
  name: FormControl<NewPluginInterface['name'] | null>;
  enabled: FormControl<NewPluginInterface['isEnabled'] | null>;
  file: FormControl<NewPluginInterface['file'] | null>;
}

@Component({
  selector: 'lib-create-plugin',
  templateUrl: './create-plugin.component.html',
  styleUrls: ['./create-plugin.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CreatePluginComponent {
  @ViewChild('panel') overlayPanel!: OverlayPanel;
  @ViewChild('fileUpload') fileUpload!: FileUpload;

  @Input() loading = false;

  @Output() onCreate: EventEmitter<NewPluginInterface> = new EventEmitter();

  formGroup: FormGroup<CreateFG> = this.createFG();

  @Input() toggle(event: Event) {
    this.overlayPanel.toggle(event);
  }

  @Input() hide() {
    this.overlayPanel.hide();
    this.formGroup.reset();
  }

  selectFile(event: FileSelectEvent) {
    for (const file of event.files) {
      this.formGroup.get('file')?.setValue(file);
      this.formGroup.get('file')?.updateValueAndValidity();
      this.formGroup.markAsDirty();
    }
  }

  create() {
    this.fileUpload.upload();

    if (this.formGroup.invalid) {
      console.error('Form is invalid!', JSON.stringify(this.formGroup.value));
      return;
    }

    this.onCreate.emit({
      name: this.formGroup.value.name as NewPluginInterface['name'],
      isEnabled: this.formGroup.value.enabled as NewPluginInterface['isEnabled'],
      file: this.formGroup.value.file as NewPluginInterface['file'],
    });
  }

  private createFG(): FormGroup<CreateFG> {
    return new FormGroup({
      name: new FormControl<NewPluginInterface['name'] | null>(null, [Validators.required]),
      enabled: new FormControl<NewPluginInterface['isEnabled']>(true),
      file: new FormControl<NewPluginInterface['file'] | null>(null, [Validators.required]),
    });
  }
}
