import {ChangeDetectionStrategy, Component, EventEmitter, Input, OnInit, Output} from '@angular/core';

import {PluginModel} from 'data-access/sdk/internal-api';


interface Column {
  field: string;
  header: string;
}

@Component({
  selector: 'lib-plugins-table',
  templateUrl: './plugins-table.component.html',
  styleUrls: ['./plugins-table.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PluginsTableComponent implements OnInit {
  @Input() plugins: PluginModel[] = [];
  @Input() loading = false;
  @Input() showDelete = false;
  @Input() reorderedCols = true;
  @Input() canUpdate = false;

  @Output() onRowReorder = new EventEmitter<PluginModel[]>();
  @Output() onDeleteSelection = new EventEmitter<{ event: MouseEvent; plugin: PluginModel }>();
  @Output() onRowClick = new EventEmitter<PluginModel>();
  @Output() onChangePluginStatus = new EventEmitter<PluginModel>();

  cols: Column[] = [
    {field: 'name', header: 'LIBS.FEATURES.PLUGINS.LIST_PAGE.TABLE_COLUMNS.NAME'},
    {field: 'enabled', header: 'LIBS.FEATURES.PLUGINS.LIST_PAGE.TABLE_COLUMNS.ENABLED'},
  ];

  ngOnInit() {
    if (this.showDelete) {
      this.cols.push({field: 'delete', header: ''});
    }
  }

  rowReorder() {
    this.onRowReorder.emit(this.plugins);
  }

  rowClick(plugin: PluginModel) {
    this.onRowClick.emit(plugin);
  }

  delete(event: MouseEvent, plugin: PluginModel) {
    this.onDeleteSelection.emit({event: event, plugin: plugin});
  }

  changePluginStatus(plugin: PluginModel, event: any) {
    event.originalEvent.stopPropagation();
    plugin.isEnabled = event.checked;
    this.onChangePluginStatus.emit(plugin);
  }
}
