import {ChangeDetectionStrategy, Component, ViewChild} from '@angular/core';
import {Unsubscriber} from 'utils/shared-services';
import {Select, Store} from '@ngxs/store';
import {Observable, take} from 'rxjs';
import {Table} from 'primeng/table';

import {GlobalRulesSelectors, PluginsLoaderActions, PluginsLoaderSelectors} from 'data-access/store';
import {NewPluginInterface, PluginModel} from 'data-access/sdk/internal-api';

import {CreatePluginComponent} from '../components/plugin-create/create-plugin.component';

@Component({
  selector: 'lib-plugins-list',
  templateUrl: './plugins-facade.component.html',
  styleUrls: ['./plugins-facade.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    Unsubscriber,
  ],
})
export class PluginsFacadeComponent {
  @ViewChild('table') table!: Table;
  @ViewChild('createPluginRef') createPluginRef!: CreatePluginComponent;

  @Select(GlobalRulesSelectors.canDelete('dashboardplugin')) canDelete$!: Observable<boolean>;
  @Select(GlobalRulesSelectors.canUpdate('dashboardplugin')) canUpdate$!: Observable<boolean>;

  plugins$ = this.store.select(PluginsLoaderSelectors.plugins);
  pluginsLoading$ = this.store.select(PluginsLoaderSelectors.pluginsLoading);
  createPluginLoading$ = this.store.select(PluginsLoaderSelectors.createPluginLoading);

  constructor(
    private readonly store: Store,
  ) {
  }

  ngOnInit() {
    this.store.dispatch(new PluginsLoaderActions.ReadList()).subscribe();
  }

  openEditModal(plugin: PluginModel) {
    this.canUpdate$
      .pipe(take(1))
      .subscribe((canUpdate) => {
        if (!canUpdate) {
          return;
        }

        this.store.dispatch(new PluginsLoaderActions.OpenEditModal(plugin.id)).subscribe();
      });
  }

  changePluginStatus(plugin: PluginModel) {
    this.store.dispatch(new PluginsLoaderActions.ChangePluginStatus(plugin))
      .subscribe();
  }

  createPlugin(plugin: NewPluginInterface) {
    this.store.dispatch(new PluginsLoaderActions.Create(plugin))
      .subscribe(() => this.createPluginRef.hide());
  }

  updatePlugin(plugins: PluginModel[]) {
    this.store.dispatch(new PluginsLoaderActions.UpdateList(plugins)).subscribe();
  }
}
