import {NgModule} from '@angular/core';
import {CommonModule} from '@angular/common';
import {RouterModule} from '@angular/router';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {NgxsModule} from '@ngxs/store';
import {TranslateModule} from '@ngx-translate/core';

// primeNG
import {TableModule} from 'primeng/table';
import {ButtonModule} from 'primeng/button';
import {RippleModule} from 'primeng/ripple';
import {InputTextModule} from 'primeng/inputtext';
import {OverlayPanelModule} from 'primeng/overlaypanel';
import {CheckboxModule} from 'primeng/checkbox';
import {CardModule} from 'primeng/card';
import {FileUploadModule} from 'primeng/fileupload';
import {InputSwitchModule} from 'primeng/inputswitch';

import {UnfreezePipe} from 'utils/shared-pipes';
import {HasPermissionDirective} from 'utils/shared-directives';
import {PluginsLoaderState} from 'data-access/store';

import {pluginsLoaderRoutes} from './lib.routes';
import {CreatePluginComponent} from './components/plugin-create/create-plugin.component';
import {PluginsFacadeComponent} from './container/plugins-facade.component';
import {EditPluginModalComponent} from './components/plugin-edit-modal/edit-plugin-modal.component';
import {DeletePluginOverlayComponent} from './components/plugin-delete-overlay/delete-plugin-overlay.component';
import {PluginsTableComponent} from './components/plugins-table/plugins-table.component';


const primeNG = [
  CheckboxModule,
  CardModule,
  InputTextModule,
  OverlayPanelModule,
  TableModule,
  ButtonModule,
  RippleModule,
  FileUploadModule,
  InputSwitchModule,
];

@NgModule({
  declarations: [
    CreatePluginComponent,
    DeletePluginOverlayComponent,
    EditPluginModalComponent,
    PluginsFacadeComponent,
    PluginsTableComponent,
  ],
  imports: [
    ...primeNG,
    CommonModule,
    RouterModule.forChild(pluginsLoaderRoutes),
    ReactiveFormsModule,
    NgxsModule.forFeature([PluginsLoaderState]),
    TranslateModule,
    HasPermissionDirective,
    UnfreezePipe,
    FormsModule,
  ],
})
export class PluginsLoaderModule {
}
