import { toSignal } from '@angular/core/rxjs-interop';
import {Component, ComponentRef, OnInit, Signal, ViewChild} from '@angular/core';
import {Select, Store} from '@ngxs/store';
import {Observable} from 'rxjs';

import {WidgetContainerDirective} from 'utils/shared-directives';
import {OutletsEnum} from 'utils/core-project-routing-utils';
import {RightSidebarActions, RightSidebarSelectors, SidebarComponentElementModel, SidebarHeaderActionModel} from 'data-access/store';

import {SidepanelMenuItem} from '../../interfaces/sidepanel-menu.interface';

@Component({
  selector: 'app-sidepanel',
  templateUrl: './sidepanel.component.html',
  styleUrls: ['./sidepanel.component.scss'],
})
export class SidepanelComponent implements OnInit {
  @ViewChild(WidgetContainerDirective, {static: true}) contentContainer!: WidgetContainerDirective;

  @Select(RightSidebarSelectors.rightSidebarOpened) rightSidepanelOpened$!: Observable<boolean>;
  @Select(RightSidebarSelectors.header) header$!: Observable<string>;
  @Select(RightSidebarSelectors.showDefaultHeader) showDefaultHeader$!: Observable<boolean>;
  @Select(RightSidebarSelectors.sidebarHeaderActions) sidebarHeaderActions$!: Observable<SidebarHeaderActionModel[]>;
  @Select(RightSidebarSelectors.component) component$!: Observable<SidebarComponentElementModel>;
  
  isBlocked: Signal<boolean> = toSignal(this.store.select(RightSidebarSelectors.isBlocked), {initialValue: false});

  rightSidebarOutlet = OutletsEnum.RIGHT_SIDEBAR;

  items: SidepanelMenuItem[] = [];

  constructor(
    private readonly store: Store,
  ) {
  }

  ngOnInit() {
    let componentRef: ComponentRef<any> | undefined;

    this.component$
      .subscribe((compElModel: SidebarComponentElementModel | undefined) => {
        if (!compElModel || !compElModel.getComponentFn) {
          if (componentRef) {
            componentRef.destroy();
          }
          return;
        }

        if (componentRef) {
          componentRef.destroy();
        }

        componentRef = this.contentContainer.viewContainerRef.createComponent(compElModel.getComponentFn());

        // unfreeze
        const data: SidebarComponentElementModel['data'] = JSON.parse(JSON.stringify(compElModel)).data;

        if (data) {
          Object.keys(data).forEach((k) => {
            if (componentRef && data) {
              componentRef.setInput(k, data[k]);
            } else {
              console.error('Component or component data not found');
            }
          });
        }
      });
  }

  hide(manual = false) {
    this.store.dispatch(new RightSidebarActions.Close(manual));
  }
}
