import {Component, Input} from '@angular/core';
import {AuthService, User} from 'data-access/sdk/api2';
import {firstValueFrom} from 'rxjs';
import {OverlayPanel} from 'primeng/overlaypanel';

@Component({
  selector: 'lib-user-string',
  templateUrl: './user-string.component.html',
  styleUrls: ['./user-string.component.scss'],
})
export class UserStringComponent {
  @Input() user!: User;
  @Input() style: any;
  @Input() styleClass: string | undefined;
  @Input() nameFormat: 'short' | 'full' | 'informal' = 'short';

  loading = true;
  loadedUser: User | undefined;

  constructor(
    private readonly authService: AuthService,
  ) {
  }

  openOverlay(event: MouseEvent, panel: OverlayPanel) {
    event.stopPropagation();
    panel.toggle(event);
    if (!this.loadedUser && this.user.id) {
      firstValueFrom(this.authService.authUsersRead({id: this.user.id, include: 'telephones, department'})).then((user) => {
        this.loadedUser = user;
        this.loading = false;
      });
    }
  }
}
