import { ChangeDetectionStrategy, Component, effect, EventEmitter, Input, Output, signal, WritableSignal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { WorkStatusEnum } from 'data-access/sdk/internal-api';
import { ButtonModule } from 'primeng/button';
import { TranslateModule } from '@ngx-translate/core';

@Component({
  selector: 'lib-work-card-flow',
  standalone: true,
  imports: [CommonModule, ButtonModule, TranslateModule],
  templateUrl: './work-card-flow.component.html',
  styleUrls: ['./work-card-flow.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class WorkCardFlowComponent {
  @Output() onChangeStatus: EventEmitter<WorkStatusEnum> = new EventEmitter<WorkStatusEnum>();
  @Output() openReturnModal: EventEmitter<void> = new EventEmitter<void>();

  readonly buttons: WritableSignal<{ label: string; onClick: () => void; severity?: 'danger' }[]> = signal([]);
  readonly _hasInspector: WritableSignal<boolean> = signal(false);
  readonly _status: WritableSignal<WorkStatusEnum | null | undefined> = signal(undefined);

  readonly handlerEffect = effect(
    () => {
      this.handler({ status: this._status(), hasInspector: this._hasInspector() });
    },
    { allowSignalWrites: true }
  );

  @Input() set hasInspector(value: boolean) {
    this._hasInspector.set(value);
  }

  @Input({ required: true }) set workStatus(value: WorkStatusEnum | null | undefined) {
    this._status.set(value);
  }

  private handler({ status, hasInspector }: { status: WorkStatusEnum | null | undefined; hasInspector: boolean }) {
    switch (true) {
      case status === WorkStatusEnum.BL:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.TO_CREATED',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.CR),
          },
        ]);
        break;
      case status === WorkStatusEnum.CR:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.TO_PROGRESS',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.WP),
          },
        ]);
        break;
      case status === WorkStatusEnum.CN:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.RETURN_TO_PROGRESS',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.WP),
          },
        ]);
        break;
      case status === WorkStatusEnum.PS:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.RETURN_TO_PROGRESS',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.WP),
          },
        ]);
        break;
      case status === WorkStatusEnum.AC:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.RETURN_TO_PROGRESS',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.WP),
          },
        ]);
        break;
      case status === WorkStatusEnum.WP && hasInspector:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.TO_CHECK',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.DN),
          },
        ]);
        break;
      case status === WorkStatusEnum.WP && !hasInspector:
        this.buttons.set([
          {
            label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.TO_DONE',
            onClick: () => this.onChangeStatus.emit(WorkStatusEnum.AC),
          },
        ]);
        break;
      case status === WorkStatusEnum.DN:
        this.buttons.set([
          { label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.APPROVE', onClick: () => this.onChangeStatus.emit(WorkStatusEnum.AC) },
          { label: 'LIBS.FEATURES.WORKS.WORK_CARD_FLOW.DECLINE', severity: 'danger', onClick: () => this.openReturnModal.emit() },
        ]);
        break;
      default:
        this.buttons.set([]);
        if (status) {
          console.warn('Status not handled (not in flow):', status);
        }
    }
  }
}

