import {ChangeDetectionStrategy, Component} from '@angular/core';
import {CommonModule} from '@angular/common';
import {TranslateModule} from '@ngx-translate/core';
import {DynamicDialogConfig, DynamicDialogRef} from 'primeng/dynamicdialog';
import {ButtonModule} from 'primeng/button';
import {Store} from '@ngxs/store';
import {WorksSelectors} from 'data-access/store';
import {WorkTypeEnum} from 'data-access/sdk/internal-api';

@Component({
  selector: 'lib-work-delete-attention-modal',
  standalone: true,
  imports: [CommonModule, ButtonModule, TranslateModule],
  templateUrl: './work-delete-attention-modal.component.html',
  styleUrls: ['./work-delete-attention-modal.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class WorkDeleteAttentionModalComponent {

  headerKey: string;

  constructor(
    private readonly dynamicDialogRef: DynamicDialogRef,
    private readonly store: Store,
  ) {
    const workType = this.store.selectSnapshot(WorksSelectors.work)?.workType;

    switch (workType) {
      case WorkTypeEnum.TS:
        this.headerKey = 'LIBS.FEATURES.WORKS.WORK_DELETE_ATTENTION_MODAL.HEADER_TS';
        break;
      case WorkTypeEnum.MS:
        this.headerKey = 'LIBS.FEATURES.WORKS.WORK_DELETE_ATTENTION_MODAL.HEADER_MS';
        break;
      default:
        console.error('Work type not found', workType);
        this.headerKey = 'LIBS.FEATURES.WORKS.WORK_DELETE_ATTENTION_MODAL.HEADER_TS';
    }
  }

  close(value: boolean = true) {
    this.dynamicDialogRef.close(value);
  }
}
