import { EffectRef, Injectable, Signal, effect } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { SelectItem, SelectItemGroup } from 'primeng/api';

import { WorkStatusEnum } from 'data-access/sdk/internal-api';
import { DepartmentItemsClass, EntityModel, UserItemsClass } from 'ui/forms-ui';

import { CreateWorkBaseRelationsFormModel } from '../interfaces/create-work-base-relations-form.model';
import { WorkFormModel } from '../interfaces/work-form.model';
import { WorkFormGroupInterface } from '../interfaces/work-form-group.interface';

@Injectable({
  providedIn: 'root',
})
export class WorkCardFormService {
  constructor() {}

  createIssueFormRelationsHandler(
    formGroup: FormGroup<WorkFormGroupInterface>,
    executorOptionsSignal: Signal<SelectItemGroup[]>,
    inspectorOptionsSignal: Signal<SelectItemGroup[]>,
    model?: WorkFormModel,
    createRelations?: CreateWorkBaseRelationsFormModel
  ): Function {
    return () => {
      const isCreate = !formGroup.controls.id.value;

      const executorOptions = executorOptionsSignal();
      const inspectorIG = inspectorOptionsSignal()[0]; // only users

      // create relations
      if (isCreate && createRelations?.endBase) {
        formGroup.controls.endBase.setValue(createRelations.endBase ?? null);
      }

      if (isCreate && createRelations?.startBase) {
        formGroup.controls.startBase.setValue(createRelations.startBase ?? null);
      }

      if (isCreate && createRelations?.status && WorkStatusEnum.check(createRelations.status)) {
        formGroup.controls.status.setValue(createRelations.status);
      }

      if (isCreate && createRelations?.toDepartmentId && executorOptions.length > 0) {
        const departmentsIG = executorOptions.find((group) => group.label === DepartmentItemsClass.labelKey);

        if (departmentsIG) {
          const executor: EntityModel | undefined = departmentsIG.items.find(
            (item) => item.value.id === createRelations.toDepartmentId
          )?.value;
          formGroup.controls.executor.setValue(executor ?? null);
        }
      }

      if (isCreate && createRelations?.toUserId && executorOptions.length > 0) {
        const usersIG = executorOptions.find((group) => group.label === UserItemsClass.labelKey);

        if (usersIG) {
          const executor: EntityModel | undefined = usersIG.items.find((item) => item.value.id === createRelations.toUserId)?.value;
          formGroup.controls.executor.setValue(executor ?? null);
        }
      }

      if (isCreate && createRelations?.valUserId && inspectorIG.items && inspectorIG.items.length > 0) {
        const usersIG = inspectorIG; // bcs only one group

        if (usersIG) {
          const inspector: EntityModel | undefined = usersIG.items.find((item) => item.value.id === createRelations.valUserId)?.value;
          formGroup.controls.inspector.setValue(inspector ?? null);
        }
      }

      if (isCreate) {
        return;
      }

      // model relations

      if (
        model &&
        (model.toUserId || model?.toDepartmentId) &&
        executorOptions &&
        executorOptions.length > 0 &&
        executorOptions.some((group) => group.items.length > 0)
      ) {
        if (model.toUserId && model?.toDepartmentId) {
          console.error('Unexpected toDepartmentId and toUserId. Use only one.');
          // TODO #errorService
        }

        const usersIG = executorOptions.find((group) => group.label === UserItemsClass.labelKey);
        const departmentsIG = executorOptions.find((group) => group.label === DepartmentItemsClass.labelKey);

        let executor: EntityModel | undefined;

        if (usersIG?.items && usersIG.items.length > 0) {
          executor = usersIG.items.find((item) => item.value.id === model?.toUserId)?.value;
        }

        if (!executor && departmentsIG?.items && departmentsIG.items.length > 0) {
          executor = departmentsIG.items.find((item) => item.value.id === model?.toDepartmentId)?.value;
        }

        formGroup.controls.executor.setValue(executor ?? null);
      }

      if (model && model.valUserId && inspectorIG?.items && inspectorIG.items.length > 0) {
        formGroup.controls.inspector.setValue(this.getEntity(model.valUserId, inspectorIG.items) ?? null);
      }
    };
  }

  private getEntity(id: number, items: SelectItem<EntityModel>[]): EntityModel | undefined {
    return items.find((item) => item.value.id === id)?.value;
  }
}
