import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';

import {BaseAdapter} from 'utils/shared-services';
import {NewWorkModel, WorkModel, WorkStatusEnum, WorkTypeEnum} from 'data-access/sdk/internal-api';

import {WorkFormModel} from '../interfaces/work-form.model';

@Injectable({
  providedIn: 'root',
})
export class WorkFormAdapterService extends BaseAdapter<WorkFormModel, Partial<WorkModel>> {
  convert(work: WorkModel): Observable<WorkFormModel> {
    const model: WorkFormModel = {
      id: work.id,
      deadline: work.deadline,
      description: work.description,
      endBase: work.endBase,
      name: work.name,
      priority: work.priority ?? 0,
      startBase: work.startBase,
      status: work.status || WorkStatusEnum.CR,
      valUserId: work.valUserId,
      workType: work.workType ?? WorkTypeEnum.TS,

      // relation ids
      toDepartmentId: work.toDepartmentId,
      toUserId: work.toUserId,
    };

    return this.createConvertObservable(model);
  }

  reverse(model: WorkFormModel): Partial<WorkModel> {
    return {
      id: model.id,
      deadline: model.deadline,
      description: model.description,
      endBase: model.endBase,
      name: model.name,
      priority: model.priority,
      startBase: model.startBase,
      status: model.status,
      workType: model.workType,

      // relation ids
      toDepartmentId: model.toDepartmentId,
      toUserId: model.toUserId,
      valUserId: model.valUserId,
    };
  }

  reverseToNewModel(model: WorkFormModel): Partial<NewWorkModel> {
    return {
      deadline: model.deadline,
      description: model.description,
      endBase: model.endBase,
      name: model.name,
      priority: model.priority,
      startBase: model.startBase,
      status: model.status,
      workType: model.workType,

      // relation ids
      toDepartmentId: model.toDepartmentId,
      toUserId: model.toUserId,
      valUserId: model.valUserId,
    };
  }
}
