import {ChangeDetectionStrategy, Component, computed, EventEmitter, Input, Output, Signal, signal, WritableSignal} from '@angular/core';
import {CommonModule} from '@angular/common';
import {FormsModule} from '@angular/forms';
import {TranslateModule} from '@ngx-translate/core';
import {InputTextModule} from 'primeng/inputtext';

import {CommentModel} from 'data-access/sdk/internal-api';

import {CommentComponent} from '../comment/comment.component';

@Component({
  selector: 'lib-comments-tab',
  templateUrl: './comments-tab.component.html',
  styleUrls: ['./comments-tab.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  standalone: true,
  imports: [CommonModule, CommentComponent, TranslateModule, InputTextModule, FormsModule],
})
export class CommentsTabComponent {
  @Input({required: true}) comments: CommentModel[] = [];
  @Input() commentAvatarLink: string | undefined;
  @Input() readonly = false;

  @Output() onCreateComment: EventEmitter<string> = new EventEmitter<string>();

  commentInput?: string;

  readonly placeholderKey: string = 'LIBS.FEATURES.ISSUES.ISSUE_COMMENTS.ADD_COMMENT';
  readonly sendingKey: string = 'LIBS.FEATURES.ISSUES.ISSUE_COMMENTS.SENDING_COMMENT';

  readonly isLoading: WritableSignal<boolean> = signal<boolean>(false);
  readonly placeholder: Signal<string> = computed<string>(() => this.isLoading() ? this.sendingKey : this.placeholderKey);

  @Input() set loading(value: boolean) {
    this.isLoading.set(value);
  }

  createComment() {
    const comment = this.commentInput;

    if (!comment || comment.trim().length === 0) {
      return;
    }

    this.onCreateComment.emit(comment);
    this.commentInput = '';
  }
}
