import {ChangeDetectionStrategy, Component, EventEmitter, Input, Output} from '@angular/core';
import {CommonModule} from '@angular/common';
import {TranslateModule} from '@ngx-translate/core';

import {LinkModel} from 'data-access/sdk/internal-api';

@Component({
  selector: 'lib-link',
  templateUrl: './link.component.html',
  styleUrls: ['./link.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  standalone: true,
  imports: [CommonModule, TranslateModule],
})
export class LinkComponent {
  @Input({required: true}) link!: LinkModel;

  @Output() onSelect: EventEmitter<void> = new EventEmitter();

  get descriptionKey(): string {
    if (this.link.isParent && this.link.type === 'work') {
      return 'LIBS.UI.FORMS_UI.LINKS.PARENT_WORK';
    }

    if (this.link.isParent && this.link.type === 'issue') {
      return 'LIBS.UI.FORMS_UI.LINKS.PARENT_ISSUE';
    }

    if (this.link.type === 'work') {
      return 'LIBS.UI.FORMS_UI.LINKS.CHILD_WORK';
    }

    if (this.link.type === 'issue') {
      return 'LIBS.UI.FORMS_UI.LINKS.CHILD_ISSUE';
    }

    console.error('Unknown link type in link component', this.link);
    return '';
  }
}
