import {
  booleanAttribute,
  ChangeDetectionStrategy,
  Component,
  computed,
  EventEmitter,
  Input,
  Output,
  Signal,
  signal,
  WritableSignal,
} from '@angular/core';
import {CommonModule} from '@angular/common';
import {TranslateModule} from '@ngx-translate/core';
import {ButtonModule} from 'primeng/button';

import {LinkModel} from 'data-access/sdk/internal-api';

import {LinkComponent} from '../link/link.component';

@Component({
  selector: 'lib-links',
  templateUrl: './links.component.html',
  styleUrls: ['./links.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  standalone: true,
  imports: [CommonModule, LinkComponent, ButtonModule, TranslateModule],
})
export class LinksComponent {
  @Input() isLoading = false;
  @Input({required: true}) header!: string;
  @Input() readonly = false;

  @Input({transform: booleanAttribute}) showCreateIssue = false;
  @Input({transform: booleanAttribute}) showCreateWork = false;

  @Output() issueCreate: EventEmitter<void> = new EventEmitter<void>();
  @Output() openLink: EventEmitter<LinkModel> = new EventEmitter<LinkModel>();
  @Output() workCreate: EventEmitter<void> = new EventEmitter<void>();

  _links: WritableSignal<LinkModel[]> = signal<LinkModel[]>([]);
  showHeader: Signal<boolean> = computed<boolean>(() => this._links().length > 0 || this.showCreateWork || this.showCreateIssue);

  @Input({required: true}) set links(links: LinkModel[]) {
    this._links.set(links ?? []);
  };
}
