import {
  AfterViewInit,
  booleanAttribute,
  ChangeDetectionStrategy,
  Component,
  ElementRef,
  EventEmitter,
  Input,
  Output,
  signal,
  ViewChild,
  WritableSignal,
} from '@angular/core';
import {CommonModule} from '@angular/common';
import {FormsModule} from '@angular/forms';
import {TranslateModule} from '@ngx-translate/core';
import {RippleModule} from 'primeng/ripple';
import {TooltipModule} from 'primeng/tooltip';

import {SidebarHeaderActionModel} from 'data-access/store';
import {CIconComponent} from 'ui/custom-icons';
import {TextFieldModule} from '@angular/cdk/text-field';

@Component({
  selector: 'lib-sidebar-header',
  standalone: true,
  imports: [CommonModule, RippleModule, TranslateModule, TooltipModule, FormsModule, CIconComponent, TextFieldModule],
  templateUrl: './sidebar-header.component.html',
  styleUrls: ['./sidebar-header.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SidebarHeaderComponent implements AfterViewInit {
  @ViewChild('inputEl') inputEl?: ElementRef<HTMLElement>;

  @Input() header: string | undefined;
  @Input({required: true}) sidebarHeaderActions: SidebarHeaderActionModel[] = [];

  @Input({transform: booleanAttribute}) editableInput = false;
  @Input() inputPlaceholder = '';
  @Input() headerIcon?: string;

  @Output() inputDeactivate: EventEmitter<string | undefined> = new EventEmitter();
  @Output() onClose: EventEmitter<void> = new EventEmitter();

  _headerInput: string | undefined = undefined;
  _autoFocusState: WritableSignal<boolean> = signal(false);

  @Input({transform: booleanAttribute}) set autofocus(value: boolean) {
    // autofocus at input change
    if (value) {
      this.inputEl?.nativeElement.focus();
      this._autoFocusState.set(true);
    }
  };

  @Input() set headerInput(value: string | undefined) {
    this._headerInput = value;
  }

  ngAfterViewInit() {
    // autofocus at init
    setTimeout(() => {
      if (this._autoFocusState()) {
        this.inputEl?.nativeElement.focus();
      }
    });
  }
}
