import {Component, EventEmitter, Input, Output} from '@angular/core';
import {ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR} from '@angular/forms';

import {EditorModule, EditorSelectionChangeEvent} from 'primeng/editor';

@Component({
  selector: 'lib-editor',
  templateUrl: './editor.component.html',
  styleUrls: ['./editor.component.scss'],
  standalone: true,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: EditorComponent,
    },
  ],
  imports: [
    EditorModule,
    FormsModule,
  ],
})
export class EditorComponent implements ControlValueAccessor {
  @Input() isLoading = false;
  @Input() readonly = false;
  @Input() placeholder = '';
  @Input() style: any;
  @Input() styleClass: string | undefined;

  @Output() onDeactivate: EventEmitter<void> = new EventEmitter();

  defaultStyle = {height: '100px'};

  value: string | undefined;

  touched = false;
  disabled = false;


  onTouched: Function = () => {
  };

  onChanged: Function = () => {
  };

  valueChanged() {
    if (!this.readonly && !this.disabled) {
      this.markAsTouched();
      this.onChanged(this.value);
    }
  }

  writeValue(val: string | undefined) {
    this.value = val;
  }

  registerOnChange(fn: Function): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: Function): void {
    this.onTouched = fn;
  }

  markAsTouched() {
    if (!this.touched) {
      this.onTouched();
      this.touched = true;
    }
  }

  setDisabledState(disabled: boolean) {
    this.disabled = disabled;
  }

  selectionChange(event: EditorSelectionChangeEvent) {
    if (event.range === null && event.oldRange !== null) {
      this.onDeactivate.emit();
    } else if (event.range !== null && event.oldRange === null) {
    }
  }
}
