import {EntityEnum, EntityModel} from '../interfaces/user-inplace-input.interface';

import {SelectItem} from 'primeng/api/selectitem';
import {SelectItemGroup} from 'primeng/api';

export abstract class BaseSelectItemsClass<M extends { id: number; name?: string } = { id: number; name?: string }> {
  protected constructor(
    readonly entity: EntityEnum,
    readonly label: string,
    private readonly models: M[],
  ) {
  }

  abstract getName(...args: any): string | undefined;

  abstract getAvatarLink(...args: any): string | undefined;

  setItemGroup(options: SelectItemGroup[]) {
    const entityIndex = options.findIndex((item) => item.label === this.label);

    if (entityIndex === -1) {
      options.push({label: this.label, items: this.models.map((model) => this.mapSelectItem(model))});
    } else {
      options[entityIndex].items = this.models.map((model) => this.mapSelectItem(model));
    }

    return options;
  }

  protected mapSelectItem(model: M): SelectItem<EntityModel> {
    const avatarLink = this.getAvatarLink ? this.getAvatarLink(model) : undefined;
    const name = (this.getName ? this.getName(model) : undefined) ?? this.getUnknownString();

    return {
      label: name,
      value: {
        id: model.id,
        name,
        avatarLink,
        entity: this.entity,
      },
    };
  }

  protected getUnknownString(): string {
    console.error('DEV ERROR: Unknown values! Need debug check!', this);
    // TODO: #error
    return '??';
  }
}
