import {UserBaseModel} from 'data-access/sdk/internal-api';

import {EntityEnum} from '../interfaces/user-inplace-input.interface';
import {BaseSelectItemsClass} from './base-select-items.class';

export class UserItemsClass extends BaseSelectItemsClass<UserBaseModel> {
  static labelKey = 'LIBS.UI.FORMS_UI.ENTITIES.USERS';

  constructor(
    models: UserBaseModel[],
    label: string = UserItemsClass.labelKey,
    private nameFormat: 'short' | 'full' | 'informal' = 'short',
  ) {
    super(
      EntityEnum.User,
      label,
      models,
    );
  }

  getAvatarLink(model: UserBaseModel) {
    return model.avatar;
  };
  getName(model: UserBaseModel) {
    return this.mapUserNameForm(model);
  };

  private mapUserNameForm(user: UserBaseModel, nameFormat = this.nameFormat) {
    switch (nameFormat) {
      case 'short':
        return user.shortName;
      case 'full':
        return user.fullName;
      case 'informal':
        return `${user.firstName} ${user.lastName}`;
      default:
        return user.shortName;
    }
  }
}
