import {CommonModule} from '@angular/common';
import {booleanAttribute, ChangeDetectionStrategy, Component, EventEmitter, Input, Output} from '@angular/core';

@Component({
  selector: 'lib-avatar-string',
  templateUrl: './avatar-string.component.html',
  styleUrls: ['./avatar-string.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  standalone: true,
  imports: [CommonModule],
})
export class AvatarStringComponent {
  @Input() avatarLink: string | undefined;
  @Input() nameColor: string | undefined;
  @Input({required: true}) name!: string;
  @Input({transform: booleanAttribute}) addIcon = false;
  @Input({transform: booleanAttribute}) clearIcon = false;
  @Input({transform: booleanAttribute}) pointerOnlyActions = false;
  @Input({transform: booleanAttribute}) readonly = false;
  @Input({transform: booleanAttribute}) showAvatar = true;

  @Output() onAddClick: EventEmitter<void> = new EventEmitter<void>();
  @Output() onAvatarClick: EventEmitter<void> = new EventEmitter<void>();
  @Output() onClearClick: EventEmitter<void> = new EventEmitter<void>();
  @Output() onStringClick: EventEmitter<void> = new EventEmitter<void>();

  clearClick() {
    if (this.readonly) {
      return;
    }

    this.onClearClick.emit();
  }

  addClick() {
    if (this.readonly) {
      return;
    }

    this.onAddClick.emit();
  }

  stringClick() {
    if (this.readonly) {
      return;
    }

    this.onStringClick.emit();
  }

  avatarClick() {
    if (this.readonly) {
      return;
    }

    this.onAvatarClick.emit();
  }
}
