import {
  booleanAttribute,
  ChangeDetectorRef,
  Directive,
  EventEmitter,
  Input,
  Output,
  signal,
  ViewChild,
  WritableSignal,
} from '@angular/core';
import { ControlValueAccessor } from '@angular/forms';
import { DropdownChangeEvent } from 'primeng/dropdown';
import { SelectItemGroup } from 'primeng/api';

import { Unsubscriber } from 'utils/shared-services';

import { EntityModel } from '../interfaces/user-inplace-input.interface';
import { InplaceBase } from '../../inplace/inplace-base.class';
import { InplaceComponent } from '../../inplace/inplace.component';

@Directive()
export abstract class BaseEntityInplaceInputComponent<V> extends InplaceBase implements ControlValueAccessor {
  protected onTouched!: Function;
  protected onChanged!: Function;

  @ViewChild('inplaceEditorComponent') inplaceEditorComponent?: InplaceComponent;

  @Input() options: SelectItemGroup[] = []; // here SelectItemGroup! remember about item.value[propName]
  @Input() nameFormat: 'short' | 'full' | 'informal' = 'short';

  @Output() onChangeValue: EventEmitter<V> = new EventEmitter();
  @Output() onClearClick: EventEmitter<EntityModel> = new EventEmitter<EntityModel>();

  value: V | undefined;

  readonly isDisabled: WritableSignal<boolean> = signal(false);

  constructor(protected readonly changeDetectorRef: ChangeDetectorRef, unsubscriber: Unsubscriber) {
    super(unsubscriber);
  }

  abstract onChange(event: DropdownChangeEvent): unknown;

  abstract writeValue(value: V | undefined): void;

  abstract selectValue(value: V | undefined): void;

  registerOnChange(fn: any): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouched = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.isDisabled.set(isDisabled);
  }

  activateInplace() {
    if (this.readonly) {
      return;
    }

    this.inplaceEditorComponent?.manualActivate();
  }

  deactivateInplace() {
    this.inplaceEditorComponent?.deactivate();
  }
}
