import {ChangeDetectionStrategy, Component, forwardRef} from '@angular/core';
import {CommonModule} from '@angular/common';
import {FormsModule, NG_VALUE_ACCESSOR} from '@angular/forms';
import {DropdownChangeEvent, DropdownModule} from 'primeng/dropdown';
import {MultiSelectModule} from 'primeng/multiselect';

import {InplaceComponent} from '../../../inplace/inplace.component';
import {AvatarStringComponent} from '../avatar-string/avatar-string.component';
import {BaseEntityInplaceInputComponent} from '../base-entity-inplace-input.component';
import {EntityModel} from '../../interfaces/user-inplace-input.interface';
import {TranslateModule} from '@ngx-translate/core';


@Component({
  selector: 'lib-multi-entity-inplace-input',
  standalone: true,
  imports: [CommonModule, InplaceComponent, AvatarStringComponent, DropdownModule, FormsModule, MultiSelectModule, TranslateModule],
  templateUrl: './multi-entity-inplace-input.component.html',
  styleUrls: ['./multi-entity-inplace-input.component.scss'],
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => MultiEntityInplaceInputComponent),
      multi: true,
    },
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class MultiEntityInplaceInputComponent extends BaseEntityInplaceInputComponent<EntityModel[]> {
  writeValue(value: EntityModel[] | undefined): void {
    if (!value) {
      this.value = undefined;
      this.changeDetectorRef.markForCheck();
      return;
    }

    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  onChange(event: DropdownChangeEvent) {
    if (!event.value || event.value.length === 0) {
      this.selectValue(undefined);
      return;
    }

    const value: EntityModel[] = event.value;

    this.selectValue(value);
  }

  selectValue(value: EntityModel[] | undefined) {
    if (this.onTouched) {
      this.onTouched();
    }
    this.value = value;

    this.onChanged(value);
    this.onChangeValue.emit(value);
  }

  clearClick(value: EntityModel) {
    const newValue = this.value?.filter((item) => item.id !== value.id);
    this.selectValue(newValue);

    this.onClearClick.emit(value);
  }
}
