import {ChangeDetectionStrategy, Component, forwardRef} from '@angular/core';
import {CommonModule} from '@angular/common';
import {FormsModule, NG_VALUE_ACCESSOR} from '@angular/forms';
import {TranslateModule} from '@ngx-translate/core';
import {DropdownChangeEvent, DropdownModule} from 'primeng/dropdown';

import {InplaceComponent} from '../../../inplace/inplace.component';
import {AvatarStringComponent} from '../avatar-string/avatar-string.component';
import {BaseEntityInplaceInputComponent} from '../base-entity-inplace-input.component';
import {EntityModel} from '../../interfaces/user-inplace-input.interface';


@Component({
  selector: 'lib-single-entity-inplace-input',
  standalone: true,
  imports: [CommonModule, InplaceComponent, AvatarStringComponent, DropdownModule, FormsModule, TranslateModule],
  templateUrl: './single-entity-inplace-input.component.html',
  styleUrls: ['./single-entity-inplace-input.component.scss'],
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => SingleEntityInplaceInputComponent),
      multi: true,
    },
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SingleEntityInplaceInputComponent extends BaseEntityInplaceInputComponent<EntityModel> {
  override onChange(event: DropdownChangeEvent) {
    if (!event.value) {
      this.selectValue(undefined);
      return;
    }

    const value: EntityModel = event.value;

    this.selectValue(value);
  }

  override selectValue(value: EntityModel | undefined) {
    if (this.onTouched) {
      this.onTouched();
    }
    this.value = value;

    this.onChanged(value);

    this.onChangeValue.emit(value);
  }

  writeValue(value: EntityModel | undefined): void {
    if (!value) {
      this.value = undefined;
      this.changeDetectorRef.markForCheck();
      return;
    }

    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  clearClick(value: EntityModel) {
    this.selectValue(undefined);

    this.onClearClick.emit(value);
  }
}
