import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input, signal, WritableSignal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';

import { Unsubscriber } from 'utils/shared-services';

import { InplaceComponent } from '../inplace/inplace.component';
import { IssueFireComponent } from './issue-fire/issue-fire.component';
import { InplaceBase } from '../inplace/inplace-base.class';

@Component({
  selector: 'lib-issue-rang',
  standalone: true,
  imports: [CommonModule, InplaceComponent, IssueFireComponent, FormsModule, TranslateModule],
  templateUrl: './inplace-issue-rang.component.html',
  styleUrls: ['./inplace-issue-rang.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: InplaceIssueRangComponent,
    },
  ],
})
export class InplaceIssueRangComponent extends InplaceBase implements ControlValueAccessor {
  @Input() override svgIcon = 'problems';
  @Input() addText = true;

  value: number | null | undefined;

  readonly isDisabled: WritableSignal<boolean> = signal(false);

  constructor(private readonly changeDetectorRef: ChangeDetectorRef, unsubscriber: Unsubscriber) {
    super(unsubscriber);
  }

  get issueText(): string {
    switch (this.value) {
      case 0:
        return 'LIBS.UI.FORMS_UI.ISSUE_RANG.LOW';
      case 1:
        return 'LIBS.UI.FORMS_UI.ISSUE_RANG.MEDIUM';
      case 2:
        return 'LIBS.UI.FORMS_UI.ISSUE_RANG.HIGH';
      case 3:
        return 'LIBS.UI.FORMS_UI.ISSUE_RANG.CRITICAL';
      default:
        console.error('Unexpected issue value: ', this.value);
        // TODO: #error
        return '/LIBS.UI.FORMS_UI.ISSUE_RANG.NONE';
    }
  }

  onTouched: Function = () => {};

  onChanged: Function = () => {};

  writeValue(value: number | null | undefined): void {
    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  changeValue(value: number | null | undefined) {
    if (this.onTouched) {
      this.onTouched();
    }

    this.value = value;

    this.onChanged(value);
    this.onDeactivate.emit();
  }

  registerOnChange(fn: Function): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: Function): void {
    this.onTouched = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.isDisabled.set(isDisabled);
  }
}
