import {booleanAttribute, ChangeDetectionStrategy, ChangeDetectorRef, Component, EventEmitter, Input, Output} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {CommonModule} from '@angular/common';

@Component({
  selector: 'lib-issue-fire',
  templateUrl: './issue-fire.component.html',
  styleUrls: ['./issue-fire.component.scss'],
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: IssueFireComponent,
    },
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
  standalone: true,
  imports: [CommonModule],
})
export class IssueFireComponent implements ControlValueAccessor {
  @Input({transform: booleanAttribute}) readonly = false;
  @Output() onChange: EventEmitter<number> = new EventEmitter();

  value = 0;
  values = [1, 2, 3];

  touched = false;
  disabled = false;

  constructor(
    private readonly changeDetectorRef: ChangeDetectorRef,
  ) {
  }

  onChangeFn = (value: number) => {
  };
  onTouchedFn = () => {
  };

  valueChanged(newVal: number) {
    if (!this.readonly && !this.disabled) {
      this.markAsTouched();
      if (newVal !== this.value) {
        this.value = newVal;
      } else {
        this.value = 0;
      }
      this.onChangeFn(this.value);
      this.onChange.emit(this.value);
    }
  }

  writeValue(value: number) {
    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  registerOnChange(onChange: any) {
    this.onChangeFn = onChange;
  }

  registerOnTouched(onTouched: any) {
    this.onTouchedFn = onTouched;
  }

  markAsTouched() {
    if (!this.touched) {
      this.onTouchedFn();
      this.touched = true;
    }
  }

  setDisabledState(disabled: boolean) {
    this.disabled = disabled;
  }
}
