import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  forwardRef,
  Input,
  signal,
  ViewChild,
  WritableSignal,
} from '@angular/core';
import { CommonModule } from '@angular/common';
import { ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR } from '@angular/forms';
import { InputTextModule } from 'primeng/inputtext';

import { Unsubscriber } from 'utils/shared-services';

import { InplaceComponent } from '../inplace/inplace.component';
import { InplaceBase } from '../inplace/inplace-base.class';

@Component({
  selector: 'lib-inplace-string-input',
  standalone: true,
  imports: [CommonModule, InplaceComponent, InputTextModule, FormsModule],
  templateUrl: './inplace-string-input.component.html',
  styleUrls: ['./inplace-string-input.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => InplaceStringInputComponent),
      multi: true,
    },
  ],
})
export class InplaceStringInputComponent extends InplaceBase implements ControlValueAccessor {
  @ViewChild('controlComponent') controlComponent?: ElementRef<HTMLInputElement>;

  @Input() override svgIcon = 'ci-text';

  value: string | null | undefined;

  readonly isDisabled: WritableSignal<boolean> = signal(false);

  constructor(private readonly changeDetectorRef: ChangeDetectorRef, unsubscriber: Unsubscriber) {
    super(unsubscriber);
  }

  onTouched: Function = () => {};

  onChanged: Function = () => {};

  writeValue(value: string | null | undefined): void {
    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  changeValue(value: string | null | undefined) {
    if (this.onTouched) {
      this.onTouched();
    }

    this.value = value;

    this.onChanged(value);
  }

  registerOnChange(fn: Function): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: Function): void {
    this.onTouched = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.isDisabled.set(isDisabled);
  }

  activateInput() {
    setTimeout(() => {
      this.controlComponent?.nativeElement.focus();

      this.onActivate.emit();
    });
  }
}
