import {ChangeDetectionStrategy, ChangeDetectorRef, Component, forwardRef, Input, signal, ViewChild, WritableSignal} from '@angular/core';
import {CommonModule} from '@angular/common';
import {ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR, ReactiveFormsModule} from '@angular/forms';
import {isArray} from 'lodash';
import {TooltipModule} from 'primeng/tooltip';

import {CustomCalendarComponent, UiCustomCalendarModule} from 'ui/custom-calendar';
import {CustomDatePipe} from 'utils/shared-pipes';
import { Unsubscriber } from 'utils/shared-services';

import {InplaceComponent} from '../inplace/inplace.component';
import {InplaceBase} from '../inplace/inplace-base.class';

@Component({
  selector: 'lib-inplace-switch-calendar-input',
  standalone: true,
  imports: [CommonModule, ReactiveFormsModule, UiCustomCalendarModule, InplaceComponent, FormsModule, TooltipModule],
  templateUrl: './inplace-switch-calendar-input.component.html',
  styleUrls: ['./inplace-switch-calendar-input.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => InplaceSwitchCalendarInputComponent),
      multi: true,
    },
    CustomDatePipe,
  ],
})
export class InplaceSwitchCalendarInputComponent extends InplaceBase implements ControlValueAccessor {
  @ViewChild('customCalendarComponent') customCalendarComponent?: CustomCalendarComponent;

  @Input() override svgIcon = 'ci-calendar';

  @Input() showClear = false;

  value: string | string[] | null | undefined;

  readonly isDisabled: WritableSignal<boolean> = signal(false);

  constructor(
    private readonly customDatePipe: CustomDatePipe,
    private readonly changeDetectorRef: ChangeDetectorRef,
    unsubscriber: Unsubscriber,
  ) {
    super(unsubscriber);
  }

  get displayedValue(): string {
    if (isArray(this.value)) {
      const value1 = this.customDatePipe.transform(this.value[0]);
      const value2 = this.customDatePipe.transform(this.value[1]);
      return `${value1} - ${value2}`;
    } else if (typeof this.value === 'string') {
      return this.customDatePipe.transform(this.value) || '';
    }

    return '';
  }

  writeValue(value: string | string[] | null | undefined): void {
    this.value = value;
    this.changeDetectorRef.markForCheck();
  }

  changeValue(value: string | string[] | null | undefined) {
    if (this.onTouched) {
      this.onTouched();
    }

    this.value = value;

    this.onChanged(value);
    this.onDeactivate.emit();
  }

  registerOnChange(fn: Function): void {
    this.onChanged = fn;
  }

  registerOnTouched(fn: Function): void {
    this.onTouched = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.isDisabled.set(isDisabled);
  }

  onTouched: Function = () => {
  };

  onChanged: Function = () => {
  };

  showCalendar() {
    this.customCalendarComponent?.show();
  }
}
