import { booleanAttribute, Directive, EventEmitter, Input, Output } from '@angular/core';
import { debounceTime, Subject } from 'rxjs';
import { Unsubscriber } from 'utils/shared-services';

/**
 * This class is a base class for all inplace components and
 * created for Inputs/Outputs variables unification.
 *
 * ---------------------
 * Don't place logic in this class!!
 */
@Directive({
  providers: [Unsubscriber],
})
export class InplaceBase {
  // prineng emits double then dropdown hide
  private readonly onDeactivateDebounceSubject: Subject<void> = new Subject();

  @Input() contentColor?: string;
  @Input() fixedHeight = 33;
  @Input() hasError = false;
  @Input() hasWarning = false;
  @Input() helpText = '';
  @Input() iconColor?: string;
  @Input() isAvatarShow = false;
  @Input() isLoading = false;
  @Input() label = '';
  @Input() layout: 'string' | 'inline' | 'table' | 'inline-avatar-wrap' = 'inline';
  @Input() svgIcon = 'default';
  @Input() tableLayoutLabelOffset = 3;
  @Input({ transform: booleanAttribute }) autoFocus = false;
  @Input({ transform: booleanAttribute }) hideDefaultValue = false;
  @Input({ transform: booleanAttribute }) hideIcon = false;
  @Input({ transform: booleanAttribute }) isWideField = false;
  @Input({ transform: booleanAttribute }) readonly = false;
  @Input({ transform: booleanAttribute }) showAvatar = false;

  @Output() onActivate: EventEmitter<void> = new EventEmitter();
  @Output() onDeactivate: EventEmitter<void> = new EventEmitter();

  constructor(protected readonly unsubscriber: Unsubscriber) {
    this.onDeactivateDebounceSubject.pipe(debounceTime(100)).subscribe((value) => this.onDeactivate.emit(value));
  }

  deactivateWithDebounce() {
    this.onDeactivateDebounceSubject.next();
  }
}
