import {
  AfterViewInit,
  booleanAttribute,
  ChangeDetectionStrategy,
  Component,
  computed,
  Input,
  Signal,
  signal,
  ViewChild,
  WritableSignal,
} from '@angular/core';
import {CommonModule} from '@angular/common';
import {TranslateModule} from '@ngx-translate/core';
import {SkeletonModule} from 'primeng/skeleton';
import {Inplace, InplaceModule} from 'primeng/inplace';

import { Unsubscriber } from 'utils/shared-services';

import {InplaceBase} from './inplace-base.class';

@Component({
  selector: 'lib-inplace',
  templateUrl: './inplace.component.html',
  styleUrls: ['./inplace.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
  standalone: true,
  imports: [CommonModule, SkeletonModule, InplaceModule, TranslateModule],
})
export class InplaceComponent extends InplaceBase implements AfterViewInit {
  @ViewChild('inplaceComponent') inplaceComponent!: Inplace;

  @Input({transform: booleanAttribute}) customActivateStrategy = false;
  @Input({transform: booleanAttribute}) manualAddHover = false;

  isLabelShown: WritableSignal<boolean> = signal(true);
  isLabelHidden: Signal<boolean> = computed(() => !this.isLabelShown());
  hasContent: WritableSignal<boolean> = signal(false);

  constructor(
    unsubscriber: Unsubscriber
  ) {
    super(unsubscriber);
  }

  @Input({transform: booleanAttribute}) set content(value: boolean) {
    this.hasContent.set(value);
    this.inplaceComponent?.deactivate();
  };

  ngAfterViewInit(): void {
    if (this.autoFocus) {
      this.manualActivate();
    }
  }

  deactivate() {
    this.isLabelShown.set(true);
    this.deactivateWithDebounce();
  }

  activate(): void {
    if (this.customActivateStrategy) {
      return;
    }

    this.manualActivate();
  }

  manualActivate(): void {
    if (this.readonly) {
      return;
    }

    this.isLabelShown.set(false);
    this.onActivate.emit();
  }
}
