import {
  ChangeDetectionStrategy,
  Component,
  EventEmitter,
  Input,
  Output,
  Signal,
  WritableSignal,
  booleanAttribute,
  signal,
} from '@angular/core';
import { CommonModule } from '@angular/common';
import { CdkDragDrop, DragDropModule, moveItemInArray, transferArrayItem } from '@angular/cdk/drag-drop';
import { TranslateModule } from '@ngx-translate/core';

import { WorkModel } from 'data-access/sdk/internal-api';
import { CIconComponent } from 'ui/custom-icons';

import { KanbanCardInterface } from '../../interfaces/kanban-card.interface';
import { KanbanCardComponent } from '../kanban-card/kanban-card.component';
import { KanbanChangeInterface } from '../../interfaces/kanban-change.interface';
import { KanbanColumnInterface } from '../../interfaces/kanban-column.interface';
import { KanbanColumnsService } from '../../services/kanban-columns.service';

@Component({
  selector: 'ui-kanban-board',
  standalone: true,
  imports: [CommonModule, DragDropModule, TranslateModule, CIconComponent, KanbanCardComponent],
  templateUrl: './kanban-board.component.html',
  styleUrls: ['./kanban-board.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class KanbanBoardComponent {
  private readonly _columns: WritableSignal<KanbanColumnInterface[]> = signal([]);

  @Input({ transform: booleanAttribute }) readonly = false;

  @Output() changeCardPosition: EventEmitter<KanbanChangeInterface> = new EventEmitter();
  @Output() openCard: EventEmitter<KanbanCardInterface> = new EventEmitter();

  readonly columns: Signal<KanbanColumnInterface[]> = this._columns.asReadonly();

  constructor(private readonly kanbanColumnsService: KanbanColumnsService) {}

  @Input({ required: true }) set works(works: WorkModel[]) {
    this.kanbanColumnsService.worksToColumns$(works).subscribe((columns) => this._columns.set(columns));
  }

  drop(event: CdkDragDrop<KanbanCardInterface[]>) {
    if (event.previousContainer === event.container) {
      moveItemInArray(event.container.data, event.previousIndex, event.currentIndex);
    } else {
      transferArrayItem(event.previousContainer.data, event.container.data, event.previousIndex, event.currentIndex);
    }

    const isPositionChanged = event.previousContainer !== event.container || event.currentIndex !== event.previousIndex;

    if (isPositionChanged) {
      this.updateCards();

      const previousCard = event.container.data[event.currentIndex - 1];
      const nextCard = event.container.data[event.currentIndex + 1];

      this.changeCardPosition.emit({
        card: event.container.data[event.currentIndex],
        currentIndex: event.currentIndex,
        nextCard,
        previousCard,
      });
    }
  }

  private updateCards() {
    this._columns.update((columns) => this.kanbanColumnsService.updateCards(columns));
  }
}
