import { Injectable } from '@angular/core';

import { KanbanCardStatusEnum } from '../interfaces/kanban-card-status.enum';
import { KanbanCardInterface } from '../interfaces/kanban-card.interface';

@Injectable({
  providedIn: 'root',
})
export class KanbanColumnSortService {
  doneColumnByPriority(a: KanbanCardInterface, b: KanbanCardInterface) {
    const aPriority = this.convertCardStatusDoneColumnPriority(a.status);
    const bPriority = this.convertCardStatusDoneColumnPriority(b.status);

    if (aPriority > bPriority) {
      return 1;
    }

    if (aPriority < bPriority) {
      return -1;
    }

    return 0;
  }

  todoColumnByPriority(a: KanbanCardInterface, b: KanbanCardInterface) {
    const aPriority = this.convertCardStatusTodoColumnPriority(a.status);
    const bPriority = this.convertCardStatusTodoColumnPriority(b.status);

    if (aPriority > bPriority) {
      return 1;
    }

    if (aPriority < bPriority) {
      return -1;
    }

    return 0;
  }

  private convertCardStatusTodoColumnPriority(status: KanbanCardStatusEnum): number {
    switch (status) {
      case KanbanCardStatusEnum.TODO:
        return 1;
      case KanbanCardStatusEnum.PAUSED:
        return 2;
      default:
        throw new Error('Unknown status ' + status);
    }
  }

  private convertCardStatusDoneColumnPriority(status: KanbanCardStatusEnum): number {
    switch (status) {
      case KanbanCardStatusEnum.DONE:
        return 1;
      case KanbanCardStatusEnum.CANCELLED:
        return 2;
      default:
        throw new Error('Unknown status ' + status);
    }
  }
}
