import { OverlayPanel } from 'primeng/overlaypanel';

export abstract class Point {
  abstract containerElement: HTMLElement;

  abstract readonly xPointCenterOffset: number;
  abstract readonly yPointCenterOffset: number;

  abstract readonly hashId: string;
  abstract readonly pointContainerId: string;
  
  abstract readonly overlayPanel?: OverlayPanel;
  abstract readonly overlayEventFn?: (point: Point) => void;
  abstract readonly overlayData?: any;
  
  private _x = 0;
  private _y = 0;
  private _zIndex = 1;
  
  overlayEventElement?: Element;

  get x() {
    return this._x - this.xPointCenterOffset;
  }

  get y() {
    return this._y - this.yPointCenterOffset;
  }

  get zIndex() {
    return this._zIndex;
  }

  abstract create(): void;

  abstract reRender(): void;

  setElementPosition(x: number, y: number, zIndex: number) {
    this.containerElement.style.position = 'absolute';
    this.containerElement.style.top = y - this.yPointCenterOffset + 'px';
    this.containerElement.style.left = x - this.xPointCenterOffset + 'px';
    this.containerElement.style.zIndex = zIndex.toString(); 

    this._x = x;
    this._y = y;
    this._zIndex = zIndex;

    this.overlayEventElement = this.createOverlayEventElement(this.xPointCenterOffset, this.yPointCenterOffset);
    this.containerElement.appendChild(this.overlayEventElement);

    if (this.overlayPanel) {
      this.containerElement.style.cursor = 'pointer';
    }
  }

  protected createOverlayEventElement(offsetX: number, offsetY: number) {
    const element = document.createElement('div');

    element.style.position = 'absolute';
    element.style.left = offsetX + 'px';
    element.style.top = offsetY + 'px';

    return element;
  }
}
