import { OverlayPanel } from 'primeng/overlaypanel';

import { Point } from '../point.class';

export class Mark extends Point {
  static readonly xPointCenterOffsetStatic = 4;

  containerElement: HTMLElement = document.createElement('div');
  xPointCenterOffset = Mark.xPointCenterOffsetStatic;
  yPointCenterOffset = 13.5;

  readonly pointBgColor = 'white';
  override pointContainerId: string;

  constructor(
    readonly hashId: string,
    public pointColor: string,
    x: number,
    y: number,
    zIndex: number,
    readonly overlayPanel?: OverlayPanel,
    readonly overlayEventFn?: (point: Point) => void,
    readonly overlayData?: any
  ) {
    super();

    this.pointContainerId = 'mark-' + this.hashId;

    this.containerElement.setAttribute('id', this.pointContainerId);

    this.create();

    super.setElementPosition(x, y, zIndex);
  }

  create(): void {
    const pointElement = this.createPointElement(this.pointColor);
    this.containerElement.appendChild(pointElement);
  }

  reRender(): void {
    throw new Error('Method not implemented.');
  }

  private createPointElement(pointColor: string, pointBgColor: string = this.pointBgColor) {
    const pointElement = document.createElementNS('http://www.w3.org/2000/svg', 'svg');

    pointElement.setAttribute('width', '8');
    pointElement.setAttribute('height', '27');
    pointElement.setAttribute('viewBox', '0 0 8 27');
    pointElement.setAttribute('fill', 'none');

    const rect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');

    rect.setAttribute('width', '2');
    rect.setAttribute('height', '24');
    rect.setAttribute('rx', '1');
    rect.setAttribute('transform', 'matrix(-1 0 0 1 5 3)');
    rect.setAttribute('fill', pointColor);

    const circle1 = document.createElementNS('http://www.w3.org/2000/svg', 'circle');

    circle1.setAttribute('cx', '4');
    circle1.setAttribute('cy', '4');
    circle1.setAttribute('r', '4');
    circle1.setAttribute('fill', pointColor);

    const circle2 = document.createElementNS('http://www.w3.org/2000/svg', 'circle');

    circle2.setAttribute('cx', '4');
    circle2.setAttribute('cy', '4');
    circle2.setAttribute('r', '2');
    circle2.setAttribute('fill', pointBgColor);

    pointElement.appendChild(rect);
    pointElement.appendChild(circle1);
    pointElement.appendChild(circle2);

    return pointElement;
  }
}
