import { Component, Input, WritableSignal, signal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';
import { SelectItemGroup } from 'primeng/api';

import {
  DepartmentItemsClass,
  InplaceCalendarInputComponent,
  InplaceDropdownInputComponent,
  InplaceDropdownOptionsInterface,
  SingleEntityInplaceInputComponent,
  UserItemsClass,
} from 'ui/forms-ui';

import { PointOverlayDataInterface } from '../../interfaces/point-overlay-data.interface';


@Component({
  selector: 'ui-milestone-point-overlay',
  standalone: true,
  templateUrl: './ui-milesstone-point-overlay.component.html',
  styleUrls: ['./ui-milesstone-point-overlay.component.scss'],
  imports: [
    CommonModule,
    SingleEntityInplaceInputComponent,
    InplaceDropdownInputComponent,
    InplaceCalendarInputComponent,
    TranslateModule,
    ReactiveFormsModule,
  ],
})
export class UiMilestonePointOverlayComponent {
  executorFC: FormControl = new FormControl();
  endbaseFC: FormControl<PointOverlayDataInterface['endBase']> = new FormControl();
  statusFC: FormControl<PointOverlayDataInterface['status']> = new FormControl();
  statusOptions: InplaceDropdownOptionsInterface[] = [];
  executorOptions: WritableSignal<SelectItemGroup[]> = signal([]);
  title = '';

  @Input({ required: true }) set data(data: PointOverlayDataInterface) {
    this.title = data.milestoneName;
    this.endbaseFC.setValue(data.endBase);

    this.statusFC.setValue(data.status);
    if (data.statusOptions) {
      this.statusOptions = data.statusOptions;
    } else {
      console.warn('Status options are not defined');
    }
    
    this.setExecutor(data.toUser, data.toDepartment);
  }

  private setExecutor(toUser: PointOverlayDataInterface['toUser'], toDepartment: PointOverlayDataInterface['toDepartment']) {
    if (toUser && toDepartment) {
      throw new Error('Can not set toUser and toDepartment at the same time');
    }

    if (toUser) {
      const items = new UserItemsClass([toUser]);
      this.executorOptions.update(values => items.setItemGroup(values));
    }

    if (toDepartment) {
      const items = new DepartmentItemsClass([toDepartment]);
      this.executorOptions.update(values => items.setItemGroup(values));
    }

    const controlValue = this.executorOptions()[0]?.items[0]?.value;
    if (controlValue) {
      this.executorFC.setValue(controlValue);
    }
  }
}
