import { CommonModule } from '@angular/common';
import {
  AfterViewInit,
  ChangeDetectionStrategy,
  Component,
  ElementRef,
  Input,
  ViewChild,
  WritableSignal,
  signal,
} from '@angular/core';
import { TranslateModule } from '@ngx-translate/core';

import { ResizeObserverDirective } from 'utils/shared-directives';
import { ProjectStageModel } from 'data-access/sdk/internal-api';

import { UiMilestonesTimelineComponent } from '../ui-milestones-timeline/ui-milestones-timeline.component';

@Component({
  selector: 'ui-milestone-timeline-container',
  templateUrl: './ui-milestone-timeline-container.component.html',
  styleUrls: ['./ui-milestone-timeline-container.component.scss'],
  standalone: true,
  imports: [CommonModule, TranslateModule, ResizeObserverDirective, UiMilestonesTimelineComponent],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UiMilestoneTimelineContainerComponent implements AfterViewInit {
  private readonly gradientWidth = 20;
  private readonly offsetX: WritableSignal<number> = signal(0);

  @ViewChild('timeline') timeline!: UiMilestonesTimelineComponent;
  @ViewChild('container') container!: ElementRef<HTMLDivElement>;

  @Input({ required: true }) stages!: ProjectStageModel[];
  @Input({ required: true }) currentMarkOffset!: number;

  @Input() dayInPx = 4;
  @Input() sideOffset = this.dayInPx * 30 * 6;
  @Input() bgColor = '#fafafa';
  @Input() nowDate: string | undefined;
  @Input() disableDefaultError = false;

  constructor(private readonly elementRef: ElementRef<HTMLElement>) {}

  ngAfterViewInit() {
    this.createGradientBlocks();
  }

  nowMarkChanged(nowMarkX: number) {
    this.offsetX.set(this.currentMarkOffset - nowMarkX - this.sideOffset);
    this.container.nativeElement.scrollLeft = this.offsetX() * -1;
    this.setContainerHeight();
  }

  private setContainerHeight() {
    this.elementRef.nativeElement.style.height = this.timeline.elementRef.nativeElement.style.height;
  }

  private createGradientBlocks() {
    const leftGradientContainer = document.createElement('div');
    leftGradientContainer.classList.add('gradient-container');
    leftGradientContainer.style.position = 'absolute';
    leftGradientContainer.style.top = 0 + 'px';
    leftGradientContainer.style.bottom = 0 + 'px';
    leftGradientContainer.style.width = this.gradientWidth + 'px';
    leftGradientContainer.style.left = 0 + 'px';
    leftGradientContainer.style.background = `linear-gradient(90deg, ${this.bgColor} 0%, transparent 100%)`;
    leftGradientContainer.style.zIndex = '200';

    const rightGradientContainer = document.createElement('div');
    rightGradientContainer.classList.add('gradient-container');
    rightGradientContainer.style.position = 'absolute';
    rightGradientContainer.style.top = 0 + 'px';
    rightGradientContainer.style.bottom = 0 + 'px';
    rightGradientContainer.style.width = this.gradientWidth + 'px';
    rightGradientContainer.style.right = 0 + 'px';
    rightGradientContainer.style.background = `linear-gradient(90deg, transparent 0%, ${this.bgColor} 100%)`;
    rightGradientContainer.style.zIndex = '200';

    this.elementRef.nativeElement.appendChild(leftGradientContainer);
    this.elementRef.nativeElement.appendChild(rightGradientContainer);
  }
}
