/* eslint-disable max-len */
import { AfterViewInit, ChangeDetectionStrategy, Component, ElementRef, Input, OnInit, ViewChild } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Mark } from 'ui/milestones-timeline';

@Component({
  selector: 'ui-milestone-timeline-dotted-line',
  standalone: true,
  imports: [CommonModule],
  // eslint-disable-next-line @typescript-eslint/quotes
  template: `<div #dottedLine class="dotted-line"></div>`,
  styles: [
    `
      :host {
        display: block;
        position: absolute;
        width: 100%;
        height: 100%;
        top: 0;
        bottom: 0;
      }

      .dotted-line {
        position: absolute;
        top: 0;
        bottom: 0;
        width: 1px;
        background-image: url("data:image/svg+xml,%3csvg width='100%25' height='100%25' xmlns='http://www.w3.org/2000/svg'%3e%3crect width='100%25' height='100%25' fill='none' stroke='%23ADB1C2FF' stroke-width='4' stroke-dasharray='2%2c 12' stroke-dashoffset='0' stroke-linecap='square'/%3e%3c/svg%3e");
      }
    `,
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UiMilestoneTimelineDottedLineComponent implements AfterViewInit {
  @ViewChild('dottedLine') dottedLine!: ElementRef<HTMLElement>;
  @Input({ required: true }) currentDayInPx!: number;

  constructor(private readonly elementRef: ElementRef<HTMLElement>) {}

  ngAfterViewInit(): void {
    const markCenterOffset = Mark.xPointCenterOffsetStatic;
    this.dottedLine.nativeElement.style.left = `${this.currentDayInPx + markCenterOffset}px`;
  }
}
