import { ChangeDetectionStrategy, Component, ElementRef, Input, OnInit, ViewChild, WritableSignal, signal } from '@angular/core';
import { CommonModule } from '@angular/common';
import * as moment from 'moment';

interface MonthBlockInterface {
  name: string;
  endDate: string;
  startDate: string;
  year: number;
  width: string;
}

@Component({
  selector: 'ui-milestones-month-header',
  standalone: true,
  imports: [CommonModule],
  templateUrl: './ui-milestones-month-header.component.html',
  styleUrls: ['./ui-milestones-month-header.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UiMilestonesMonthHeaderComponent implements OnInit {
  @ViewChild('monthWrapper', { static: true }) monthWrapper!: ElementRef<HTMLElement>;

  @Input({ required: true }) currentDayInPx!: number;
  @Input() dayInPx = 4;
  @Input() currentDay = moment().diff(moment(0), 'days');

  months: WritableSignal<MonthBlockInterface[]> = signal([]);

  readonly monthOffset = 10;

  constructor() {}

  ngOnInit(): void {
    const millisecondsInDay = 24 * 60 * 60 * 1000;
    const currentDate = moment(this.currentDay * millisecondsInDay).format('YYYY-MM-DD');

    this.months.set(this.createMonths(currentDate));

    if (this.months().length < 1) {
      throw new Error('Months length is less than 1');
    }

    const monthAxisX = moment(currentDate).diff(this.months()[0].startDate, 'days') * this.dayInPx;
    const monthAxisOffset = (monthAxisX - this.currentDayInPx) * -1;
    this.monthWrapper.nativeElement.style.left = monthAxisOffset + 'px';
  }

  createMonths(currentDate: string, monthOffset: number = this.monthOffset) {
    const currentMonth = moment(currentDate);

    const months: MonthBlockInterface[] = [];
    const startMonth = currentMonth.subtract(monthOffset, 'month');
    for (let i = 0; i < monthOffset * 2; i++) {
      const monthDate = startMonth.add(1, 'month').startOf('month');
      const month = this.createMonthBlock(monthDate.format('YYYY-MM-DD'));
      months.push(month);
    }

    return months;
  }

  createMonthBlock(monthDate: string, dayInPx: number = this.dayInPx): MonthBlockInterface {
    const momentObj = moment(monthDate);
    const daysInMonth = momentObj.daysInMonth();
    const daysLineX = daysInMonth * dayInPx;

    return {
      name: momentObj.format('MMM'),
      year: momentObj.year(),
      width: daysLineX + 'px',
      startDate: momentObj.startOf('month').format('YYYY-MM-DD'),
      endDate: momentObj.endOf('month').format('YYYY-MM-DD'),
    };
  }
}
