import { Injectable, Signal, WritableSignal, signal, ChangeDetectorRef } from '@angular/core';

import { PointOverlayDataInterface } from '../interfaces/point-overlay-data.interface';
import { MILESTONE_STATUSES } from '../constants/milestone-statuses';
import { Point } from '../classes/point.class';

@Injectable({
  providedIn: 'root',
})
export class PointOverlayService {
  private readonly _activePointOverlayData: WritableSignal<PointOverlayDataInterface> = signal<PointOverlayDataInterface>(
    {} as PointOverlayDataInterface
  );
  readonly activePointOverlayData: Signal<PointOverlayDataInterface> = this._activePointOverlayData.asReadonly();

  readonly overlayStaticData: Partial<PointOverlayDataInterface> = {
    statusOptions: MILESTONE_STATUSES,
  };

  setActivePointOverlayData(data: PointOverlayDataInterface) {
    this._activePointOverlayData.set(data);
  }

  createOverlayFn(point: Point) {}

  createEvents(points: Point[], changeDetectorRef: ChangeDetectorRef) {
    points.forEach((point) => {
      if (!point.containerElement) {
        return;
      }

      point.containerElement.addEventListener('click', (event) => {
        if (point.overlayPanel && !point.overlayPanel.overlayVisible) {
          if (point.overlayEventFn) {
            point.overlayEventFn(point);
          } else {
            this.setActivePointOverlayData({...point.overlayData, ...this.overlayStaticData});
          }

          point.overlayPanel.show(event, point.overlayEventElement);
          changeDetectorRef.detectChanges();
        }
      });
    });
  }
}
