import {ElementRef, Injectable} from '@angular/core';
import {BehaviorSubject} from 'rxjs';


@Injectable()
export class PluginAttrService {
  private _inited = new BehaviorSubject<boolean>(false);
  // eslint-disable-next-line @typescript-eslint/member-ordering
  isInited$ = this._inited.asObservable();

  private _attributes: { [name: string]: string } = {};
  private elementRef!: ElementRef<HTMLElement>;


  get attributes() {
    return this._attributes;
  }

  init(el: ElementRef) {
    this.elementRef = el;

    this.getAttributes();

    this._inited.next(true);
  }

  private getAttributes() {
    try {
      this.elementRef.nativeElement.getAttributeNames()
        .forEach(name => {
          const attr = this.elementRef.nativeElement.getAttribute(name);
          if (attr) {
            this._attributes[name] = attr;
          }
        });
    } catch (e) {
      console.error(e);
    }
  }
}
