import {HTTP_INTERCEPTORS, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest} from '@angular/common/http';
import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';
import {TranslateService} from '@ngx-translate/core';

@Injectable()
export class PluginAuthInterceptor implements HttpInterceptor {
  constructor(
    private translateService: TranslateService,
  ) {
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    if (this.translateService.currentLang) {
      request = request.clone({
        setHeaders: {
          'Accept-Language': this.translateService.currentLang,
        },
      });
    }

    if (localStorage.getItem('token')) {
      request = request.clone({
        setHeaders: {
          Authorization: 'Token ' + localStorage.getItem('token'),
        },
      });
    }

    return next.handle(request);
  }
}

export const pluginAuthInterceptorProvider = {provide: HTTP_INTERCEPTORS, useClass: PluginAuthInterceptor, multi: true};
