import {TranslateLoader} from '@ngx-translate/core';
import {filter, finalize, map, Observable, of, switchMap} from 'rxjs';
import {HttpClient} from '@angular/common/http';
import {inject} from '@angular/core';
import {PluginsService} from 'data-access/sdk/api2';
import {PluginAttrService} from './plugin-attr.service';

export function pluginHttpLoaderFactory(environment: {
  [key: string]: any;
}) {
  return new PluginTranslateHttpLoader(inject(HttpClient), inject(PluginAttrService), environment, inject(PluginsService));
}

/**
 * На данный момент динамическая смена переводов в плагинах не поддерживается.
 */
export class PluginTranslateHttpLoader extends TranslateLoader {
  firstLoad = true;

  constructor(
    private readonly http: HttpClient,
    private readonly pluginAttrService: PluginAttrService,
    private readonly environment: {
      [key: string]: string | boolean | undefined;
      production?: boolean;
    },
    private readonly pluginsService: PluginsService,
  ) {
    super();
  }

  getTranslation(lang: string): Observable<any> {
    return this.pluginAttrService.isInited$
      .pipe(
        finalize(() => this.firstLoad = false),
        filter(isInited => isInited),
        switchMap(() => {
          const attrPluginId: string | undefined = this.pluginAttrService.attributes['pluginid'];
          const attrLang: string | undefined = this.pluginAttrService.attributes['lang'];

          const isProd: boolean | undefined = this.environment['production'] ?? true;
          const pluginId: number | undefined = attrPluginId ? Number(attrPluginId) : undefined;

          const loadLang: string = this.firstLoad
            ? attrLang || lang
            : lang;

          if (!pluginId) {
            console.error('Plugin id is not defined. Translation will not be loaded.');
            return of({});
          }

          return this.query(pluginId, loadLang, isProd);
        }),
      );
  }

  private query(pluginId: number, lang: string, isProd: boolean) {
    if (isProd) {
      return this.pluginsService.pluginsDashboardPluginsTranslations({id: pluginId, lang})
        .pipe(map(response => response.translations));
    }

    return this.http.get(`assets/i18n/${lang}.json`);
  }
}
