import {Directive, EffectRef, EventEmitter, Input, Output} from '@angular/core';
import {AbstractControl, FormGroup} from '@angular/forms';
import {SelectItem, SelectItemGroup} from 'primeng/api';
import {EntityModel} from 'ui/forms-ui';

@Directive()
export abstract class CardFormBaseDirective<
  F extends { [keyF in keyof F]: AbstractControl<any, any>; }, // FormGroup model
  O extends { [keyO in keyof O]: string | number | number[] | unknown }, // form model (output object)
  D extends { [keyD in keyof D]: string | number } // create dependencies
> {
  protected formRelationsHandler: Function | undefined;

  @Input() isLoading = false;
  @Input() readonly = false;

  @Output() formChanges: EventEmitter<O> = new EventEmitter<O>();

  form!: FormGroup<F>;

  abstract initForm(issue?: O, createRelations?: D): void;

  abstract onIssueFormChanges(): void;

  protected abstract createFormGroup(model?: O, createRelations?: D): FormGroup<F>;

  protected getEntities(ids: number[], selectItemGroup: SelectItemGroup): EntityModel[] {
    return ids
      .map((id) => this.getEntity(id, selectItemGroup.items))
      .filter((entity) => !!entity) as EntityModel[];
  }

  protected getEntity(id: number, items: SelectItem<EntityModel>[]): EntityModel | undefined {
    return items.find((item) => item.value.id === id)?.value;
  }
}
