import { Directive, ElementRef, EventEmitter, Output, OnDestroy } from '@angular/core';

const entriesMap = new WeakMap();

const resizeObserver = new ResizeObserver((entries) => {
  for (const entry of entries) {
    if (entriesMap.has(entry.target)) {
      const componentElement = entriesMap.get(entry.target);
      componentElement.resizeCallback(entry);
    }
  }
});

@Directive({ selector: '[resizeObserver]', standalone: true })
export class ResizeObserverDirective implements OnDestroy {
  @Output('resizeObserver')
  resizeEmitter: EventEmitter<ResizeObserverEntry> = new EventEmitter();

  constructor(private el: ElementRef) {
    const target = this.el.nativeElement;
    entriesMap.set(target, this);
    resizeObserver.observe(target);
  }

  resizeCallback(entry: ResizeObserverEntry) {
    this.resizeEmitter.emit(entry);
  }

  ngOnDestroy() {
    const target = this.el.nativeElement;
    resizeObserver.unobserve(target);
    entriesMap.delete(target);
  }
}
