const readdirSync = require('fs').readdirSync;
const remove = require('fs-extra').remove;
const concat = require('concat');
const archiver = require('archiver');
const {createWriteStream} = require('fs');
const crypto = require('crypto');
const moment = require('moment');

const widgetsPath = './dist/dashboard-widgets';
const prefix = 'dw-';

const build = async () => {
  const dirs = getDirectories(widgetsPath);

  for (const {fullDirPath, name} of dirs) {
    const jsFilesPaths = getJSFilesInDir(fullDirPath);
    const hash = crypto.createHash('md5').update(name + moment().toISOString()).digest('hex');

    await concat(jsFilesPaths, `${fullDirPath}/index.${hash}.js`);
    await Promise.all([...jsFilesPaths, `${fullDirPath}/index.html`].map((file) => remove(file)));
    await createArchive(prefix + name, fullDirPath);
  }
};

build();

function getDirectories(source) {
  return readdirSync(source, {withFileTypes: true})
    .filter((dirent) => dirent.isDirectory())
    .map((dirent) => ({
      ...dirent,
      fullDirPath: `${dirent.path}/${dirent.name}`,
    }));
}

function getJSFilesInDir(source) {
  return readdirSync(source, {withFileTypes: true})
    .filter((dirent) => !dirent.isDirectory() && dirent.name.endsWith('.js'))
    .map((dirent) => `${dirent.path}/${dirent.name}`);
}

async function createArchive(name, path) {
  const output = createWriteStream(path + `/${name}.zip`);
  const archive = archiver('zip', {
    zlib: {level: 9}, // Sets the compression level.
  });

  output.on('close', function () {
    // eslint-disable-next-line no-console
    console.info(archive.pointer() + ' total bytes');
    // eslint-disable-next-line no-console
    console.info('archiver has been finalized and the output file descriptor has closed.');
  });

  output.on('end', function () {
    // eslint-disable-next-line no-console
    console.info('Data has been drained');
  });

  archive.on('warning', function (err) {
    if (err.code === 'ENOENT') {
      console.warn(err);
    } else {
      throw err;
    }
  });

  archive.pipe(output);

  archive.on('error', function (err) {
    throw err;
  });

  archive.glob('**/*.[!zip]*', {cwd: path});

  return archive.finalize();
}
